#include "qcontextmenuevent.h"
#include "qevent.h"
#include "../Types/qpoint.h"

using namespace v8;

Persistent<Function> QContextMenuEventWrap::constructor;

QContextMenuEventWrap::QContextMenuEventWrap(const v8::Arguments& args) : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
  
  //test
  if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QEvent") {
  	QEventWrap* evt_wrap = ObjectWrap::Unwrap<QEventWrap>(args[0]->ToObject());
	  QEvent* event = evt_wrap->GetWrapped();
    q_ = static_cast<QContextMenuEvent *>(event);
  }
}

QContextMenuEventWrap::~QContextMenuEventWrap() {
  //delete q_;
}

void QContextMenuEventWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QContextMenuEvent"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	
  // Event
	ADD_METHOD(accept)
	ADD_METHOD(ignore)
	ADD_METHOD(isAccepted)
	ADD_METHOD(setAccepted)
	//ADD_METHOD(spontaneous)
	ADD_METHOD(type)
	
	// dropEvent
	ADD_METHOD(globalPos)
	ADD_METHOD(pos)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QContextMenuEvent"), constructor);
}

Handle<Value> QContextMenuEventWrap::New(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = new QContextMenuEventWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QContextMenuEventWrap::NewInstance(QContextMenuEvent *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QContextMenuEventWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QContextMenuEventWrap* w = ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}


//void 	accept ()
Handle<Value> QContextMenuEventWrap::accept(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();

	q->accept();
	
	return scope.Close(Undefined());
}

//void 	ignore ()
Handle<Value> QContextMenuEventWrap::ignore(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();

	q->ignore();
	
	return scope.Close(Undefined());
}

// isAccepted () -> bool
Handle<Value> QContextMenuEventWrap::isAccepted(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();
	
	return scope.Close(Boolean::New(q->isAccepted()));
}

// setAccepted (bool)
Handle<Value> QContextMenuEventWrap::setAccepted(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();

	q->setAccepted(args[0]->BooleanValue());
	
	return scope.Close(Undefined());
}

// type () -> int
Handle<Value> QContextMenuEventWrap::type(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();
	
	return scope.Close(Integer::New(q->type()));
}


// globalPos () -> object QPoint
Handle<Value> QContextMenuEventWrap::globalPos(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();
	
  return scope.Close( QPointWrap::NewInstance(q->globalPos()) );
}

// pos () -> object QPoint
Handle<Value> QContextMenuEventWrap::pos(const Arguments& args) {
  HandleScope scope;

  QContextMenuEventWrap* w = node::ObjectWrap::Unwrap<QContextMenuEventWrap>(args.This());
  QContextMenuEvent* q = w->GetWrapped();
	
  return scope.Close( QPointWrap::NewInstance(q->pos()) );
}

