#include "qdropevent.h"
#include "qevent.h"
#include "../Types/qpoint.h"

using namespace v8;

Persistent<Function> QDropEventWrap::constructor;

QDropEventWrap::QDropEventWrap(const v8::Arguments& args) : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
  
  //test
  if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QEvent") {
  	QEventWrap* evt_wrap = ObjectWrap::Unwrap<QEventWrap>(args[0]->ToObject());
	  QEvent* event = evt_wrap->GetWrapped();
    q_ = static_cast<QDropEvent *>(event);
  }
}

QDropEventWrap::~QDropEventWrap() {
  //delete q_;
}

void QDropEventWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDropEvent"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	
  // Event
	ADD_METHOD(accept)
	ADD_METHOD(ignore)
	ADD_METHOD(isAccepted)
	ADD_METHOD(setAccepted)
	//ADD_METHOD(spontaneous)
	ADD_METHOD(type)
	
	// dropEvent
	ADD_METHOD(acceptProposedAction)
	ADD_METHOD(mimeData)
	ADD_METHOD(pos)
	ADD_METHOD(source)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDropEvent"), constructor);
}

Handle<Value> QDropEventWrap::New(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = new QDropEventWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDropEventWrap::NewInstance(QDropEvent *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDropEventWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QDropEventWrap* w = ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}


//void 	accept ()
Handle<Value> QDropEventWrap::accept(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();

	q->accept();
	
	return scope.Close(Undefined());
}

//void 	ignore ()
Handle<Value> QDropEventWrap::ignore(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();

	q->ignore();
	
	return scope.Close(Undefined());
}

//bool 	isAccepted () const
Handle<Value> QDropEventWrap::isAccepted(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
	
	return scope.Close(Boolean::New(q->isAccepted()));
}

//void 	setAccepted ( bool accepted )
Handle<Value> QDropEventWrap::setAccepted(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();

	q->setAccepted(args[0]->BooleanValue());
	
	return scope.Close(Undefined());
}

//int 	type () const
Handle<Value> QDropEventWrap::type(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
	
	return scope.Close(Integer::New(q->type()));
}

//Handle<Value> QDropEventWrap::mimeData(const Arguments& args) {
//  HandleScope scope;
//
//  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
//  QDropEvent* q = w->GetWrapped();
//  	
//	return scope.Close(QMimeDataWrap::NewInstance(q->mimeData()));
//}

//mimeData ([bool asString]) -> object QMimeData
Handle<Value> QDropEventWrap::mimeData(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
  
  const QMimeData * m = q->mimeData();
  
  bool asString = false;
  if (args.Length() > 0) {
  	asString = args[0]->BooleanValue();
  }
  
  // convert to native JS object
	Local<Object> dataObj = v8::Object::New(); //m->formats().count()
	
	if (asString){
		// A) DATA AS STRING
		foreach(QString format, m->formats()){
			dataObj->Set(
				qt_v8::FromQString(format),
				qt_v8::FromQString(m->data(format).data())
			);
		}
	}else{
		foreach(QString format, m->formats()){
			// B) AS RAW DATA (Slow Buffer)
			QByteArray ba = m->data(format);
			node::Buffer *buf = node::Buffer::New(ba.size());
			memcpy(node::Buffer::Data(buf), ba.data(), ba.size());		
			dataObj->Set(
				qt_v8::FromQString(format),
				buf->handle_
			);
		}
	}
	
	return scope.Close(dataObj);
}

//void 	acceptProposedAction ()
Handle<Value> QDropEventWrap::acceptProposedAction(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();

	q->acceptProposedAction();
	
	return scope.Close(Undefined());
}

// pos () -> object QPoint
Handle<Value> QDropEventWrap::pos(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
	
  return scope.Close( QPointWrap::NewInstance(q->pos()) );
}

// CUSTOM: source () -> objectName | undefined
Handle<Value> QDropEventWrap::source(const Arguments& args) {
  HandleScope scope;

  QDropEventWrap* w = node::ObjectWrap::Unwrap<QDropEventWrap>(args.This());
  QDropEvent* q = w->GetWrapped();
	
	QWidget* src = q->source();
	if (src)
  	return scope.Close(qt_v8::FromQString(src->objectName()));
  else
 		return scope.Close(Undefined());
}
