#include "qevent.h"

using namespace v8;

Persistent<Function> QEventWrap::constructor;

QEventWrap::QEventWrap() : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
}

QEventWrap::~QEventWrap() {
  //delete q_;
}

void QEventWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QEvent"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods  
	ADD_METHOD(accept)
	ADD_METHOD(ignore)
	ADD_METHOD(isAccepted)
	ADD_METHOD(setAccepted)
	//ADD_METHOD(spontaneous)
	ADD_METHOD(type)
	
	ADD_METHOD(destroy)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QEvent"), constructor);
}

Handle<Value> QEventWrap::New(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = new QEventWrap();
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QEventWrap::NewInstance(QEvent *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

//void 	accept ()
Handle<Value> QEventWrap::accept(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();

	q->accept();
	
	return scope.Close(Undefined());
}

//void 	ignore ()
Handle<Value> QEventWrap::ignore(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();

	q->ignore();
	
	return scope.Close(Undefined());
}

//bool 	isAccepted () const
Handle<Value> QEventWrap::isAccepted(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();
	
	return scope.Close(Boolean::New(q->isAccepted()));
}

//void 	setAccepted ( bool accepted )
Handle<Value> QEventWrap::setAccepted(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();

	q->setAccepted(args[0]->BooleanValue());
	
	return scope.Close(Undefined());
}

//int 	type () const
Handle<Value> QEventWrap::type(const Arguments& args) {
  HandleScope scope;

  QEventWrap* w = node::ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();
	
	return scope.Close(Integer::New(q->type()));
}

Handle<Value> QEventWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QEventWrap* w = ObjectWrap::Unwrap<QEventWrap>(args.This());
  QEvent* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}