
#include "qmouseevent.h"
#include "qevent.h"
#include "../Types/qpoint.h"

using namespace v8;

Persistent<Function> QMouseEventWrap::constructor;

QMouseEventWrap::QMouseEventWrap(const v8::Arguments& args) : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
  
  //test
  if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QEvent") {
  	QEventWrap* evt_wrap = ObjectWrap::Unwrap<QEventWrap>(args[0]->ToObject());
	  QEvent* event = evt_wrap->GetWrapped();
    q_ = static_cast<QMouseEvent *>(event);
  }else if(args.Length()==5){
  	
  	QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[1]->ToObject());
  	QPoint* pos = point_wrap->GetWrapped();
  
  	QMouseEvent *m = new QMouseEvent(
  		(QEvent::Type)args[0]->IntegerValue(),
  		*pos,
  		(Qt::MouseButton)args[2]->IntegerValue(),
  		(Qt::MouseButtons)args[3]->IntegerValue(),
  		(Qt::KeyboardModifiers)args[4]->IntegerValue()
  	);
  }else if(args.Length()==6){
  	
  	QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[1]->ToObject());
  	QPoint* pos = point_wrap->GetWrapped();
  	
  	QPointWrap* point_wrap2 = ObjectWrap::Unwrap<QPointWrap>(args[2]->ToObject());
  	QPoint* globalPos = point_wrap2->GetWrapped();
  	
  	QMouseEvent *m = new QMouseEvent(
  		(QEvent::Type)args[0]->IntegerValue(),
  		*pos,
  		*globalPos,
  		(Qt::MouseButton)args[3]->IntegerValue(),
  		(Qt::MouseButtons)args[4]->IntegerValue(),
  		(Qt::KeyboardModifiers)args[5]->IntegerValue()
  	);
  }
}

QMouseEventWrap::~QMouseEventWrap() {
  //delete q_;
}

void QMouseEventWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMouseEvent"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);
  
	ADD_METHOD(accept)
	ADD_METHOD(ignore)
	
	ADD_METHOD(x)
	ADD_METHOD(y)
	ADD_METHOD(button)
	
	ADD_METHOD(globalX)
	ADD_METHOD(globalY)
	
	ADD_METHOD(destroy)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMouseEvent"), constructor);
}

Handle<Value> QMouseEventWrap::New(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = new QMouseEventWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMouseEventWrap::NewInstance(QMouseEvent *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QMouseEventWrap::x(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

  return scope.Close(Number::New(q->x()));
}

Handle<Value> QMouseEventWrap::y(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

  return scope.Close(Number::New(q->y()));
}

Handle<Value> QMouseEventWrap::button(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

  return scope.Close(Number::New(q->button()));
}

Handle<Value> QMouseEventWrap::globalX(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

  return scope.Close(Number::New(q->globalX()));
}

Handle<Value> QMouseEventWrap::globalY(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

  return scope.Close(Number::New(q->globalY()));
}

Handle<Value> QMouseEventWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QMouseEventWrap* w = ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

//void 	accept ()
Handle<Value> QMouseEventWrap::accept(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

	q->accept();
	
	return scope.Close(Undefined());
}

//void 	ignore ()
Handle<Value> QMouseEventWrap::ignore(const Arguments& args) {
  HandleScope scope;

  QMouseEventWrap* w = node::ObjectWrap::Unwrap<QMouseEventWrap>(args.This());
  QMouseEvent* q = w->GetWrapped();

	q->ignore();
	
	return scope.Close(Undefined());
}