#include "qtouchevent.h"
#include "qevent.h"
//#include "../Types/qpoint.h"

using namespace v8;

Persistent<Function> QTouchEventWrap::constructor;

QTouchEventWrap::QTouchEventWrap(const v8::Arguments& args) : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
}

QTouchEventWrap::~QTouchEventWrap() {
  //delete q_;
}

void QTouchEventWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTouchEvent"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);
  
  // Wrapped methods
	ADD_METHOD(destroy)
	
	ADD_METHOD(accept)
	ADD_METHOD(ignore)
	ADD_METHOD(isAccepted)
	ADD_METHOD(setAccepted)
	ADD_METHOD(type)
	
	ADD_METHOD(deviceType)
	ADD_METHOD(touchPoints)
	ADD_METHOD(widget)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTouchEvent"), constructor);
}

Handle<Value> QTouchEventWrap::New(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = new QTouchEventWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTouchEventWrap::NewInstance(QTouchEvent *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// destroy ()
Handle<Value> QTouchEventWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QTouchEventWrap* w = ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// accept ()
Handle<Value> QTouchEventWrap::accept(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();

	q->accept();
	
	return scope.Close(Undefined());
}

// ignore ()
Handle<Value> QTouchEventWrap::ignore(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();

	q->ignore();
	
	return scope.Close(Undefined());
}

// isAccepted () -> bool
Handle<Value> QTouchEventWrap::isAccepted(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();
	
	return scope.Close(Boolean::New(q->isAccepted()));
}

// setAccepted (bool)
Handle<Value> QTouchEventWrap::setAccepted(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();

	q->setAccepted(args[0]->BooleanValue());
	
	return scope.Close(Undefined());
}

// type () -> int
Handle<Value> QTouchEventWrap::type(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();
	
  return scope.Close(Integer::New(q->type()));
}

// deviceType () -> int
Handle<Value> QTouchEventWrap::deviceType(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();

  return scope.Close(Integer::New(q->deviceType()));
}

// touchPoints () -> array
Handle<Value> QTouchEventWrap::touchPoints(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();

	QList<QTouchEvent::TouchPoint> tp =  q->touchPoints();
		
  // create array
  Handle<Array> tpArray = v8::Array::New(tp.count());
  for (int i=0;i<tp.count();i++){  	
  	tpArray->Set(i, QTouchPointWrap::NewInstance(&tp[i]));
  }
  
  return scope.Close(tpArray);
}

// widget () -> QWidget
Handle<Value> QTouchEventWrap::widget(const Arguments& args) {
  HandleScope scope;

  QTouchEventWrap* w = node::ObjectWrap::Unwrap<QTouchEventWrap>(args.This());
  QTouchEvent* q = w->GetWrapped();
  
  return scope.Close(QWidgetWrap::NewInstance(q->widget())); 
}
