#include "quiloader.h"

using namespace v8;

Persistent<Function> QUiLoaderWrap::constructor;

// Supported implementations:
//   QUiLoader ( )
//   QUiLoader ( QString filename )
QUiLoaderWrap::QUiLoaderWrap(const Arguments& args) {
  q_ = new QUiLoader();  
}

QUiLoaderWrap::~QUiLoaderWrap() {
  //delete q_;
}

void QUiLoaderWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QUiLoader"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
  ADD_METHOD(load)
  
	ADD_METHOD(addPluginPath)
	ADD_METHOD(clearPluginPaths)
	ADD_METHOD(pluginPaths)
	ADD_METHOD(setWorkingDirectory)
	ADD_METHOD(workingDirectory)
	
	ADD_METHOD(destroy)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QUiLoader"), constructor);
}

Handle<Value> QUiLoaderWrap::New(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = new QUiLoaderWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QUiLoaderWrap::NewInstance(QUiLoader *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QUiLoaderWrap* w = node::ObjectWrap::Unwrap<QUiLoaderWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}


// load (string filename | object buffer[, parentWidget]) -> widget 
Handle<Value> QUiLoaderWrap::load(const Arguments& args) {
  HandleScope scope;
	//ASSERT_NARGS(1)
	
  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	QWidget * parentWidget = 0;
	
	if (args.Length() > 1){ // TEST ONLY: QWIDGET ALS CONTAINER ???
		QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[1]->ToObject());
		parentWidget = widget_wrap->GetWrapped();
	}
	
	QWidget * widget;
		
	if (args[0]->IsString()){

		QFile file(qt_v8::ToQString(args[0]->ToString()));
		//if (!file.exists())	RETURN_INT_AND_EXIT(0)		
		
		widget = q->load(&file, parentWidget); //, win); // hier KEIN parent (oder 0) angeben!
		return scope.Close(QWidgetWrap::NewInstance(widget));

	}else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="Buffer"){
		
		size_t size = node::Buffer::Length(args[0]->ToObject());
		char *ui = node::Buffer::Data(args[0]->ToObject());
		
		//return scope.Close(Boolean::New(q->loadFromData( QByteArray(buf, size) )));

		QBuffer qbuf;
		qbuf.setData(ui);
		//qbuf.setData(winLatinToUTF8(ui));
	
		qbuf.open(QIODevice::ReadOnly);
		widget = q->load(&qbuf, parentWidget); //, win);
		return scope.Close(QWidgetWrap::NewInstance(widget));
		
	}else{
		// throw ...
  }
  
  //return scope.Close(Boolean::New( ok ));
  return scope.Close(Undefined());
}

// addPluginPath (string path)
Handle<Value> QUiLoaderWrap::addPluginPath(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	q->addPluginPath(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// clearPluginPaths ()
Handle<Value> QUiLoaderWrap::clearPluginPaths(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	q->clearPluginPaths();
	
  return scope.Close(Undefined());
}

// pluginPaths () -> array of strings
Handle<Value> QUiLoaderWrap::pluginPaths(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	QStringList pathList = q->pluginPaths();
	Handle<Array> pathArray = v8::Array::New(pathList.count());
	for (int i=0;i<pathList.count();i++){
		pathArray->Set(i, qt_v8::FromQString(pathList[i]) );
	}
	
  return scope.Close(pathArray);
}

// setWorkingDirectory (string dir)
Handle<Value> QUiLoaderWrap::setWorkingDirectory(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	q->setWorkingDirectory(QDir(qt_v8::ToQString(args[0]->ToString())));
	
  return scope.Close(Undefined());
}

// workingDirectory () -> string
Handle<Value> QUiLoaderWrap::workingDirectory(const Arguments& args) {
  HandleScope scope;

  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();

	QDir d = q->workingDirectory();
	
  return scope.Close(qt_v8::FromQString(d.path()));
}

Handle<Value> QUiLoaderWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QUiLoaderWrap* w = ObjectWrap::Unwrap<QUiLoaderWrap>(args.This());
  QUiLoader* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}