#include "qaction.h"
#include "qobject.h"

using namespace v8;

Persistent<Function> QActionWrap::constructor;

QActionWrap::QActionWrap(const Arguments& args) : q_(NULL) {
	if (args.Length() > 1){
	  QIcon ico;
	  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
			 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
	  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
		  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
		  QImage* img = img_wrap->GetWrapped();
			QPixmap pm = QPixmap::fromImage(*img);
			ico.addPixmap(pm);
		}  
		q_ = new QAction(ico, qt_v8::ToQString(args[1]->ToString()), 0);
	}else if(args.Length() > 0){
  	q_ = new QAction(qt_v8::ToQString(args[0]->ToString()), 0);
  }else{
  	q_ = new QAction(0);
  }
}

QActionWrap::~QActionWrap() {
  //delete q_;
}

void QActionWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QAction"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(setCheckable)
	ADD_METHOD(setChecked)
	ADD_METHOD(setIcon)
	ADD_METHOD(setShortcut)
	ADD_METHOD(setStatusTip)
	ADD_METHOD(setText)
	ADD_METHOD(text)
	ADD_METHOD(setShortcuts)
	ADD_METHOD(setEnabled)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QAction"), constructor);
}

Handle<Value> QActionWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QActionWrap* w = new QActionWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QActionWrap::NewInstance(QAction *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QActionWrap* w = node::ObjectWrap::Unwrap<QActionWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QActionWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
	QAction* q = w->GetWrapped();
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="changed") 
		QObject::connect(q, SIGNAL(changed()), rec, SLOT(evtV()));	
	else if (sign=="hovered")
		QObject::connect(q, SIGNAL(hovered()), rec, SLOT(evtV()));
	else if (sign=="toggled")
		QObject::connect(q, SIGNAL(toggled(bool)), rec, SLOT(evtB(bool)));
	else if (sign=="triggered")
		QObject::connect(q, SIGNAL(triggered(bool)), rec, SLOT(evtB(bool)));	

	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QActionWrap::off(const Arguments& args) {
	HandleScope scope;
  QActionWrap* w = node::ObjectWrap::Unwrap<QActionWrap>(args.This());
 	QAction* q = w->GetWrapped();
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);	
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());	
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
			
		if (sign=="changed") 
			QObject::disconnect(q, SIGNAL(changed()), rec, 0);	
		else if (sign=="hovered") 
			QObject::disconnect(q, SIGNAL(hovered()), rec, 0);	
		else if (sign=="toggled") 
			QObject::disconnect(q, SIGNAL(toggled(bool)), rec, 0);
		else if (sign=="triggered") 
			QObject::disconnect(q, SIGNAL(triggered(bool)), rec, 0);	
		// delete receivers?
	}
	return args.This();
}

Handle<Value> QActionWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QActionWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QActionWrap* w2 = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}


Handle<Value> QActionWrap::setCheckable(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setCheckable(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setChecked(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setChecked(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
    
  QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
		 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}
  if (!ico.isNull()) q->setIcon(ico);
  
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setShortcut(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setShortcut(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setStatusTip(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
	QAction* q = w->GetWrapped();
  q->setStatusTip(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setText(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::text(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  return scope.Close(qt_v8::FromQString(q->text()));
}

Handle<Value> QActionWrap::setShortcuts(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setShortcuts((QKeySequence::StandardKey) args[0]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QActionWrap::setEnabled(const Arguments& args) {
  HandleScope scope;
  QActionWrap* w = ObjectWrap::Unwrap<QActionWrap>(args.This());
  QAction* q = w->GetWrapped();
  q->setEnabled(args[0]->BooleanValue());
  return scope.Close(Undefined());
}