#include <QTranslator>
#include <QLocale>
#include "qapplication.h"
#include "qobject.h"

using namespace v8;

Persistent<Function> QApplicationWrap::constructor;

int QApplicationWrap::argc_ = 0;
char** QApplicationWrap::argv_ = NULL;

QApplicationWrap::QApplicationWrap() {
  q_ = new QApplication(argc_, argv_);
}

QApplicationWrap::~QApplicationWrap() {
  //delete q_;
}

void QApplicationWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QApplication"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
  ADD_METHOD(on)
  ADD_METHOD(off)
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(setObjectName)
	ADD_METHOD(objectName)
	
  ADD_METHOD(processEvents)
  ADD_METHOD(exec)
  
  ADD_METHOD(setStyleSheet)
  ADD_METHOD(styleSheet)
  
  ADD_METHOD(setQuitOnLastWindowClosed)
  ADD_METHOD(setActiveWindow)

	ADD_METHOD(focusWidget)
		
	ADD_METHOD(postEvent)
	ADD_METHOD(sendEvent)
	
	ADD_METHOD(tr)
	//ADD_METHOD(trUtf8)
	ADD_METHOD(installTranslator)
	ADD_METHOD(systemLocaleName)
	
	ADD_METHOD(applicationName)
	ADD_METHOD(setApplicationName)
	ADD_METHOD(organizationName)
	ADD_METHOD(setOrganizationName)
	ADD_METHOD(applicationVersion)
	ADD_METHOD(setApplicationVersion)
	
	// C) STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(translate)

//applicationVersion
//setApplicationVersion
//organizationDomain
//libraryPaths
//addLibraryPath
//applicationDirPath
//applicationFilePath
//arguments
//exit
//setAttribute

	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QApplication"), constructor);
}

Handle<Value> QApplicationWrap::New(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = new QApplicationWrap();
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QApplicationWrap::on(const Arguments& args) {
	HandleScope scope;
	
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
		
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
//	if (sign=="focusChanged") 
//		QObject::connect(q, SIGNAL(focusChanged()), rec, SLOT(evtV()));	

	if (sign=="lastWindowClosed")
		QObject::connect(q, SIGNAL(lastWindowClosed()), rec, SLOT(evtV()));

	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off(*event) (TODO: off(event, callback) )
Handle<Value> QApplicationWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
	QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);	
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
				
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
//		if (sign=="focusChanged") 
//			QObject::disconnect(q, SIGNAL(focusChanged()), rec, 0);

		if (sign=="lastWindowClosed") 
			QObject::disconnect(q, SIGNAL(lastWindowClosed()), rec, 0);	
			
		// delete receivers?
	}
		
	return args.This(); // return action ?
}

Handle<Value> QApplicationWrap::processEvents(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

  q->processEvents();

  return scope.Close(Undefined());
}

Handle<Value> QApplicationWrap::exec(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

  q->exec();

  return scope.Close(Undefined());
}

Handle<Value> QApplicationWrap::setStyleSheet(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

  q->setStyleSheet(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

Handle<Value> QApplicationWrap::styleSheet(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->styleSheet()));
}

// setQuitOnLastWindowClosed ( bool quit )
Handle<Value> QApplicationWrap::setQuitOnLastWindowClosed(const Arguments& args) {
  HandleScope scope;

  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();

	q->setQuitOnLastWindowClosed(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

// setActiveWindow ( object QWidget )
Handle<Value> QApplicationWrap::setActiveWindow(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* w = node::ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();
  
  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  q->setActiveWindow(widget_wrap->GetWrapped());
    
  return scope.Close(Undefined());
}

// focusWidget () -> object widget
Handle<Value> QApplicationWrap::focusWidget(const Arguments& args) {
  HandleScope scope;
  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();
  
  QWidget * widget = q->focusWidget();
  
  //TODO: wrap according to actual className
  
  return scope.Close(QWidgetWrap::NewInstance(widget));
}

// destroy
Handle<Value> QApplicationWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QApplicationWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* w = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplicationWrap* w2 = ObjectWrap::Unwrap<QApplicationWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}

// postEvent
Handle<Value> QApplicationWrap::postEvent(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
  QWidgetWrap* ww = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  QWidget* w = ww->GetWrapped();
  
  QEventWrap* ew = ObjectWrap::Unwrap<QEventWrap>(args[1]->ToObject());
  QEvent* e = ew->GetWrapped();
  
//  QMainWindowWrap* ww = ObjectWrap::Unwrap<QMainWindowWrap>(args[0]->ToObject());
//  QMainWindow* w = ww->GetWrapped();
//  
//  QMouseEventWrap* ew = ObjectWrap::Unwrap<QMouseEventWrap>(args[1]->ToObject());
//  QMouseEvent* e = ew->GetWrapped();
  
  a->postEvent(w, e);
  
  return scope.Close(Undefined());
}

// sendEvent
Handle<Value> QApplicationWrap::sendEvent(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
  QWidgetWrap* ww = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  QWidget* w = ww->GetWrapped();
  
  QEventWrap* ew = ObjectWrap::Unwrap<QEventWrap>(args[1]->ToObject());
  QEvent* e = ew->GetWrapped();
  
//  QMainWindowWrap* ww = ObjectWrap::Unwrap<QMainWindowWrap>(args[0]->ToObject());
//  QMainWindow* w = ww->GetWrapped();
//  
//  QMouseEventWrap* ew = ObjectWrap::Unwrap<QMouseEventWrap>(args[1]->ToObject());
//  QMouseEvent* e = ew->GetWrapped();
  
  a->sendEvent(w, e);
  
  return scope.Close(Undefined());
}

// tr
Handle<Value> QApplicationWrap::tr(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
  if (args.Length()==1)
	  return scope.Close(qt_v8::FromQString( a->tr(
	  	*String::AsciiValue(args[0]->ToString())
	  ) ));
	else if (args.Length()==2)
		return scope.Close(qt_v8::FromQString( a->tr(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString())
	  ) ));
	else if (args.Length()>2)
		return scope.Close(qt_v8::FromQString( a->tr(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString()),
	  	args[2]->IntegerValue()
	  ) ));
	return scope.Close(Undefined());
}

// trUtf8
//Handle<Value> QApplicationWrap::trUtf8(const Arguments& args) {
//  HandleScope scope;
//  
//  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
//  QApplication* a = aw->GetWrapped();
//  
//  return scope.Close(qt_v8::FromQString( a->trUtf8(
//  	*String::AsciiValue(args[0]->ToString()),
//  	*String::AsciiValue(args[1]->ToString()),
//  	args[2]->IntegerValue()
//  ) ));
//}

// installTranslator
Handle<Value> QApplicationWrap::installTranslator(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
  QTranslator* translator = new QTranslator;
 	translator->load(qt_v8::ToQString(args[0]->ToString()));
	a->installTranslator(translator);
  
  return scope.Close(Undefined());
}

// setObjectName
Handle<Value> QApplicationWrap::setObjectName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
	a->setObjectName(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

// objectName
Handle<Value> QApplicationWrap::objectName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
    
  return scope.Close(qt_v8::FromQString(a->objectName()));
}

// systemLocaleName
Handle<Value> QApplicationWrap::systemLocaleName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
    
  return scope.Close(qt_v8::FromQString(QLocale::system().name()));
}


// applicationName
Handle<Value> QApplicationWrap::applicationName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
    
  return scope.Close(qt_v8::FromQString(a->applicationName()));
}

// setApplicationName
Handle<Value> QApplicationWrap::setApplicationName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
	a->setApplicationName(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

// organizationName
Handle<Value> QApplicationWrap::organizationName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
    
  return scope.Close(qt_v8::FromQString(a->organizationName()));
}

// setOrganizationName
Handle<Value> QApplicationWrap::setOrganizationName(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
	a->setOrganizationName(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

// applicationVersion
Handle<Value> QApplicationWrap::applicationVersion(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
    
  return scope.Close(qt_v8::FromQString(a->applicationVersion()));
}

// setApplicationVersion
Handle<Value> QApplicationWrap::setApplicationVersion(const Arguments& args) {
  HandleScope scope;
  
  QApplicationWrap* aw = ObjectWrap::Unwrap<QApplicationWrap>(args.This());
  QApplication* a = aw->GetWrapped();
  
	a->setApplicationVersion(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}



// translate [static]
Handle<Value> QApplicationWrap::translate(const Arguments& args) {
  HandleScope scope;

  if (args.Length()==2)
	  return scope.Close(qt_v8::FromQString( QApplication::translate(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString())
	  ) ));
	else if (args.Length()==3)
		return scope.Close(qt_v8::FromQString( QApplication::translate(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString()),
	  	*String::AsciiValue(args[2]->ToString())
	  ) ));
	else if (args.Length()==4)
		return scope.Close(qt_v8::FromQString( QApplication::translate(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString()),
	  	*String::AsciiValue(args[2]->ToString()),
	  	(QCoreApplication::Encoding)args[3]->IntegerValue()
	  ) ));
	else if (args.Length()>4)
		return scope.Close(qt_v8::FromQString( QApplication::translate(
	  	*String::AsciiValue(args[0]->ToString()),
	  	*String::AsciiValue(args[1]->ToString()),
	  	*String::AsciiValue(args[2]->ToString()),
	  	(QCoreApplication::Encoding)args[3]->IntegerValue(),
	  	args[4]->IntegerValue()
	  ) ));
	return scope.Close(Undefined());
}