#include "qclipboard.h"
#include "node_buffer.h"
#include "qobject.h"

using namespace v8;

Persistent<Function> QClipboardWrap::constructor;

QClipboardWrap::QClipboardWrap(const Arguments& args) : q_(NULL) {
  //q_ = new QClipboard;
  q_ = QApplication::clipboard();
}

QClipboardWrap::~QClipboardWrap() {
  //delete q_;
}

void QClipboardWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QClipboard"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(clear)
  //ADD_METHOD(formats)
	ADD_METHOD(image)
	ADD_METHOD(mimeData)
	ADD_METHOD(setImage)
	ADD_METHOD(setMimeData)
	ADD_METHOD(setText)
	ADD_METHOD(text)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QClipboard"), constructor);
}

Handle<Value> QClipboardWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QClipboardWrap* w = new QClipboardWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

//Handle<Value> QClipboardWrap::NewInstance(QClipboard *q) {
//  HandleScope scope;
//  
//  Local<Object> instance = constructor->NewInstance(0, NULL);
//  QClipboardWrap* w = node::ObjectWrap::Unwrap<QClipboardWrap>(instance);
//  w->SetWrapped(q);
//
//  return scope.Close(instance);
//}

Handle<Value> QClipboardWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
	QClipboard* q = w->GetWrapped();

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="dataChanged") 
		QObject::connect(q, SIGNAL(dataChanged()), rec, SLOT(evtV()));	
			 
	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QClipboardWrap::off(const Arguments& args) {
	HandleScope scope;
  QClipboardWrap* w = node::ObjectWrap::Unwrap<QClipboardWrap>(args.This());
 	QClipboard* q = w->GetWrapped();
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);	
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="dataChanged") 
			QObject::disconnect(q, SIGNAL(dataChanged()), rec, 0);	
		// delete receivers?
	}
	return args.This();
}

//- clear ()
Handle<Value> QClipboardWrap::clear(const Arguments& args) {
  HandleScope scope;
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();
  
  q->clear();
  
  return scope.Close(Undefined());
}

//- formats () -> array
//Handle<Value> QClipboardWrap::formats(const Arguments& args) {
//  HandleScope scope;
//  
//  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
//  QClipboard* q = w->GetWrapped();
//  
//	const QMimeData * mime = q->mimeData();
//	QStringList formatList = mime->formats();
//
//	Handle<Array> formatArray = v8::Array::New(formatList.count());
//	for (int i=0;i<formatList.count();i++){
//		formatArray->Set(i, qt_v8::FromQString(formatList[i]) );
//	}
//  
//  return scope.Close(formatArray);
//}

//- image () -> object QImage
Handle<Value> QClipboardWrap::image(const Arguments& args) {
  HandleScope scope;
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();
  
  QImage * img = new QImage(q->image()); //.mirrored()
  
  return scope.Close(QImageWrap::NewInstance(img));
}

//- mimeData (string format) -> buffer (string|bytearray)
//Handle<Value> QClipboardWrap::mimeData(const Arguments& args) {
//  HandleScope scope;
//  //ASSERT_NARGS(1)
//  
//  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
//  QClipboard* q = w->GetWrapped();
//  
//	const QMimeData * mime = q->mimeData();
//	QByteArray ba = mime->data(qt_v8::ToQString(args[0]->ToString()));
//		
//	// QByteArray to Buffer
//	node::Buffer *buf = node::Buffer::New(ba.size());
//	memcpy(node::Buffer::Data(buf), ba.data(), ba.size());
//	  
//	return scope.Close( buf->handle_ );
//}

// mimeData ( Mode mode = Clipboard )
//Handle<Value> QClipboardWrap::mimeData(const Arguments& args) {
//  HandleScope scope;
//  //ASSERT_NARGS(1)
//  
//  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
//  QClipboard* q = w->GetWrapped();
//  
//	const QMimeData * mime = q->mimeData();	
//	return scope.Close(QMimeDataWrap::NewInstance((QMimeData *)mime));
//}

// mimeData([bool asString])
Handle<Value> QClipboardWrap::mimeData(const Arguments& args) {
  HandleScope scope;
  //ASSERT_NARGS(1)
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();
  
	const QMimeData * m = q->mimeData();
	
  bool asString = false;
  if (args.Length() > 0) {
  	asString = args[0]->BooleanValue();
  }
  
  // convert to native JS object
	Handle<Object> dataObj = v8::Object::New(); //m->formats().count()
	
	if (asString){
		// A) DATA AS STRING
		foreach(QString format, m->formats()){
			dataObj->Set(
				qt_v8::FromQString(format),
				qt_v8::FromQString(m->data(format).data())
			);
		}
	}else{
		foreach(QString format, m->formats()){
			QByteArray ba = m->data(format);
			node::Buffer *buf = node::Buffer::New(ba.size());
			memcpy(node::Buffer::Data(buf), ba.data(), ba.size());		
			dataObj->Set(
				qt_v8::FromQString(format),
				buf->handle_
			);
		}
	}
	
	return scope.Close(dataObj);
}

//- setImage (object QImage) // | string filename
Handle<Value> QClipboardWrap::setImage(const Arguments& args) {
  HandleScope scope;
  // ASSERT_NARGS(1)
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();

//  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
//		 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
//		 
//  } else 
  
  if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
	  q->setImage(*img);
	}
  return scope.Close(Undefined());
}

//- setMimeData (string format, buffer data) // string|buffer data
//Handle<Value> QClipboardWrap::setMimeData(const Arguments& args) {
//  HandleScope scope;
//  //ASSERT_NARGS(2)
//  
//  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
//  QClipboard* q = w->GetWrapped();
//
//	size_t size = node::Buffer::Length(args[1]->ToObject());
//	char *data = node::Buffer::Data(args[1]->ToObject());
//	QMimeData * mime = new QMimeData();
//	mime->setData(qt_v8::ToQString(args[0]->ToString()), QByteArray(data, size));
//	q->setMimeData(mime);
//	
//	//delete mime;
//					
//  return scope.Close(Undefined());
//}

//setMimeData ( QMimeData * src, Mode mode = Clipboard )
Handle<Value> QClipboardWrap::setMimeData(const Arguments& args) {
  HandleScope scope;
  //ASSERT_NARGS(2)
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();

//	QMimeDataWrap* mime_wrap = ObjectWrap::Unwrap<QMimeDataWrap>(args[0]->ToObject());
//	QMimeData* mime = mime_wrap->GetWrapped();
//	
//	q->setMimeData(mime);
	
	//delete mime;
					
  return scope.Close(Undefined());
}

//- setText (string)
Handle<Value> QClipboardWrap::setText(const Arguments& args) {
  HandleScope scope;
  //ASSERT_NARGS(1)
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();
  
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- text () -> string
Handle<Value> QClipboardWrap::text(const Arguments& args) {
  HandleScope scope;
  
  QClipboardWrap* w = ObjectWrap::Unwrap<QClipboardWrap>(args.This());
  QClipboard* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text()));
}
