#include "qdrag.h"
#include "../Widgets/qwidget.h"
#include "../Types/qpixmap.h"
//#include "qmimedata.h"
#include "qobject.h"

using namespace v8;

Persistent<Function> QDragWrap::constructor;

// QDrag ( QWidget * dragSource )
QDragWrap::QDragWrap(const v8::Arguments& args) : q_(NULL) {
  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());	
  q_ = new QDrag(widget_wrap->GetWrapped());
}

QDragWrap::~QDragWrap() {
  //delete q_;
}

void QDragWrap::Initialize(Handle<Object> targ) {
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDrag"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	ADD_METHOD(on)
	ADD_METHOD(off)
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(exec)
	ADD_METHOD(mimeData)
	ADD_METHOD(setMimeData)
	//		ADD_METHOD(setData)
	ADD_METHOD(pixmap)
	ADD_METHOD(setPixmap)
	ADD_METHOD(setDragCursor)
	ADD_METHOD(source)
	ADD_METHOD(target)
	ADD_METHOD(setHotSpot)

  constructor = Persistent<Function>::New(tpl->GetFunction()); 
  targ->Set(String::NewSymbol("QDrag"), constructor);
}

Handle<Value> QDragWrap::New(const Arguments& args) {
  HandleScope scope;

  QDragWrap* w = new QDragWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDragWrap::NewInstance(QDrag *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDragWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="actionChanged")
		QObject::connect(q, SIGNAL(actionChanged(Qt::DropAction)), rec, SLOT(evtI(Qt::DropAction)));	
	else if (sign=="targetChanged")
		QObject::connect(q, SIGNAL(targetChanged(QWidget *)), rec, SLOT(evtW(QWidget *)));

	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDragWrap::off(const Arguments& args) {
	HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
 	QDrag* q = w->GetWrapped();

	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="actionChanged") //WIDGET
			QObject::disconnect(q, SIGNAL(actionChanged(Qt::DropAction)), rec, 0);	
		else if (sign=="targetChanged") 
			QObject::disconnect(q, SIGNAL(targetChanged(QWidget *)), rec, 0);	
		
		// delete receivers?	  
	}
	return args.This();
}

// exec (int)
Handle<Value> QDragWrap::exec(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
  Qt::DropActions supportedActions = Qt::MoveAction;
	if (args.Length()>0){
		supportedActions = (Qt::DropActions)args[0]->IntegerValue();
	}
  
	return scope.Close(Integer::New(q->exec(supportedActions)));
}

// mimeData () -> QMimeData
//Handle<Value> QDragWrap::mimeData(const Arguments& args) {
//  HandleScope scope;
//  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
//  QDrag* q = w->GetWrapped();
//  
//	QMimeData * m = q->mimeData();
//  
//	return scope.Close(QMimeDataWrap::NewInstance(m));
//}

Handle<Value> QDragWrap::mimeData(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
	QMimeData * m = q->mimeData();
	
  // convert to native JS object
	Handle<Object> dataObj = v8::Object::New(); //m->formats().count()
	foreach(QString format, m->formats()){
		QByteArray ba = m->data(format);
		node::Buffer *buf = node::Buffer::New(ba.size());
		memcpy(node::Buffer::Data(buf), ba.data(), ba.size());		
		dataObj->Set(
			qt_v8::FromQString(format),
			buf->handle_
		);
	}
	
	return scope.Close(dataObj);
}


// setMimeData (object QMimeData)
//Handle<Value> QDragWrap::setMimeData(const Arguments& args) {
//  HandleScope scope;
//	ASSERT_NARGS(1)
//  ASSERT_OBJECT_TYPE(args[0], "QMimeData")
//    	
//  if (args[0]->IsObject()) {
//  	
//	  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
//	  QDrag* q = w->GetWrapped();
//  
//		QMimeDataWrap* m_wrap = ObjectWrap::Unwrap<QMimeDataWrap>(args[0]->ToObject());
//		QMimeData* m = m_wrap->GetWrapped();
//	
//	
//		//q->setMimeData(m); //Crash? // Ownership of the data is transferred to the QDrag object.
//	
//	
//	 	// create a copy OK!
//	  QMimeData * mimeCopy = new QMimeData();
//		foreach(QString format, m->formats()){
//		  // Checking for custom MIME types
//		  if(format.startsWith("application/x-qt")){	// Retrieving true format name
//		    int indexBegin = format.indexOf('"') + 1;
//		    int indexEnd = format.indexOf('"', indexBegin);
//		    format = format.mid(indexBegin, indexEnd - indexBegin);
//		  }
//		  mimeCopy->setData(format, m->data(format));
//		}
//		q->setMimeData(mimeCopy);
//		
//	
//	}
//	 
//  return scope.Close(Undefined());
//}

//TMP
//Handle<Value> QDragWrap::setData(const Arguments& args) {
//  HandleScope scope;
//	ASSERT_NARGS(2)
//
//  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
//  QDrag* q = w->GetWrapped();
//  
//  QMimeData * mime = new QMimeData();
//  
//  size_t size = node::Buffer::Length(args[1]->ToObject());
//	char *buf = node::Buffer::Data(args[1]->ToObject());
//  //return scope.Close(Boolean::New(q->loadFromData( QByteArray(buf, size) )));
//
//  mime->setData(qt_v8::ToQString(args[0]->ToString()), QByteArray(buf, size));
//  
//	q->setMimeData(mime);
//	 
//  return scope.Close(Undefined());
//}

Handle<Value> QDragWrap::setMimeData(const Arguments& args) {
  HandleScope scope;
	ASSERT_NARGS(1)

  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
  
  // CONVERT JS OBJECT TO QMIMEDATA
  Local<Object> dataObj = args[0]->ToObject();
	Local<Array> props = dataObj->GetPropertyNames();

	QMimeData * mime = new QMimeData();
	for(unsigned int i = 0; i < props->Length(); i++) {
		Local<Object> bufObj = dataObj->Get(props->Get(i))->ToObject();	
		size_t size = node::Buffer::Length(bufObj);
		char *buf = node::Buffer::Data(bufObj);
		
//		DBGJS(props->Get(i)->ToString());
//		DBGS(buf);
		
		mime->setData(
			qt_v8::ToQString(props->Get(i)->ToString()), 
			QByteArray(buf, size)
		);

  }
 
	q->setMimeData(mime);
	 
  return scope.Close(Undefined());
}

// pixmap () -> object QPixmap
Handle<Value> QDragWrap::pixmap(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
	QPixmap pm = q->pixmap();
  
	return scope.Close(QPixmapWrap::NewInstance(pm));
}

//setPixmap (object QPixmap)
Handle<Value> QDragWrap::setPixmap(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
  //if (args[0]->IsObject()) {
	QPixmapWrap* pm_wrap = ObjectWrap::Unwrap<QPixmapWrap>(args[0]->ToObject());
	QPixmap* pm = pm_wrap->GetWrapped();

  q->setPixmap(*pm);	
  
  return scope.Close(Undefined());
}

//setDragCursor ( object QPixmap & cursor, int Qt::DropAction action )
Handle<Value> QDragWrap::setDragCursor(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = node::ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
  
	QPixmapWrap* pm_wrap = ObjectWrap::Unwrap<QPixmapWrap>(args[0]->ToObject());
	QPixmap* pm = pm_wrap->GetWrapped();
	
	q->setDragCursor(*pm, (Qt::DropAction)args[1]->IntegerValue());
  
  return scope.Close(Undefined());
}

//source () -> object QWidget
Handle<Value> QDragWrap::source(const Arguments& args) {
  HandleScope scope;

  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();

	QWidget * widget = q->source();
	return scope.Close(QWidgetWrap::NewInstance(widget));
}

//target () -> object QWidget
Handle<Value> QDragWrap::target(const Arguments& args) {
  HandleScope scope;

  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();

	QWidget * widget = q->target();
	return scope.Close(QWidgetWrap::NewInstance(widget));
}

//setHotSpot (object QPoint)
Handle<Value> QDragWrap::setHotSpot(const Arguments& args) {
  HandleScope scope;

  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();

	QPointWrap* p_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
	QPoint* p = p_wrap->GetWrapped();
	
	q->setHotSpot(*p);// crash!?
	
	//TEST
//	QPoint p2 = new QPoint(*p);
//	q->setHotSpot(p2);
		
  return scope.Close(Undefined());
}

Handle<Value> QDragWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDrag* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QDragWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QDragWrap* w = ObjectWrap::Unwrap<QDragWrap>(args.This());
  QDragWrap* w2 = ObjectWrap::Unwrap<QDragWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}