#include "qfilesystemmodel.h"
#include "qobject.h"

using namespace v8;

Persistent<Function> QFileSystemModelWrap::constructor;

QFileSystemModelWrap::QFileSystemModelWrap(const Arguments& args) : q_(NULL) {
  q_ = new QFileSystemModel;
}

QFileSystemModelWrap::~QFileSystemModelWrap() {
  //delete q_;
}

void QFileSystemModelWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QFileSystemModel"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(fileName)
	ADD_METHOD(filePath)
	ADD_METHOD(isDir)
	ADD_METHOD(setFilter)
	ADD_METHOD(setRootPath)
	
	ADD_METHOD(index)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QFileSystemModel"), constructor);
}

Handle<Value> QFileSystemModelWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = new QFileSystemModelWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QFileSystemModelWrap::NewInstance(QFileSystemModel *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QFileSystemModelWrap* w = node::ObjectWrap::Unwrap<QFileSystemModelWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QFileSystemModelWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS	
	if (sign=="directoryLoaded")
		QObject::connect(q, SIGNAL(directoryLoaded(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="rootPathChanged")
		QObject::connect(q, SIGNAL(rootPathChanged(const QString &)), rec, SLOT(evtS(const QString &)));
		
	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QFileSystemModelWrap::off(const Arguments& args) {
	HandleScope scope;
  QFileSystemModelWrap* w = node::ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
 	QFileSystemModel* q = w->GetWrapped();
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);	
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="directoryLoaded")
			QObject::disconnect(q, SIGNAL(directoryLoaded(const QString &)), rec, 0);	
		else if (sign=="rootPathChanged")
			QObject::disconnect(q, SIGNAL(rootPathChanged(const QString &)), rec, 0);	
			
		// delete receivers?
	}
	return args.This();
}

Handle<Value> QFileSystemModelWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QFileSystemModelWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModelWrap* w2 = ObjectWrap::Unwrap<QFileSystemModelWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}


//Handle<Value> QFileSystemModelWrap::fileInfo(const Arguments& args) {
//  HandleScope scope;
//  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
//  QFileSystemModel* q = w->GetWrapped();
//  
//  QModelIndexWrap* model_index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
//  QFileInfo info = q->fileInfo(*model_index_wrap->GetWrapped());
//    
//  return QFileInfoWrap::NewInstance(&info);
//}

Handle<Value> QFileSystemModelWrap::fileName(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
  QModelIndexWrap* model_index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
  
  return scope.Close(qt_v8::FromQString(q->fileName(*model_index_wrap->GetWrapped())));
}

Handle<Value> QFileSystemModelWrap::filePath(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
  QModelIndexWrap* model_index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
    
  return scope.Close(qt_v8::FromQString(q->filePath(*model_index_wrap->GetWrapped())));
}

Handle<Value> QFileSystemModelWrap::isDir(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
  QModelIndexWrap* model_index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
    
  return scope.Close(Boolean::New(q->isDir(*model_index_wrap->GetWrapped())));
}

// setRootPath  (str) -> QModelIndex
Handle<Value> QFileSystemModelWrap::setRootPath(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
  return scope.Close(QModelIndexWrap::NewInstance(q->setRootPath(qt_v8::ToQString(args[0]->ToString()))));
}

//QDir::Dirs	0x001	List directories that match the filters.
//QDir::AllDirs	0x400	List all directories; i.e. don't apply the filters to directory names.
//QDir::Files	0x002	List files.
//QDir::Drives	0x004	List disk drives (ignored under Unix).
//QDir::NoSymLinks	0x008	Do not list symbolic links (ignored by operating systems that don't support symbolic links).
//QDir::NoDotAndDotDot	0x1000	Do not list the special entries "." and "..".
//QDir::NoDot	0x2000	Do not list the special entry ".".
//QDir::NoDotDot	0x4000	Do not list the special entry "..".
//QDir::AllEntries	Dirs | Files | Drives	List directories, files, drives and symlinks (this does not list broken symlinks unless you specify System).
//QDir::Readable	0x010	List files for which the application has read access. The Readable value needs to be combined with Dirs or Files.
//QDir::Writable	0x020	List files for which the application has write access. The Writable value needs to be combined with Dirs or Files.
//QDir::Executable	0x040	List files for which the application has execute access. The Executable value needs to be combined with Dirs or Files.
//QDir::Modified	0x080	Only list files that have been modified (ignored on Unix).
//QDir::Hidden	0x100	List hidden files (on Unix, files starting with a ".").
//QDir::System	0x200	List system files (on Unix, FIFOs, sockets and device files are included; on Windows, .lnk files are included)
//QDir::CaseSensitive	0x800	The filter should be case sensitive.

Handle<Value> QFileSystemModelWrap::setFilter(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
	q->setFilter((QDir::Filters)args[0]->IntegerValue());
				
  return scope.Close(Undefined());
}

// index ( const QString & path, int column = 0 )
Handle<Value> QFileSystemModelWrap::index(const Arguments& args) {
  HandleScope scope;
  
  QFileSystemModelWrap* w = ObjectWrap::Unwrap<QFileSystemModelWrap>(args.This());
  QFileSystemModel* q = w->GetWrapped();
  
  int col = args.Length()>1 ? args[1]->IntegerValue() : 0;
  
	QModelIndex m = q->index(
		qt_v8::ToQString(args[0]->ToString()),
		col
	);
				
  return scope.Close(QModelIndexWrap::NewInstance(m));
}

