#include "qgridlayout.h"

using namespace v8;

Persistent<Function> QGridLayoutWrap::constructor;

QGridLayoutWrap::QGridLayoutWrap(const Arguments& args) : q_(NULL) {
  q_ = new QGridLayout;
}

QGridLayoutWrap::~QGridLayoutWrap() {
  //delete q_;
}

void QGridLayoutWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QGridLayout"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(addLayout)
  ADD_METHOD(addWidget)

	ADD_METHOD(setSpacing)
  ADD_METHOD(setContentsMargins)
  
	ADD_METHOD(setHorizontalSpacing)
	ADD_METHOD(setVerticalSpacing)
	
	ADD_METHOD(setSizeConstraint)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QGridLayout"), constructor);
}

Handle<Value> QGridLayoutWrap::New(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = new QGridLayoutWrap(args);//
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QGridLayoutWrap::NewInstance(QGridLayout *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QGridLayoutWrap* w = node::ObjectWrap::Unwrap<QGridLayoutWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QGridLayoutWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QGridLayoutWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QGridLayoutWrap* w1 = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayoutWrap* w2 = ObjectWrap::Unwrap<QGridLayoutWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}

//Handle<Value> QGridLayoutWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
//	QGridLayout* q = w->GetWrapped();
//
//	return args.This(); // return action ?
//}

//grid: addLayout ( QGridLayout * layout, int row, int column, Qt::Alignment alignment = 0 )
// box: addLayout ( QGridLayout * layout, int stretch = 0 )
Handle<Value> QGridLayoutWrap::addLayout(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();

	QGridLayoutWrap* layout_wrap = ObjectWrap::Unwrap<QGridLayoutWrap>(args[0]->ToObject()); // TODO: OTHER LAYOUTS!
	
	int alignment = 0;
	if (args.Length()>3){
		alignment = args[3]->IntegerValue();
	}
	q->addLayout(
		layout_wrap->GetWrapped(),
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	(Qt::Alignment)alignment
	);

  return scope.Close(Undefined());
}

//grid: addWidget ( QWidget * widget, int row, int column, Qt::Alignment alignment = 0 )
// box: addWidget ( QWidget * widget, int stretch = 0, Qt::Alignment alignment = 0 )
Handle<Value> QGridLayoutWrap::addWidget(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());

	int alignment = 0;
	if (args.Length()>3){
		alignment = args[3]->IntegerValue();
	}
	q->addWidget(
		widget_wrap->GetWrapped(),
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	(Qt::Alignment)alignment
	);

  return scope.Close(Undefined());
}

// setSpacing(int)
Handle<Value> QGridLayoutWrap::setSpacing(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
  
  q->setSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setContentsMargins ( int left, int top, int right, int bottom )
Handle<Value> QGridLayoutWrap::setContentsMargins(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
  
  q->setContentsMargins(
  	args[0]->IntegerValue(),
  	args[1]->IntegerValue(),
  	args[2]->IntegerValue(),
  	args[3]->IntegerValue()
	);
  
  return scope.Close(Undefined());
}

// setHorizontalSpacing (int)
Handle<Value> QGridLayoutWrap::setHorizontalSpacing(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
  
  q->setHorizontalSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setVerticalSpacing (int)
Handle<Value> QGridLayoutWrap::setVerticalSpacing(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
  
  q->setVerticalSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setSizeConstraint (int)
Handle<Value> QGridLayoutWrap::setSizeConstraint(const Arguments& args) {
  HandleScope scope;

  QGridLayoutWrap* w = ObjectWrap::Unwrap<QGridLayoutWrap>(args.This());
  QGridLayout* q = w->GetWrapped();
  
  q->setSizeConstraint((QLayout::SizeConstraint)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}