#include "qhboxlayout.h"

using namespace v8;

Persistent<Function> QHBoxLayoutWrap::constructor;

QHBoxLayoutWrap::QHBoxLayoutWrap(const Arguments& args) : q_(NULL) {
  q_ = new QHBoxLayout;
}

QHBoxLayoutWrap::~QHBoxLayoutWrap() {
  //delete q_;
}

void QHBoxLayoutWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QHBoxLayout"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(addLayout)
  ADD_METHOD(addWidget)
	
	ADD_METHOD(setSpacing)
  ADD_METHOD(setContentsMargins)
  
	ADD_METHOD(addSpacing)
	ADD_METHOD(addStretch)
	
	ADD_METHOD(setSizeConstraint)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QHBoxLayout"), constructor);
}

Handle<Value> QHBoxLayoutWrap::New(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = new QHBoxLayoutWrap(args);//
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QHBoxLayoutWrap::NewInstance(QHBoxLayout *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QHBoxLayoutWrap* w = node::ObjectWrap::Unwrap<QHBoxLayoutWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

//Handle<Value> QHBoxLayoutWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
//	QHBoxLayout* q = w->GetWrapped();
//
//	return args.This(); // return action ?
//}

//grid: addLayout ( QHBoxLayout * layout, int row, int column, Qt::Alignment alignment = 0 )
// box: addLayout ( QHBoxLayout * layout, int stretch = 0 )
Handle<Value> QHBoxLayoutWrap::addLayout(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();

	QHBoxLayoutWrap* layout_wrap = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args[0]->ToObject()); // TODO: OTHER LAYOUTS!
			
	int stretch = 0;
	if (args.Length()>1){
		stretch = args[1]->IntegerValue();
	}
	q->addLayout(
		layout_wrap->GetWrapped(),
		stretch
	);

  return scope.Close(Undefined());
}

//grid: addWidget ( QWidget * widget, int row, int column, Qt::Alignment alignment = 0 )
// box: addWidget ( QWidget * widget, int stretch = 0, Qt::Alignment alignment = 0 )
Handle<Value> QHBoxLayoutWrap::addWidget(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	
	int stretch = 0;
	int alignment = 0;
	if (args.Length()>1){
		stretch = args[1]->IntegerValue();
	}
	if (args.Length()>2){
		alignment = args[2]->IntegerValue();
	}
	q->addWidget(
		widget_wrap->GetWrapped(),
		stretch,
		(Qt::Alignment)alignment
	);
			
  return scope.Close(Undefined());
}

Handle<Value> QHBoxLayoutWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QHBoxLayoutWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QHBoxLayoutWrap* w1 = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayoutWrap* w2 = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}

// setSpacing(int)
Handle<Value> QHBoxLayoutWrap::setSpacing(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
  
  q->setSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setContentsMargins ( int left, int top, int right, int bottom )
Handle<Value> QHBoxLayoutWrap::setContentsMargins(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
  
  q->setContentsMargins(
  	args[0]->IntegerValue(),
  	args[1]->IntegerValue(),
  	args[2]->IntegerValue(),
  	args[3]->IntegerValue()
	);
  
  return scope.Close(Undefined());
}

// addSpacing (int)
Handle<Value> QHBoxLayoutWrap::addSpacing(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
  
  q->addSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// addStretch (int)
Handle<Value> QHBoxLayoutWrap::addStretch(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
  
  q->addStretch(args.Length()?args[0]->IntegerValue():0);
  
  return scope.Close(Undefined());
}

// setSizeConstraint (int)
Handle<Value> QHBoxLayoutWrap::setSizeConstraint(const Arguments& args) {
  HandleScope scope;

  QHBoxLayoutWrap* w = ObjectWrap::Unwrap<QHBoxLayoutWrap>(args.This());
  QHBoxLayout* q = w->GetWrapped();
  
  q->setSizeConstraint((QLayout::SizeConstraint)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}