#include "qobject.h"

using namespace v8;

Persistent<Function> QObjectWrap::constructor;

QObjectWrap::QObjectWrap(const Arguments& args) : q_(NULL) {
	QObject * q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QObjectWrap* w_parent = ObjectWrap::Unwrap<QObjectWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	q_ = new QObject(q_parent);
}

QObjectWrap::~QObjectWrap() {
  //delete q_;
}

void QObjectWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QObject"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods  
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QObject"), constructor);
}

Handle<Value> QObjectWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QObjectWrap* w = new QObjectWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QObjectWrap::NewInstance(QObject *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QObjectWrap* w = node::ObjectWrap::Unwrap<QObjectWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QObjectWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QObjectWrap* w = ObjectWrap::Unwrap<QObjectWrap>(args.This());
  QObject* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QObjectWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QObjectWrap* w = ObjectWrap::Unwrap<QObjectWrap>(args.This());
  QObjectWrap* w2 = ObjectWrap::Unwrap<QObjectWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}
