#include "qsettings.h"
#include "../Types/qvariant.h"

#include "node_buffer.h"

using namespace v8;

Persistent<Function> QSettingsWrap::constructor;

// QSettings ( const QString & organization, const QString & application = QString(), QObject * parent = 0 )
// QSettings ( Scope scope, const QString & organization, const QString & application = QString(), QObject * parent = 0 )
	// QSettings ( Format format, Scope scope, const QString & organization, const QString & application = QString(), QObject * parent = 0 )
// QSettings ( const QString & fileName, Format format, QObject * parent = 0 )
// QSettings ( QObject * parent = 0 )

QSettingsWrap::QSettingsWrap(const Arguments& args) : q_(NULL) {
	//if (args.Length() > 0) {
		
	int format = args[0]->IntegerValue();
	int scope = args[1]->IntegerValue();
	QString orgName = qt_v8::ToQString(args[2]->ToString());
	QString appName = qt_v8::ToQString(args[3]->ToString());
		
  q_ = new QSettings((QSettings::Format)format, (QSettings::Scope)scope, orgName, appName);
}

QSettingsWrap::~QSettingsWrap() {
  //delete q_;
}

void QSettingsWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QSettings"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
  
  ADD_METHOD(clear)
  
	ADD_METHOD(value)
	ADD_METHOD(setValue)
	
	ADD_METHOD(beginGroup)
	ADD_METHOD(endGroup)
	
	ADD_METHOD(childGroups)
	ADD_METHOD(childKeys)
	
//	beginReadArray ( const QString & prefix )
//	beginWriteArray ( const QString & prefix, int size = -1 )
//	.endArray();
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QSettings"), constructor);
}

Handle<Value> QSettingsWrap::New(const Arguments& args) {
  HandleScope scope;
  QSettingsWrap* w = new QSettingsWrap(args);
  w->Wrap(args.This());
  return args.This();
}

//Handle<Value> QSettingsWrap::NewInstance(QSettings *q) {
//  HandleScope scope;
//  
//  Local<Object> instance = constructor->NewInstance(0, NULL);
//  QSettingsWrap* w = node::ObjectWrap::Unwrap<QSettingsWrap>(instance);
//  w->SetWrapped(q);
//
//  return scope.Close(instance);
//}

//- clear ()
Handle<Value> QSettingsWrap::clear(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
  
  q->clear();
  
  return scope.Close(Undefined());
}

//- value ()
Handle<Value> QSettingsWrap::value(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
   
	QString & key = qt_v8::ToQString(args[0]->ToString());
  QVariant v = q->value(key);
			
  return scope.Close(QVariantWrap::NewInstance(v));
}

//- setValue ()
Handle<Value> QSettingsWrap::setValue(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
  
  QString & key = qt_v8::ToQString(args[0]->ToString());
  
  QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[1]->ToObject());
  QVariant* val = vw->GetWrapped();
  
  q->setValue(key, *val);
  
  return scope.Close(Undefined());
}

//- beginGroup (QString & prefix)
Handle<Value> QSettingsWrap::beginGroup(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
  
  q->beginGroup(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

// endGroup ()
Handle<Value> QSettingsWrap::endGroup(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
  
  q->endGroup();
  
  return scope.Close(Undefined());
}

// childGroups ()
Handle<Value> QSettingsWrap::childGroups(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
    
  QStringList groupList = q->childGroups();
	Handle<Array> groupArray = v8::Array::New(groupList.count());
	for (int i=0;i<groupList.count();i++){
		groupArray->Set(i, qt_v8::FromQString(groupList[i]) );
	}
  return scope.Close(groupArray);
}

// childKeys ()
Handle<Value> QSettingsWrap::childKeys(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
      
  QStringList keyList = q->childKeys();
	Handle<Array> keyArray = v8::Array::New(keyList.count());
	for (int i=0;i<keyList.count();i++){
		keyArray->Set(i, qt_v8::FromQString(keyList[i]) );
	}
  return scope.Close(keyArray);
}

Handle<Value> QSettingsWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QSettingsWrap* w = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettings* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QSettingsWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QSettingsWrap* w1 = ObjectWrap::Unwrap<QSettingsWrap>(args.This());
  QSettingsWrap* w2 = ObjectWrap::Unwrap<QSettingsWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}