
#include "qsound.h"

using namespace v8;

Persistent<Function> QSoundWrap::constructor;

// Supported implementations:
//   QSound ( QString filename )
QSoundWrap::QSoundWrap(const Arguments& args) : q_(NULL) {
  q_ = new QSound(qt_v8::ToQString(args[0]->ToString()));
}

QSoundWrap::~QSoundWrap() {
  //delete q_;
}

void QSoundWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QSound"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(play)
  ADD_METHOD(fileName)
  ADD_METHOD(setLoops)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QSound"), constructor);
}

Handle<Value> QSoundWrap::New(const Arguments& args) {
  HandleScope scope;

  QSoundWrap* w = new QSoundWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QSoundWrap::play(const Arguments& args) {
  HandleScope scope;

  QSoundWrap* w = ObjectWrap::Unwrap<QSoundWrap>(args.This());
  QSound* q = w->GetWrapped();

  q->play();

  return scope.Close(Undefined());
}

Handle<Value> QSoundWrap::fileName(const Arguments& args) {
  HandleScope scope;

  QSoundWrap* w = ObjectWrap::Unwrap<QSoundWrap>(args.This());
  QSound* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->fileName()));
}

Handle<Value> QSoundWrap::setLoops(const Arguments& args) {
  HandleScope scope;

  QSoundWrap* w = ObjectWrap::Unwrap<QSoundWrap>(args.This());
  QSound* q = w->GetWrapped();

  q->setLoops(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QSoundWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QSoundWrap* w = ObjectWrap::Unwrap<QSoundWrap>(args.This());
  QSound* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QSoundWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QSoundWrap* w1 = ObjectWrap::Unwrap<QSoundWrap>(args.This());
  QSoundWrap* w2 = ObjectWrap::Unwrap<QSoundWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}