#include "qsystemtrayicon.h"
#include "qobject.h"

using namespace v8;

//template class QImpl<QSystemTrayIcon>;

/*************************************************
 QSystemTrayIcon (KEIN WIDGET)
*************************************************/

Persistent<Function> QSystemTrayIconWrap::constructor;

QSystemTrayIconWrap::QSystemTrayIconWrap(const Arguments& args, bool eventsFlag=false) : q_(NULL) {  
	//if (eventsFlag)
	//	q_ = new QImpl<QSystemTrayIcon>;
	//else
  	q_ = new QSystemTrayIcon;
	//hasEvents = eventsFlag;
}

QSystemTrayIconWrap::~QSystemTrayIconWrap() {
  //delete q_;
}

void QSystemTrayIconWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QSystemTrayIcon"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(setContextMenu)
  ADD_METHOD(setIcon)
	ADD_METHOD(setToolTip)
	ADD_METHOD(show)
	ADD_METHOD(hide)

//- setContextMenu (int menuID)
//- setIcon (string|image icon)
//- setToolTip (string)
//- show ()
//- hide ()
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QSystemTrayIcon"), constructor);
}

Handle<Value> QSystemTrayIconWrap::New(const Arguments& args) {
  HandleScope scope;
  
  bool eventsFlag = false;
  if (args.Length() > 0) {
  	eventsFlag = args[0]->BooleanValue();
  }
  QSystemTrayIconWrap* w = new QSystemTrayIconWrap(args, eventsFlag);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QSystemTrayIconWrap::NewInstance(QSystemTrayIcon *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QSystemTrayIconWrap* w = node::ObjectWrap::Unwrap<QSystemTrayIconWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QSystemTrayIconWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
	QSystemTrayIcon* q = w->GetWrapped();

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="activated") 
		QObject::connect(q, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),rec, SLOT(evtE3(QSystemTrayIcon::ActivationReason)));	
	else if (sign=="messageClicked") 
		QObject::connect(q, SIGNAL(messageClicked()), rec, SLOT(evtV()));	
		 
	else {
		delete rec;
		rec = 0;
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QSystemTrayIconWrap::off(const Arguments& args) {
	HandleScope scope;
  QSystemTrayIconWrap* w = node::ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
 	QSystemTrayIcon* q = w->GetWrapped();
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);	
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="activated") 
			QObject::disconnect(q, SIGNAL(activated(int)), rec, 0);	
		else if (sign=="messageClicked") 
			QObject::disconnect(q, SIGNAL(messageClicked()), rec, 0);	
		// delete receivers?
	}
	
	return args.This();
}

//- hide()
Handle<Value> QSystemTrayIconWrap::hide(const Arguments& args) {
  HandleScope scope;
  
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();
  
  q->hide();
  
  return scope.Close(Undefined());
}

//- setContextMenu (obj menu)
Handle<Value> QSystemTrayIconWrap::setContextMenu(const Arguments& args) {
  HandleScope scope;
  
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();
    
  QMenuWrap* menu_wrap = ObjectWrap::Unwrap<QMenuWrap>(args[0]->ToObject());  
	q->setContextMenu(menu_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- setIcon (string filename | object QImage)
Handle<Value> QSystemTrayIconWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  // ASSERT_NARGS(1)
  
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();

	QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
		 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
		 
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
  	
//		QImage img(imgInfo.iWidth, imgInfo.iHeight, QImage::Format_ARGB32);
//		PMoaVoid pPixels;
//		pObj->pMmImage->LockPixels (pMoaValue, &pPixels);
//		memcpy(img.bits(), (char *)pPixels, imgInfo.iWidth * imgInfo.iHeight * 4); //imgInfo.iTotalDepth/8;
//		pObj->pMmImage->UnlockPixels (pMoaValue);
//		QPixmap pm = QPixmap::fromImage(img.mirrored());
  	
  	// C:LOAD ICON FROM IMAGE
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}
	
  if (!ico.isNull()) q->setIcon(ico);
  return scope.Close(Undefined());
}

//- setToolTip (string)
Handle<Value> QSystemTrayIconWrap::setToolTip(const Arguments& args) {
  HandleScope scope;
  
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();
  
  q->setToolTip(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- show()
Handle<Value> QSystemTrayIconWrap::show(const Arguments& args) {
  HandleScope scope;
  
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();
  
  q->show();
  
  return scope.Close(Undefined());
}

Handle<Value> QSystemTrayIconWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QSystemTrayIconWrap* w = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIcon* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QSystemTrayIconWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QSystemTrayIconWrap* w1 = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args.This());
  QSystemTrayIconWrap* w2 = ObjectWrap::Unwrap<QSystemTrayIconWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}