#include "qvboxlayout.h"

using namespace v8;

Persistent<Function> QVBoxLayoutWrap::constructor;

QVBoxLayoutWrap::QVBoxLayoutWrap(const Arguments& args) : q_(NULL) {  
	q_ = new QVBoxLayout;
}

QVBoxLayoutWrap::~QVBoxLayoutWrap() {
  //delete q_;
}

void QVBoxLayoutWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QVBoxLayout"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
  
  ADD_METHOD(addLayout)
  ADD_METHOD(addWidget)
  
  ADD_METHOD(setSpacing)
  ADD_METHOD(setContentsMargins)
  
	ADD_METHOD(addSpacing)
	ADD_METHOD(addStretch)
	
	ADD_METHOD(setSizeConstraint)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QVBoxLayout"), constructor);
}

Handle<Value> QVBoxLayoutWrap::New(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = new QVBoxLayoutWrap(args);//
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QVBoxLayoutWrap::NewInstance(QVBoxLayout *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QVBoxLayoutWrap* w = node::ObjectWrap::Unwrap<QVBoxLayoutWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QVBoxLayoutWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QVBoxLayoutWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QVBoxLayoutWrap* w1 = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayoutWrap* w2 = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}

//Handle<Value> QVBoxLayoutWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
//	QVBoxLayout* q = w->GetWrapped();
//
//	return args.This(); // return action ?
//}

//grid: addLayout ( QVBoxLayout * layout, int row, int column, Qt::Alignment alignment = 0 )
// box: addLayout ( QVBoxLayout * layout, int stretch = 0 )
Handle<Value> QVBoxLayoutWrap::addLayout(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();

	QVBoxLayoutWrap* layout_wrap = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args[0]->ToObject());
	
	int stretch = 0;
	if (args.Length()>1){
		stretch = args[1]->IntegerValue();
	}
	q->addLayout(
		layout_wrap->GetWrapped(),
		stretch
	);

  return scope.Close(Undefined());
}

//grid: addWidget ( QWidget * widget, int row, int column, Qt::Alignment alignment = 0 )
// box: addWidget ( QWidget * widget, int stretch = 0, Qt::Alignment alignment = 0 )
Handle<Value> QVBoxLayoutWrap::addWidget(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	//QFrameWrap* widget_wrap = ObjectWrap::Unwrap<QFrameWrap>(args[0]->ToObject());
	
	int stretch = 0;
	int alignment = 0;
	if (args.Length()>1){
		stretch = args[1]->IntegerValue();
	}
	if (args.Length()>2){
		stretch = args[2]->IntegerValue();
	}
	q->addWidget(
		widget_wrap->GetWrapped(),
		stretch,
		(Qt::Alignment)alignment
	);
	
  return scope.Close(Undefined());
}

// setSpacing(int)
Handle<Value> QVBoxLayoutWrap::setSpacing(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
  
  q->setSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setContentsMargins ( int left, int top, int right, int bottom )
Handle<Value> QVBoxLayoutWrap::setContentsMargins(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
  
  q->setContentsMargins(
  	args[0]->IntegerValue(),
  	args[1]->IntegerValue(),
  	args[2]->IntegerValue(),
  	args[3]->IntegerValue()
	);
  
  return scope.Close(Undefined());
}

// addSpacing (int)
Handle<Value> QVBoxLayoutWrap::addSpacing(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
  
  q->addSpacing(args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// addStretch (int)
Handle<Value> QVBoxLayoutWrap::addStretch(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
  
  q->addStretch(args.Length()?args[0]->IntegerValue():0);
  
  return scope.Close(Undefined());
}

// setSizeConstraint (int)
Handle<Value> QVBoxLayoutWrap::setSizeConstraint(const Arguments& args) {
  HandleScope scope;

  QVBoxLayoutWrap* w = ObjectWrap::Unwrap<QVBoxLayoutWrap>(args.This());
  QVBoxLayout* q = w->GetWrapped();
  
  q->setSizeConstraint((QLayout::SizeConstraint)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}
