#include "qcolor.h"

using namespace v8;

Persistent<Function> QColorWrap::constructor;

// Supported implementations:
//   QColor ( int r, int g, int b, int a = 255 )
//   QColor ( QString color )
//   QColor ( QColor )
QColorWrap::QColorWrap(const Arguments& args) {
  if (args.Length() >= 3) {
    // QColor ( int r, int g, int b, int a = 255 )
    q_ = new QColor(
        args[0]->IntegerValue(), 
        args[1]->IntegerValue(),
        args[2]->IntegerValue(), 
        args[3]->IsNumber() ? args[3]->IntegerValue() : 255
    );
  } else if (args[0]->IsString()) {
    // QColor ( QString color )
    q_ = new QColor( qt_v8::ToQString(args[0]->ToString()) );
  } else if (args[0]->IsObject()) {
    // QColor ( QColor color )
    QString arg0_constructor = 
        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());

    if (arg0_constructor != "QColor")
      ThrowException(Exception::TypeError(
        String::New("QColor::QColor: bad argument")));

    // Unwrap obj
    QColorWrap* q_wrap = ObjectWrap::Unwrap<QColorWrap>(
        args[0]->ToObject());
    QColor* q = q_wrap->GetWrapped();

    q_ = new QColor(*q);
  }
}

QColorWrap::~QColorWrap() {
  //delete q_;
}

void QColorWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QColor"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
  ADD_METHOD(red)
  ADD_METHOD(green)
  ADD_METHOD(blue)
  ADD_METHOD(alpha)
  ADD_METHOD(name)

	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QColor"), constructor);
}

Handle<Value> QColorWrap::New(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = new QColorWrap(args);
  w->Wrap(args.This());

  return args.This();
}

//static
Handle<Value> QColorWrap::NewInstance(QColor q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QColorWrap* w = node::ObjectWrap::Unwrap<QColorWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QColorWrap::red(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();

  return scope.Close(Number::New(q->red()));
}

Handle<Value> QColorWrap::green(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();

  return scope.Close(Number::New(q->green()));
}

Handle<Value> QColorWrap::blue(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();

  return scope.Close(Number::New(q->blue()));
}

Handle<Value> QColorWrap::alpha(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();

  return scope.Close(Number::New(q->alpha()));
}

Handle<Value> QColorWrap::name(const Arguments& args) {
  HandleScope scope;

  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();

  QString name = q->name();

  return scope.Close(qt_v8::FromQString(name));
}

Handle<Value> QColorWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColor* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QColorWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QColorWrap* w = ObjectWrap::Unwrap<QColorWrap>(args.This());
  QColorWrap* w2 = ObjectWrap::Unwrap<QColorWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}