#include "qfont.h"

using namespace v8;

Persistent<Function> QFontWrap::constructor;

// Supported implementations:
//   QFont ( )
//   QFont ( const QString & family, int pointSize = -1, int weight = -1, 
//     bool italic = false )
//   QFont ( QFont font )
QFontWrap::QFontWrap(const Arguments& args) : q_(NULL) {
  if (args.Length() == 0) {
    // QFont ()

    q_ = new QFont;
    return;
  }

  // QFont ( QFont font )

  if (args.Length() == 1 && args[0]->IsObject()) {
    QString arg0_constructor = 
        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());

    if (arg0_constructor != "QFont")
      ThrowException(Exception::TypeError(
        String::New("QFont::QFont: bad argument")));

    // Unwrap obj
    QFontWrap* q_wrap = ObjectWrap::Unwrap<QFontWrap>(
        args[0]->ToObject());
    QFont* q = q_wrap->GetWrapped();

    q_ = new QFont(*q);
  }

  // QFont ( const QString & family, int pointSize = -1, int weight = -1, 
  //   bool italic = false )

  if (args.Length() == 1 && args[0]->IsString()) {
    q_ = new QFont(qt_v8::ToQString(args[0]->ToString()));
    return;
  }

  if (args.Length() == 2) {
    q_ = new QFont(qt_v8::ToQString(args[0]->ToString()), 
        args[1]->IntegerValue());
    return;
  }

  if (args.Length() == 3) {
    q_ = new QFont(qt_v8::ToQString(args[0]->ToString()), 
        args[1]->IntegerValue(), args[2]->IntegerValue());
    return;
  }

  if (args.Length() == 4) {
    q_ = new QFont(qt_v8::ToQString(args[0]->ToString()), 
        args[1]->IntegerValue(), args[2]->IntegerValue(),
        args[3]->BooleanValue());
    return;
  }
}

QFontWrap::~QFontWrap() {
  //delete q_;
}

void QFontWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QFont"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(SetFamily)
	ADD_METHOD(Family)
	ADD_METHOD(SetPixelSize)
	ADD_METHOD(PixelSize)
	ADD_METHOD(SetPointSize)
	ADD_METHOD(PointSize)
	ADD_METHOD(SetPointSizeF)
	ADD_METHOD(PointSizeF)
	ADD_METHOD(ToString)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QFont"), constructor);
}

Handle<Value> QFontWrap::New(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = new QFontWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QFontWrap::NewInstance(QFont q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QFontWrap* w = node::ObjectWrap::Unwrap<QFontWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QFontWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QFontWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QFontWrap* w1 = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFontWrap* w2 = ObjectWrap::Unwrap<QFontWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}


Handle<Value> QFontWrap::SetFamily(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  q->setFamily(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QFontWrap::Family(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->family()));
}

Handle<Value> QFontWrap::SetPixelSize(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  q->setPixelSize(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QFontWrap::PixelSize(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  return scope.Close(Number::New(q->pixelSize()));
}

Handle<Value> QFontWrap::SetPointSize(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  q->setPointSize(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QFontWrap::PointSize(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  return scope.Close(Number::New(q->pointSize()));
}

Handle<Value> QFontWrap::SetPointSizeF(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  q->setPointSizeF(args[0]->NumberValue());

  return scope.Close(Undefined());
}

Handle<Value> QFontWrap::PointSizeF(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  return scope.Close(Number::New(q->pointSizeF()));
}

Handle<Value> QFontWrap::ToString(const Arguments& args) {
  HandleScope scope;

  QFontWrap* w = ObjectWrap::Unwrap<QFontWrap>(args.This());
  QFont* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->toString()));
}

