#include "qimage.h"

using namespace v8;

Persistent<Function> QImageWrap::constructor;

// Supported implementations:
//   QImage ( )
//   QImage ( QString filename )
QImageWrap::QImageWrap(const Arguments& args) {
  q_ = new QImage();  
}

QImageWrap::~QImageWrap() {
  //delete q_;
}

void QImageWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QImage"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(isNull)
  ADD_METHOD(load)
  //ADD_METHOD(loadFromData)
  ADD_METHOD(save)
  //ADD_METHOD(grabWidget)
  //ADD_METHOD(grabWindow)
  ADD_METHOD(depth)
  ADD_METHOD(height)
  ADD_METHOD(width)
  ADD_METHOD(pixel)
  ADD_METHOD(setPixel)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QImage"), constructor);
}

Handle<Value> QImageWrap::New(const Arguments& args) {
  HandleScope scope;

  QImageWrap* w = new QImageWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QImageWrap::NewInstance(QImage *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QImageWrap* w = node::ObjectWrap::Unwrap<QImageWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// isNull() -> bool
Handle<Value> QImageWrap::isNull(const Arguments& args) {
  HandleScope scope;

  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

  return scope.Close(Boolean::New(q->isNull()));
}

// load (string filename, *string format) -> bool
// load (object buffer, *string format) -> bool
Handle<Value> QImageWrap::load(const Arguments& args) {
  HandleScope scope;
	//ASSERT_NARGS(1)
	
  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

	const char * format = 0;
	if (args.Length()>1){
		//gIsUtf8?X.toUtf8():X.toLatin1()
		format = qt_v8::ToQString(args[1]->ToString()).toLatin1();
	}
	bool ok = false;
	
	if (args[0]->IsString()){
		
		ok = q->load( qt_v8::ToQString(args[0]->ToString()), format);
		return scope.Close(Boolean::New( ok ));

	}else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="Buffer"){
		
		size_t size = node::Buffer::Length(args[0]->ToObject());
		char *buf = node::Buffer::Data(args[0]->ToObject());
		//return scope.Close(Boolean::New(q->loadFromData( QByteArray(buf, size) )));
			
		// loadFromData((uchar *)data, size, format))
		// loadFromData ( const QByteArray & data, const char * format = 0 )
		ok = q->loadFromData(QByteArray(buf, size), format);
		return scope.Close(Boolean::New( ok ));

	}else{
		// throw ...
  }
  
  return scope.Close(Boolean::New( ok ));
}

// loadFromData (object buffer) -> bool
//Handle<Value> QImageWrap::loadFromData(const Arguments& args) {
//  HandleScope scope;
//
//  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
//  QImage* q = w->GetWrapped();
//
//	size_t size = node::Buffer::Length(args[0]->ToObject());
//	char *buf = node::Buffer::Data(args[0]->ToObject());
//
//  return scope.Close(Boolean::New(q->loadFromData( QByteArray(buf, size) )));
//}

// save (string filename, *string format, int quality) -> bool
Handle<Value> QImageWrap::save(const Arguments& args) {
  HandleScope scope;
	//ASSERT_NARGS(1)
	
  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();
  
	const char * format = 0;
	if (args.Length()>1){
		format = qt_v8::ToQString(args[1]->ToString()).toLatin1();
	}
	
	int quality = -1;
	if (args.Length()>2){
		quality = args[2]->IntegerValue();
	}
	
  //save ( const QString & fileName, const char * format = 0, int quality = -1 ) const
  bool ok = q->save(qt_v8::ToQString(args[0]->ToString()), format, quality);
  
  return scope.Close(Boolean::New( ok ));
}

// depth () -> int
Handle<Value> QImageWrap::depth(const Arguments& args) {
  HandleScope scope;

  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

  return scope.Close(Integer::New(q->depth()));
}

// height () -> int
Handle<Value> QImageWrap::height(const Arguments& args) {
  HandleScope scope;

  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

  return scope.Close(Integer::New(q->height()));
}

// width () -> int
Handle<Value> QImageWrap::width(const Arguments& args) {
  HandleScope scope;

  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

  return scope.Close(Integer::New(q->width()));
}

// pixel (int x, int y) -> int #AARRGGBB
Handle<Value> QImageWrap::pixel(const Arguments& args) {
  HandleScope scope;
	//ASSERT_NARGS(2)
	
  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();

	QRgb rgb = q->pixel(args[0]->IntegerValue(), args[1]->IntegerValue());
	
  return scope.Close(Integer::New((int)rgb));
}

// setPixel (int x, int y, int rgba)
Handle<Value> QImageWrap::setPixel(const Arguments& args) {
  HandleScope scope;
  //ASSERT_NARGS(3)

  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();
	
	q->setPixel(args[0]->IntegerValue(), args[1]->IntegerValue(), (QRgb)args[2]->IntegerValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QImageWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImage* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QImageWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QImageWrap* w1 = ObjectWrap::Unwrap<QImageWrap>(args.This());
  QImageWrap* w2 = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}


//case kQtActionSaveData:
//	{
//		QByteArray ba;
//		QBuffer buffer(&ba);
//		buffer.open(QIODevice::WriteOnly);
//		if (callPtr->nargs>3){
//			const char * format;
//			GetArgByIndex(4, &moaValue);
//			pObj->pMmValue->ValueToStringPtr(&moaValue, &format);
//			img->save(&buffer, format);
//			pObj->pMmValue->ValueReleaseStringPtr(&moaValue);
//		}else{
//			img->save(&buffer);
//		}
//		RETURN_QBYTEARRAY(ba)


//// grabWidget (object QWidget) -> object QImage
//Handle<Value> QImageWrap::grabWidget(const Arguments& args) {
//  HandleScope scope;
//	//ASSERT_NARGS(1)
//	
//  QImageWrap* w = ObjectWrap::Unwrap<QImageWrap>(args.This());
//  QImage* q = w->GetWrapped();
//  
//	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
//	QWidget* widget = widget_wrap->GetWrapped();
//  QPixmap p = QPixmap::grabWidget(widget);
//  
//	//if (od->createdByDir) delete img; // ???
//	new QImage(p.toImage()); //.mirrored()
//  //od->createdByDir = true;
//  
//  return scope.Close(Boolean::New( ok ));
//}

//ASSERT_NARGS(4)
//long widgetId;
//GetArgByIndex(4, &moaValue);
//pObj->pMmValue->ValueToInteger(&moaValue, &widgetId);
//if (widgetId<1 || widgetId > pObj->dc->widgetDataList.count()) RETURN_INT_AND_EXIT(0)
//WidgetData * wd = pObj->dc->widgetDataList[widgetId-1];
//if (!wd || !wd->widget)	RETURN_INT_AND_EXIT(0)
//QPixmap p = QPixmap::grabWidget(wd->widget);
//if (od->createdByDir) delete img; // ???
//od->object = (QObject *)new QImage(p.toImage()); //.mirrored()
//od->createdByDir = true;

