#include "qlistwidgetitem.h"

using namespace v8;

Persistent<Function> QListWidgetItemWrap::constructor;

QListWidgetItemWrap::QListWidgetItemWrap(const Arguments& args) : q_(NULL) {
  q_ = new QListWidgetItem;
}

QListWidgetItemWrap::~QListWidgetItemWrap() {
  //delete q_;
}

void QListWidgetItemWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QListWidgetItem"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(flags)
	ADD_METHOD(isSelected)
	ADD_METHOD(setFlags)
	ADD_METHOD(setIcon)
	ADD_METHOD(setSelected)
	ADD_METHOD(setText)
	ADD_METHOD(text)
	
	ADD_METHOD(font)
	ADD_METHOD(setFont)
	
	ADD_METHOD(data)
	ADD_METHOD(setData)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QListWidgetItem"), constructor);
}

Handle<Value> QListWidgetItemWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = new QListWidgetItemWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QListWidgetItemWrap::NewInstance(QListWidgetItem *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QListWidgetItemWrap* w = node::ObjectWrap::Unwrap<QListWidgetItemWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

//Handle<Value> QListWidgetItemWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//	
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
//	QListWidgetItem* q = w->GetWrapped();
//	
//	return args.This(); // return action ?
//}

Handle<Value> QListWidgetItemWrap::destroy(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();

	delete q;

  return scope.Close(Undefined());
}

// equals
Handle<Value> QListWidgetItemWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItemWrap* w2 = ObjectWrap::Unwrap<QListWidgetItemWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}



//- setText (int col, string)
Handle<Value> QListWidgetItemWrap::setText(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- text (int col) -> string
Handle<Value> QListWidgetItemWrap::text(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text()));
}

//- flags () -> int
Handle<Value> QListWidgetItemWrap::flags(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
    
  return scope.Close( Integer::New(q->flags()) );
}

//- setFlags (int)
Handle<Value> QListWidgetItemWrap::setFlags(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  q->setFlags((Qt::ItemFlags)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

//- setIcon (int col, string|image icon[, iconFromFile])
Handle<Value> QListWidgetItemWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
    
	QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
  	
  	if (args.Length()>1 && args[1]->BooleanValue()){// TEST: get icon from file
	    QFileInfo info(qt_v8::ToQString(args[0]->ToString()));
	    QFileIconProvider ip;
	    ico = ip.icon(info);
	  }else{
	  	ico.addFile(qt_v8::ToQString(args[0]->ToString()));
		}
    
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}

  if (!ico.isNull()) q->setIcon(ico);
  
  return scope.Close(Undefined());
}

//- isSelected () -> bool
Handle<Value> QListWidgetItemWrap::isSelected(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
    
  return scope.Close( Boolean::New(q->isSelected()) );
}

//- setSelected (bool)
Handle<Value> QListWidgetItemWrap::setSelected(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  q->setSelected(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

//- font (int column) -> string
Handle<Value> QListWidgetItemWrap::font(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  QFont font = q->font();
  
  return scope.Close(qt_v8::FromQString(font.toString()));
}

//- setFont (int column, string fontname*, int pointSize, int weight, bool italic)
Handle<Value> QListWidgetItemWrap::setFont(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
    
  int pointSize = -1;
  int weight = -1;
  bool italic = false;
  if (args.Length() > 1) pointSize = args[1]->IntegerValue();
  if (args.Length() > 2) weight = args[2]->IntegerValue();
  if (args.Length() > 3) italic = args[3]->BooleanValue();
  q->setFont(QFont(qt_v8::ToQString(args[1]->ToString()), pointSize, weight, italic));
  
  return scope.Close(Undefined());
}

// data ( int role ) -> QVariant
Handle<Value> QListWidgetItemWrap::data(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
    	
  QVariant v = q->data(args[0]->IntegerValue()); // Qt::EditRole
			
  return scope.Close(QVariantWrap::NewInstance(v));
}

// setData ( int role, QVariant data )
Handle<Value> QListWidgetItemWrap::setData(const Arguments& args) {
  HandleScope scope;
  
  QListWidgetItemWrap* w = ObjectWrap::Unwrap<QListWidgetItemWrap>(args.This());
  QListWidgetItem* q = w->GetWrapped();
  
  QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[1]->ToObject());
  QVariant* v = vw->GetWrapped();
  
  q->setData(args[0]->IntegerValue(), *v);
  
  return scope.Close(Undefined());
}