
#include "qmatrix.h"

using namespace v8;

Persistent<Function> QMatrixWrap::constructor;

// Supported implementations:
//   QMatrix ( )
//   QMatrix ( qreal m11, qreal m12, qreal m21, qreal m22, qreal dx, qreal dy )
//   QMatrix ( QMatrix matrix )
QMatrixWrap::QMatrixWrap(const Arguments& args) : q_(NULL) {
  if (args.Length() == 0) {
    // QMatrix ( )

    q_ = new QMatrix;
  } else if (args[0]->IsObject()) {
    // QMatrix ( QMatrix matrix )

    QString arg0_constructor = 
        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());

    if (arg0_constructor != "QMatrix")
      ThrowException(Exception::TypeError(
        String::New("QMatrix::QMatrix: bad argument")));

    // Unwrap obj
    QMatrixWrap* q_wrap = ObjectWrap::Unwrap<QMatrixWrap>(
        args[0]->ToObject());
    QMatrix* q = q_wrap->GetWrapped();

    q_ = new QMatrix(*q);
  } else if (args.Length() == 6) {
    // QMatrix(qreal m11, qreal m12, qreal m21, qreal m22, qreal dx, qreal dy)

    q_ = new QMatrix(args[0]->NumberValue(), args[1]->NumberValue(),
                     args[2]->NumberValue(), args[3]->NumberValue(),
                     args[4]->NumberValue(), args[5]->NumberValue());
  }
}

QMatrixWrap::~QMatrixWrap() {
  //delete q_;
}

void QMatrixWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMatrix"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	ADD_METHOD(m11)
	ADD_METHOD(m12)
	ADD_METHOD(m21)
	ADD_METHOD(m22)
	ADD_METHOD(dx)
	ADD_METHOD(dy)
	ADD_METHOD(translate)
	ADD_METHOD(scale)

	ADD_METHOD(destroy)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMatrix"), constructor);
}

Handle<Value> QMatrixWrap::New(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = new QMatrixWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMatrixWrap::NewInstance(QMatrix q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMatrixWrap* w = node::ObjectWrap::Unwrap<QMatrixWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QMatrixWrap::m11(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->m11()));
}

Handle<Value> QMatrixWrap::m12(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->m12()));
}

Handle<Value> QMatrixWrap::m21(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->m21()));
}

Handle<Value> QMatrixWrap::m22(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->m22()));
}

Handle<Value> QMatrixWrap::dx(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->dx()));
}

Handle<Value> QMatrixWrap::dy(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  return scope.Close(Number::New(q->dy()));
}

Handle<Value> QMatrixWrap::translate(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  q->translate(args[0]->NumberValue(), args[1]->NumberValue());

  return scope.Close(args.This());
}

Handle<Value> QMatrixWrap::scale(const Arguments& args) {
  HandleScope scope;

  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();

  q->scale(args[0]->NumberValue(), args[1]->NumberValue());

  return scope.Close(args.This());
}


Handle<Value> QMatrixWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QMatrixWrap* w = ObjectWrap::Unwrap<QMatrixWrap>(args.This());
  QMatrix* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}