#include "qmodelindex.h"

using namespace v8;

Persistent<Function> QModelIndexWrap::constructor;

QModelIndexWrap::QModelIndexWrap(const Arguments& args) : q_(NULL) {
  q_ = new QModelIndex;
}

QModelIndexWrap::~QModelIndexWrap() {
  //delete q_;
}

void QModelIndexWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QModelIndex"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(column)
	ADD_METHOD(row)
	ADD_METHOD(data)
	
	ADD_METHOD(isValid)
	ADD_METHOD(child)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QModelIndex"), constructor);
}

Handle<Value> QModelIndexWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QModelIndexWrap* w = new QModelIndexWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QModelIndexWrap::NewInstance(QModelIndex q) {//
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QModelIndexWrap* w = node::ObjectWrap::Unwrap<QModelIndexWrap>(instance);
  w->SetWrapped(q);

//  Local<Object> instance = constructor->NewInstance(0, NULL);
//  QPointWrap* w = node::ObjectWrap::Unwrap<QPointWrap>(instance);
//  w->SetWrapped(q);
  
  return scope.Close(instance);
}

//Handle<Value> QModelIndexWrap::On(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//	
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
//	QModelIndex* q = w->GetWrapped();
//	
//	return args.This(); // return action ?
//}

Handle<Value> QModelIndexWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QModelIndexWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndexWrap* w2 = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( *w->GetWrapped()==*w2->GetWrapped() ));
}


Handle<Value> QModelIndexWrap::column(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
  return scope.Close(Integer::New(q->column()));
}

Handle<Value> QModelIndexWrap::row(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
  return scope.Close(Integer::New(q->row()));
}

Handle<Value> QModelIndexWrap::data(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->data().toString()));
}

Handle<Value> QModelIndexWrap::isValid(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
  return scope.Close(Boolean::New(q->isValid()));
}

// child (int row, int column) -> QModelIndexWrap
Handle<Value> QModelIndexWrap::child(const Arguments& args) {
  HandleScope scope;
  QModelIndexWrap* w = ObjectWrap::Unwrap<QModelIndexWrap>(args.This());
  QModelIndex* q = w->GetWrapped();
  
  QModelIndex m = q->child(args[0]->IntegerValue(), args[1]->IntegerValue());
  
  return scope.Close(QModelIndexWrap::NewInstance(m));
}