
#include "qpainter.h"

#include "qbrush.h"
#include "qcolor.h"
#include "qfont.h"
#include "qimage.h"
#include "qmatrix.h"
#include "qpainterpath.h"
#include "qpen.h"
#include "qpixmap.h"
#include "qpoint.h"

#include "../Widgets/qwidget.h"

using namespace v8;

Persistent<Function> QPainterWrap::constructor;

QPainterWrap::QPainterWrap(const Arguments& args) {
  q_ = new QPainter();
}
QPainterWrap::~QPainterWrap() {
  //delete q_;
}

void QPainterWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPainter"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(begin)
  ADD_METHOD(end)
  
  ADD_METHOD(save)
  ADD_METHOD(restore)
  
  // DRAW
  ADD_METHOD(drawArc)
  ADD_METHOD(drawChord)
  ADD_METHOD(drawEllipse)
	ADD_METHOD(drawImage)
  ADD_METHOD(drawLine)
  ADD_METHOD(drawPath)
  	//ADD_METHOD(drawPicture) //TODO
  ADD_METHOD(drawPie)
	ADD_METHOD(drawPixmap)
  ADD_METHOD(drawPoint)
  ADD_METHOD(drawPoints)
  ADD_METHOD(drawPolygon)
  ADD_METHOD(drawRect)
  ADD_METHOD(drawRoundedRect)
	ADD_METHOD(drawText)

  ADD_METHOD(isActive)
  ADD_METHOD(setPen)
  ADD_METHOD(setFont)
  ADD_METHOD(setMatrix)
  ADD_METHOD(fillRect)
  ADD_METHOD(strokePath)
  	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPainter"), constructor);
}

Handle<Value> QPainterWrap::New(const Arguments& args) {
  HandleScope scope;

//  if (args.Length()>0) {
//    return ThrowException(Exception::TypeError(
//        String::New("QPainterWrap: use begin() for initialization")));
//  }

  QPainterWrap* w = new QPainterWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPainterWrap::begin(const Arguments& args) {
  HandleScope scope;

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  if (!args[0]->IsObject())
    return ThrowException(Exception::TypeError(
        String::New("QPainterWrap:Begin: bad arguments")));

  QString constructor_name = 
    qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
  
  // Determine argument type (from its constructor) so we can unwrap it
  if (constructor_name == "QPixmap") {
    // QPixmap
    QPixmapWrap* pixmap_wrap = ObjectWrap::Unwrap<QPixmapWrap>(
        args[0]->ToObject());
    QPixmap* pixmap = pixmap_wrap->GetWrapped();

    return scope.Close(Boolean::New( q->begin(pixmap) ));
  } else if (constructor_name == "QWidget") {
    // QWidget
    QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(
        args[0]->ToObject());
    QWidget* widget = widget_wrap->GetWrapped();

    return scope.Close(Boolean::New( q->begin(widget) ));
  }

  // Unknown argument type
  return scope.Close(Boolean::New( false ));
}

Handle<Value> QPainterWrap::end(const Arguments& args) {
  HandleScope scope;

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  return scope.Close(Boolean::New( q->end() ));
}


Handle<Value> QPainterWrap::save(const Arguments& args) {
  HandleScope scope;

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->save();

  return scope.Close(Undefined());
}

Handle<Value> QPainterWrap::restore(const Arguments& args) {
  HandleScope scope;

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->restore();

  return scope.Close(Undefined());
}


//##########################################################
// DRAW
//##########################################################

//- drawArc (int x, int y, int width, int height, int startAngle, int spanAngle)
Handle<Value> QPainterWrap::drawArc(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(6)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()||!args[4]->IsNumber()||!args[5]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
      
  q->drawArc(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	args[3]->IntegerValue(),
  	args[4]->IntegerValue(), 
  	args[5]->IntegerValue()
  );

  return scope.Close(Undefined());
}

//- drawChord (int x, int y, int width, int height, int startAngle, int spanAngle)
Handle<Value> QPainterWrap::drawChord(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(6)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()||!args[4]->IsNumber()||!args[5]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->drawChord(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	args[3]->IntegerValue(),
  	args[4]->IntegerValue(), 
  	args[5]->IntegerValue()
  );

  return scope.Close(Undefined());
}

//- drawEllipse (int x, int y, int width, int height)
Handle<Value> QPainterWrap::drawEllipse(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(4)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
      
  q->drawEllipse(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	args[3]->IntegerValue()
  );

  return scope.Close(Undefined());
}

//   drawImage( int x, int y, QImage image )
Handle<Value> QPainterWrap::drawImage(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(3)
  if (!args[0]->IsNumber()||!args[1]->IsNumber()) THROW_ERROR_BAD_TYPES
  ASSERT_OBJECT_TYPE(args[2], "QImage")
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg2_constructor;
//  if (args[2]->IsObject()) {
//    arg2_constructor = 
//        qt_v8::ToQString(args[2]->ToObject()->GetConstructorName());
//  }
//
//  if (arg2_constructor != "QImage" ) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::DrawImage: image argument not recognized")));
//  }

  // Unwrap QImage
  QImageWrap* image_wrap = ObjectWrap::Unwrap<QImageWrap>(args[2]->ToObject());
  QImage* image = image_wrap->GetWrapped();
	
//  if (image->isNull()) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::DrawImage: image is null, no size set?")));
//  }
	ASSERT_NOT_NULL(image)
	
  q->drawImage(args[0]->IntegerValue(), args[1]->IntegerValue(), *image);

  return scope.Close(Undefined());
}

//- drawLine (int x1, int y1, int x2, int y2)
Handle<Value> QPainterWrap::drawLine(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(4)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()) THROW_ERROR_BAD_TYPES
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->drawLine(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	args[3]->IntegerValue()
  );

  return scope.Close(Undefined());
}

//- drawPath (QPainterPath path)
Handle<Value> QPainterWrap::drawPath(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
  ASSERT_OBJECT_TYPE(args[0], "QPainterPath")
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
  
  // Unwrap QPainterPath
  QPainterPathWrap* path_wrap = ObjectWrap::Unwrap<QPainterPathWrap>(args[0]->ToObject());
  QPainterPath* path = path_wrap->GetWrapped();

  q->drawPath(*path);

  return scope.Close(Undefined());
}

//- drawPicture (int x, int y, int objectID) [TODO]
//Handle<Value> QPainterWrap::drawPicture(const Arguments& args) {
//  HandleScope scope;
//
//	ASSERT_NARGS(3)
//	if (!args[0]->IsNumber()||!args[1]->IsNumber()) THROW_ERROR_BAD_TYPES
//  ASSERT_OBJECT_TYPE(args[2], "QPicture")
//  
//  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
//  QPainter* q = w->GetWrapped();
//  
//  // Unwrap QPicture
//  QPictureWrap* pict_wrap = ObjectWrap::Unwrap<QPictureWrap>(args[2]->ToObject());
//  QPicture* pict = pict_wrap->GetWrapped();
//
//  q->drawPicture(
//  	args[0]->IntegerValue(),
//  	args[1]->IntegerValue(),
//  	pict
//  );
//
//  return scope.Close(Undefined());
//}

//- drawPie (int x, int y, int width, int height, int startAngle, int spanAngle)
Handle<Value> QPainterWrap::drawPie(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(6)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()||!args[4]->IsNumber()||!args[5]->IsNumber()) THROW_ERROR_BAD_TYPES
 
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->drawPie(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	args[2]->IntegerValue(), 
  	args[3]->IntegerValue(),
  	args[4]->IntegerValue(), 
  	args[5]->IntegerValue()
  );

  return scope.Close(Undefined());
}

//   drawPixmap(int x, int y, QPixmap pixmap)
Handle<Value> QPainterWrap::drawPixmap(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(3)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()) THROW_ERROR_BAD_TYPES
	ASSERT_OBJECT_TYPE(args[2], "QPixmap")
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg2_constructor;
//  if (args[2]->IsObject()) {
//    arg2_constructor = 
//        qt_v8::ToQString(args[2]->ToObject()->GetConstructorName());
//  }
//
//  if (arg2_constructor != "QPixmap" ) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::DrawPixmap: pixmap argument not recognized")));
//  }
  
  // Unwrap QPixmap
  QPixmapWrap* pixmap_wrap = ObjectWrap::Unwrap<QPixmapWrap>(args[2]->ToObject());
  QPixmap* pixmap = pixmap_wrap->GetWrapped();

//  if (pixmap->isNull()) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::DrawPixmap: pixmap is null, no size set?")));
//  }
	ASSERT_NOT_NULL(pixmap)
	
  q->drawPixmap(args[0]->IntegerValue(), args[1]->IntegerValue(), *pixmap);

  return scope.Close(Undefined());
}

//- drawPoint (object QPoint)
Handle<Value> QPainterWrap::drawPoint(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QPoint")
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
  
  // Unwrap QPoint
  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[2]->ToObject());
  QPoint* point = point_wrap->GetWrapped();
	
  q->drawPoint(*point);

  return scope.Close(Undefined());
}

//- drawPoints (array)
Handle<Value> QPainterWrap::drawPoints(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	if (!args[0]->IsArray()) THROW_ERROR_BAD_TYPES
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
  
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  int cnt = props->Length();
  QPoint * points = new QPoint[cnt];
  
  for(int i = 0; i < cnt; i++) {

		// Unwrap QPoint
		QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(obj->Get(i)->ToObject());
		QPoint* point = point_wrap->GetWrapped();

		points[i] = QPoint(*point);//p.x, p.y); // ???
  }
  
	q->drawPoints(points, cnt);
	delete points;
	
  return scope.Close(Undefined());
}

//- drawPolygon (array)
Handle<Value> QPainterWrap::drawPolygon(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	if (!args[0]->IsArray()) THROW_ERROR_BAD_TYPES
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
  
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  int cnt = props->Length();
  QPoint * points = new QPoint[cnt];
  
  for(int i = 0; i < cnt; i++) {

		// Unwrap QPoint
		QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(obj->Get(i)->ToObject());
		QPoint* point = point_wrap->GetWrapped();

		points[i] = QPoint(*point);//p.x, p.y); // ???
  }
  
	q->drawPolygon(points, cnt);
	delete points;
	
  return scope.Close(Undefined());
}

//- drawRect (rect)
Handle<Value> QPainterWrap::drawRect(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(1)
  ASSERT_OBJECT_TYPE(args[0], "QRect")
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
    
  // Unwrap QRect
  QRectWrap* rect_wrap = ObjectWrap::Unwrap<QRectWrap>(args[0]->ToObject());
  QRect* rect = rect_wrap->GetWrapped();
  
  q->drawRect(*rect);

  return scope.Close(Undefined());
}

//- drawRoundedRect (rect, float xRadius, float yRadius) TODO: mode
Handle<Value> QPainterWrap::drawRoundedRect(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(3)
  ASSERT_OBJECT_TYPE(args[0], "QRect")
	if (!args[1]->IsNumber() || !args[2]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
  	
  // Unwrap QRect
  QRectWrap* rect_wrap = ObjectWrap::Unwrap<QRectWrap>(args[0]->ToObject());
  QRect* rect = rect_wrap->GetWrapped();
  
  q->drawRoundedRect(
  	*rect,
  	args[1]->NumberValue(), 
  	args[2]->NumberValue()
  );

  return scope.Close(Undefined());
}

//   drawText(int x, int y, "text")
Handle<Value> QPainterWrap::drawText(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(3)
  if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsString()) THROW_ERROR_BAD_TYPES
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  q->drawText(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
    qt_v8::ToQString(args[2]->ToString())
  );

  return scope.Close(Undefined());
}











Handle<Value> QPainterWrap::isActive(const Arguments& args) {
  HandleScope scope;

  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

  return scope.Close(Boolean::New( q->isActive() ));
}

//   setPen( QPen pen )
Handle<Value> QPainterWrap::setPen(const Arguments& args) {
  HandleScope scope;
  
	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QPen")
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg0_constructor;
//  if (args[0]->IsObject()) {
//    arg0_constructor = 
//        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
//  }
//
//  if (arg0_constructor != "QPen")
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::SetPen: bad argument")));


	
  // Unwrap obj
  QPenWrap* pen_wrap = ObjectWrap::Unwrap<QPenWrap>( args[0]->ToObject());
  QPen* pen = pen_wrap->GetWrapped();

  q->setPen(*pen);

  return scope.Close(Undefined());
}

Handle<Value> QPainterWrap::setFont(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QFont")
		
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg0_constructor;
//  if (args[0]->IsObject()) {
//    arg0_constructor = 
//        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
//  }
//
//  if (arg0_constructor != "QFont")
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::SetFont: bad argument")));
	
  // Unwrap obj
  QFontWrap* font_wrap = ObjectWrap::Unwrap<QFontWrap>(args[0]->ToObject());
  QFont* font = font_wrap->GetWrapped();

  q->setFont(*font);

  return scope.Close(Undefined());
}

// This seems to be undocumented in Qt, but it exists!
Handle<Value> QPainterWrap::setMatrix(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QMatrix")
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg0_constructor;
//  if (args[0]->IsObject()) {
//    arg0_constructor = 
//        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
//  }
//
//  if (arg0_constructor != "QMatrix")
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::SetMatrix: bad argument")));

  // Unwrap obj
  QMatrixWrap* matrix_wrap = ObjectWrap::Unwrap<QMatrixWrap>(args[0]->ToObject());
  QMatrix* matrix = matrix_wrap->GetWrapped();

  q->setMatrix(*matrix, args[1]->BooleanValue());

  return scope.Close(Undefined());
}

//   fillRect(int x, int y, int w, int h, QBrush brush)
//   fillRect(int x, int y, int w, int h, QColor color)
//   fillRect(int x, int y, int w, int h, Qt::GlobalColor color)
Handle<Value> QPainterWrap::fillRect(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(5)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  if (!args[0]->IsNumber() || !args[1]->IsNumber() || !args[2]->IsNumber() ||
//      !args[3]->IsNumber())
//    return scope.Close(Undefined());
  
  QString arg4_constructor;
  if (args[4]->IsObject()) {
    arg4_constructor = qt_v8::ToQString(args[4]->ToObject()->GetConstructorName());
  }
  
  if (arg4_constructor == "QBrush") {
    // fillRect(int x, int y, int w, int h, QBrush brush)

    // Unwrap QBrush
    QBrushWrap* brush_wrap = ObjectWrap::Unwrap<QBrushWrap>(args[4]->ToObject());
    QBrush* brush = brush_wrap->GetWrapped();

    q->fillRect(args[0]->IntegerValue(), args[1]->IntegerValue(),
                args[2]->IntegerValue(), args[3]->IntegerValue(), 
                *brush);
  } else if (arg4_constructor == "QColor") {
    // fillRect(int x, int y, int w, int h, QColor color)

    // Unwrap QColor
    QColorWrap* color_wrap = ObjectWrap::Unwrap<QColorWrap>(args[4]->ToObject());
    QColor* color = color_wrap->GetWrapped();

    q->fillRect(args[0]->IntegerValue(), args[1]->IntegerValue(),
                args[2]->IntegerValue(), args[3]->IntegerValue(), 
                *color);
  } else if (args[4]->IsNumber()) {
    // fillRect(int x, int y, int w, int h, Qt::GlobalColor color)

    q->fillRect(args[0]->IntegerValue(), args[1]->IntegerValue(),
                args[2]->IntegerValue(), args[3]->IntegerValue(), 
                (Qt::GlobalColor)args[4]->IntegerValue());
  } else {
    THROW_ERROR_BAD_TYPES
    //return ThrowException(Exception::TypeError(String::New("QPainterWrap:fillRect: bad arguments")));
  }

  return scope.Close(Undefined());
}

//   strokePath( QPainterPath path, QPen pen )
Handle<Value> QPainterWrap::strokePath(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(2)
  ASSERT_OBJECT_TYPE(args[0], "QPainterPath")
  ASSERT_OBJECT_TYPE(args[1], "QPen")
  
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();

//  QString arg0_constructor;
//  if (args[0]->IsObject()) {
//    arg0_constructor = 
//        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
//  }
//
//  if (arg0_constructor != "QPainterPath" ) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::StrokePath: bad arguments")));
//  }

  
//  QString arg1_constructor;
//  if (args[1]->IsObject()) {
//    arg1_constructor = 
//        qt_v8::ToQString(args[1]->ToObject()->GetConstructorName());
//  }
//
//  if (arg1_constructor != "QPen" ) {
//    return ThrowException(Exception::TypeError(
//      String::New("QPainterWrap::StrokePath: bad arguments")));
//  }
  
  // Unwrap QPainterPath
  QPainterPathWrap* path_wrap = ObjectWrap::Unwrap<QPainterPathWrap>(args[0]->ToObject());
  QPainterPath* path = path_wrap->GetWrapped();

  // Unwrap QPen
  QPenWrap* pen_wrap = ObjectWrap::Unwrap<QPenWrap>(args[1]->ToObject());
  QPen* pen = pen_wrap->GetWrapped();

  q->strokePath(*path, *pen);

  return scope.Close(Undefined());
}

Handle<Value> QPainterWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPainterWrap* w = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainter* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPainterWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPainterWrap* w1 = ObjectWrap::Unwrap<QPainterWrap>(args.This());
  QPainterWrap* w2 = ObjectWrap::Unwrap<QPainterWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}