#include "qpainterpath.h"

using namespace v8;

Persistent<Function> QPainterPathWrap::constructor;

// Supported implementations:
//   QPainterPath ( ??? )
QPainterPathWrap::QPainterPathWrap(const Arguments& args) {
  q_ = new QPainterPath();
}

QPainterPathWrap::~QPainterPathWrap() {
  //delete q_;
}

void QPainterPathWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPainterPath"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

	//- [#moveTo, float x, float y]
	//- [#lineTo, float x, float y]
//- [#cubicTo, float c1X, float c1Y, float c2X, float c2Y, float endPointX, float endPointY]
//- [#quadTo,  float cx, float cy, float endPointX, float endPointY]
//- [#arcTo, float x, float y, float width, float height, float startAngle, float sweepLength]
//- [#closeSubpath]

	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(closeSubpath)
  ADD_METHOD(currentPosition)
  
  ADD_METHOD(arcTo)
  ADD_METHOD(cubicTo)
  ADD_METHOD(lineTo)
  ADD_METHOD(moveTo)
  ADD_METHOD(quadTo)
  	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPainterPath"), constructor);
}

Handle<Value> QPainterPathWrap::New(const Arguments& args) {
  HandleScope scope;

  QPainterPathWrap* w = new QPainterPathWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPainterPathWrap::closeSubpath(const Arguments& args) {
  HandleScope scope;

  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  q->closeSubpath();
  
  return scope.Close(Undefined());
}

Handle<Value> QPainterPathWrap::currentPosition(const Arguments& args) {
  HandleScope scope;

  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  return scope.Close(QPointFWrap::NewInstance(q->currentPosition()));
}

// Supported versions:
//   lineTo( QPointF() )
Handle<Value> QPainterPathWrap::lineTo(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QPointF")
	
  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  // lineTo( QPointF point )
  QPointFWrap* pointf_wrap = ObjectWrap::Unwrap<QPointFWrap>(args[0]->ToObject());
  QPointF* pointf = pointf_wrap->GetWrapped();

  q->lineTo(*pointf);

  return scope.Close(Undefined());
}

// Supported versions:
//   moveTo( QPointF() )
Handle<Value> QPainterPathWrap::moveTo(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	ASSERT_OBJECT_TYPE(args[0], "QPointF")
	
  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  // moveTo( QPointF point )
  QPointFWrap* pointf_wrap = ObjectWrap::Unwrap<QPointFWrap>(args[0]->ToObject());
  QPointF* pointf = pointf_wrap->GetWrapped();

  q->moveTo(*pointf);

  return scope.Close(Undefined());
}

// Supported versions:
//   arcTo (float x, float y, float width, float height, float startAngle, float sweepLength)
Handle<Value> QPainterPathWrap::arcTo(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(6)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()||!args[4]->IsNumber()||!args[5]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  q->arcTo(
  	args[0]->NumberValue(), 
  	args[1]->NumberValue(), 
  	args[2]->NumberValue(), 
  	args[3]->NumberValue(),
  	args[4]->NumberValue(), 
  	args[5]->NumberValue()
  );
  
  return scope.Close(Undefined());
}

// Supported versions:
//   cubicTo (float c1X, float c1Y, float c2X, float c2Y, float endPointX, float endPointY)
Handle<Value> QPainterPathWrap::cubicTo(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(6)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()||!args[4]->IsNumber()||!args[5]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  q->cubicTo(
  	args[0]->NumberValue(), 
  	args[1]->NumberValue(), 
  	args[2]->NumberValue(), 
  	args[3]->NumberValue(),
  	args[4]->NumberValue(), 
  	args[5]->NumberValue()
  );
  
  return scope.Close(Undefined());
}

// Supported versions:
//   quadTo (float cx, float cy, float endPointX, float endPointY)
Handle<Value> QPainterPathWrap::quadTo(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(4)
	if (!args[0]->IsNumber()||!args[1]->IsNumber()||!args[2]->IsNumber()||!args[3]->IsNumber()) THROW_ERROR_BAD_TYPES

  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();

  q->quadTo(
  	args[0]->NumberValue(), 
  	args[1]->NumberValue(), 
  	args[2]->NumberValue(), 
  	args[3]->NumberValue()
  );
  
  return scope.Close(Undefined());
}

Handle<Value> QPainterPathWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPainterPathWrap* w = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPath* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPainterPathWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPainterPathWrap* w1 = ObjectWrap::Unwrap<QPainterPathWrap>(args.This());
  QPainterPathWrap* w2 = ObjectWrap::Unwrap<QPainterPathWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}