
#include "qpalette.h"
#include "qcolor.h"
//#include "qobjecttpl.h"

using namespace v8;

/*************************************************
 QPaletteWrap
*************************************************/

Persistent<Function> QPaletteWrap::constructor;

QPaletteWrap::QPaletteWrap(const v8::Arguments& args) : q_(NULL) {
  // Standalone constructor not implemented
  // Use SetWrapped()
  q_ = new QPalette;
}

QPaletteWrap::~QPaletteWrap() {
  //delete q_;
}

void QPaletteWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPalette"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

//		const QColor & 	color ( ColorRole role ) const
//		setColor ( ColorRole role, const QColor & color )
		
  // Wrapped method
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(color)
	ADD_METHOD(setColor)
		
	//geerbt
	//ADD_OBJECT_METHODS(QPaletteWrap, QPalette)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPalette"), constructor);
}

Handle<Value> QPaletteWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QPaletteWrap* w = new QPaletteWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QPaletteWrap::NewInstance(QPalette q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPaletteWrap* w = node::ObjectWrap::Unwrap<QPaletteWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// color (int ColorRole) -> object QColor
Handle<Value> QPaletteWrap::color(const Arguments& args) {
  HandleScope scope;
  QPaletteWrap* w = ObjectWrap::Unwrap<QPaletteWrap>(args.This());
  QPalette* q = w->GetWrapped();
  
  const QColor & col = q->color((QPalette::ColorRole)args[0]->IntegerValue());
	
  return scope.Close(QColorWrap::NewInstance(col));
}

// setColor (int ColorRole, object QColor)
Handle<Value> QPaletteWrap::setColor(const Arguments& args) {
  HandleScope scope;
  QPaletteWrap* w = ObjectWrap::Unwrap<QPaletteWrap>(args.This());
  QPalette* q = w->GetWrapped();
  
  QColorWrap* color_wrap = ObjectWrap::Unwrap<QColorWrap>(args[1]->ToObject());  
  q->setColor((QPalette::ColorRole)args[0]->IntegerValue(), *color_wrap->GetWrapped());
  
  return scope.Close(Undefined());
}

Handle<Value> QPaletteWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPaletteWrap* w = ObjectWrap::Unwrap<QPaletteWrap>(args.This());
  QPalette* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPaletteWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPaletteWrap* w1 = ObjectWrap::Unwrap<QPaletteWrap>(args.This());
  QPaletteWrap* w2 = ObjectWrap::Unwrap<QPaletteWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}