
#include "qpen.h"

using namespace v8;

Persistent<Function> QPenWrap::constructor;

// Supported implementations:
//   QPen (QBrush brush, qreal width, Qt::PenStyle style = Qt::SolidLine, Qt::PenCapStyle cap = Qt::SquareCap, Qt::PenJoinStyle join = Qt::BevelJoin )
//   QPen (QColor color)
//   QPen ()
QPenWrap::QPenWrap(const Arguments& args) {
	
  QString arg0_constructor;
  
  if (args[0]->IsObject()) {
  	arg0_constructor = qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
  }else{
    // QPen ()
    q_ = new QPen();
    return;
  }

  if (arg0_constructor == "QColor") {
    // QPen (QColor color)

    // Unwrap QColor
    QColorWrap* color_wrap = ObjectWrap::Unwrap<QColorWrap>(args[0]->ToObject());
    QColor* color = color_wrap->GetWrapped();

    q_ = new QPen(*color);
    return;
  } else if (arg0_constructor == "QBrush") {    
    // QPen (QBrush brush, qreal width, Qt::PenStyle style = Qt::SolidLine, Qt::PenCapStyle cap = Qt::SquareCap, Qt::PenJoinStyle join = Qt::BevelJoin )
    
    // Unwrap QBrush
    QBrushWrap* brush_wrap = ObjectWrap::Unwrap<QBrushWrap>(args[0]->ToObject());
    QBrush* brush = brush_wrap->GetWrapped();

    qreal width(args[1]->NumberValue());

    if (args.Length() == 2) {
      q_ = new QPen(*brush, width);
      return;
      
    }else if (args.Length() == 3) {
//      Qt::PenStyle style((Qt::PenStyle)args[2]->IntegerValue());

      q_ = new QPen(*brush, width, 
      	(Qt::PenStyle)args[2]->IntegerValue()
      );
      return;
      
    }else if (args.Length() == 4) {
//      Qt::PenStyle style((Qt::PenStyle)args[2]->IntegerValue());
//      Qt::PenCapStyle cap((Qt::PenCapStyle)args[3]->IntegerValue());

      q_ = new QPen(*brush, width, 
      	(Qt::PenStyle)args[2]->IntegerValue(), 
      	(Qt::PenCapStyle)args[3]->IntegerValue()
      );
      return;
      
    }else { // if (args.Length() == 5) 
//      Qt::PenStyle style((Qt::PenStyle)args[2]->IntegerValue());
//      Qt::PenCapStyle cap((Qt::PenCapStyle)args[3]->IntegerValue());
//      Qt::PenJoinStyle join((Qt::PenJoinStyle)args[4]->IntegerValue());

      q_ = new QPen(*brush, width, 
      	(Qt::PenStyle)args[2]->IntegerValue(), 
      	(Qt::PenCapStyle)args[3]->IntegerValue(), 
      	(Qt::PenJoinStyle)args[4]->IntegerValue()
      );
      return;
    }
  } // QPen (QBrush, ...)
}

QPenWrap::~QPenWrap() {
  //delete q_;
}

void QPenWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPen"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPen"), constructor);
}

Handle<Value> QPenWrap::New(const Arguments& args) {
  HandleScope scope;

  QPenWrap* w = new QPenWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPenWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPenWrap* w = ObjectWrap::Unwrap<QPenWrap>(args.This());
  QPen* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPenWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPenWrap* w1 = ObjectWrap::Unwrap<QPenWrap>(args.This());
  QPenWrap* w2 = ObjectWrap::Unwrap<QPenWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}