
#include "qpixmap.h"

using namespace v8;

Persistent<Function> QPixmapWrap::constructor;

QPixmapWrap::QPixmapWrap(int width, int height) : q_(NULL) {
  q_ = new QPixmap(width, height);
}

QPixmapWrap::~QPixmapWrap() {
  //delete q_;
}

void QPixmapWrap::Initialize(Handle<Object> target) {
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPixmap"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype	
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(width)
	ADD_METHOD(height)
	ADD_METHOD(save)
	ADD_METHOD(fill)
	
	ADD_METHOD(load)
	//ADD_METHOD(loadFromData)
  	
  constructor = Persistent<Function>::New(tpl->GetFunction()); 
  target->Set(String::NewSymbol("QPixmap"), constructor);
}

Handle<Value> QPixmapWrap::New(const Arguments& args) {
  HandleScope scope;

  QPixmapWrap* w = new QPixmapWrap(args[0]->IntegerValue(), args[1]->IntegerValue());
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPixmapWrap::NewInstance(QPixmap q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPixmapWrap* w = node::ObjectWrap::Unwrap<QPixmapWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QPixmapWrap::width(const Arguments& args) {
  HandleScope scope;

  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();

  return scope.Close(Number::New(q->width()));
}

Handle<Value> QPixmapWrap::height(const Arguments& args) {
  HandleScope scope;

  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();

  return scope.Close(Number::New(q->height()));
}

Handle<Value> QPixmapWrap::save(const Arguments& args) {
  HandleScope scope;

  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();

  QString file(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Boolean::New( q->save(file) ));
}

// Supports:
//    fill()
//    fill(QColor color)
Handle<Value> QPixmapWrap::fill(const Arguments& args) {
  HandleScope scope;

  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();

  if (args[0]->IsObject()) {
    // Unwrap QColor
    QColorWrap* color_wrap = ObjectWrap::Unwrap<QColorWrap>(
        args[0]->ToObject());
    QColor* color = color_wrap->GetWrapped();

    q->fill(*color);
  } else {
    q->fill();
  }

  return scope.Close(Undefined());
}



//Handle<Value> QPixmapWrap::load(const Arguments& args) {
//  HandleScope scope;
//
//  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
//  QPixmap* q = w->GetWrapped();
//
//  QString file(qt_v8::ToQString(args[0]->ToString()));
//
//  return scope.Close(Boolean::New( q->save(file) ));
//}



// load (string filename, *string format) -> bool
// load (object buffer, *string format) -> bool
Handle<Value> QPixmapWrap::load(const Arguments& args) {
  HandleScope scope;
	//ASSERT_NARGS(1)
	
  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();

	const char * format = 0;
	if (args.Length()>1){
		//gIsUtf8?X.toUtf8():X.toLatin1()
		format = qt_v8::ToQString(args[1]->ToString()).toLatin1();
	}
	bool ok = false;
	
	if (args[0]->IsString()){
		
		ok = q->load( qt_v8::ToQString(args[0]->ToString()), format);
		return scope.Close(Boolean::New( ok ));

	}else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="Buffer"){
		
		size_t size = node::Buffer::Length(args[0]->ToObject());
		char *buf = node::Buffer::Data(args[0]->ToObject());
		//return scope.Close(Boolean::New(q->loadFromData( QByteArray(buf, size) )));
			
		// loadFromData((uchar *)data, size, format))
		// loadFromData ( const QByteArray & data, const char * format = 0 )
		ok = q->loadFromData(QByteArray(buf, size), format);
		return scope.Close(Boolean::New( ok ));

	}else{
		// throw ...
  }
  
  return scope.Close(Boolean::New( ok ));
}

Handle<Value> QPixmapWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPixmapWrap* w = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmap* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPixmapWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPixmapWrap* w1 = ObjectWrap::Unwrap<QPixmapWrap>(args.This());
  QPixmapWrap* w2 = ObjectWrap::Unwrap<QPixmapWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}