#include "qpointf.h"

using namespace v8;

Persistent<Function> QPointFWrap::constructor;

// Supported implementations:
//   QPointF (qreal x, qreal y)
QPointFWrap::QPointFWrap(const Arguments& args) : q_(NULL) {
  if (args[0]->IsNumber() && args[1]->IsNumber()) {
    q_ = new QPointF(args[0]->NumberValue(), args[1]->NumberValue());
  } else {
    q_ = new QPointF;
  }
}

QPointFWrap::~QPointFWrap() {
  //delete q_;
}

void QPointFWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPointF"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
//  tpl->PrototypeTemplate()->Set(String::NewSymbol("x"),
//      FunctionTemplate::New(X)->GetFunction());
//  tpl->PrototypeTemplate()->Set(String::NewSymbol("y"),
//      FunctionTemplate::New(Y)->GetFunction());
//  tpl->PrototypeTemplate()->Set(String::NewSymbol("isNull"),
//      FunctionTemplate::New(IsNull)->GetFunction());

	ADD_METHOD(destroy)
	// ADD_METHOD(equals)
	
	ADD_METHOD(x)
	ADD_METHOD(y)
	ADD_METHOD(isNull)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPointF"), constructor);
}

Handle<Value> QPointFWrap::New(const Arguments& args) {
  HandleScope scope;

  QPointFWrap* w = new QPointFWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPointFWrap::NewInstance(QPointF q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPointFWrap* w = node::ObjectWrap::Unwrap<QPointFWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QPointFWrap::x(const Arguments& args) {
  HandleScope scope;

  QPointFWrap* w = ObjectWrap::Unwrap<QPointFWrap>(args.This());
  QPointF* q = w->GetWrapped();

  return scope.Close(Number::New(q->x()));
}

Handle<Value> QPointFWrap::y(const Arguments& args) {
  HandleScope scope;

  QPointFWrap* w = ObjectWrap::Unwrap<QPointFWrap>(args.This());
  QPointF* q = w->GetWrapped();

  return scope.Close(Number::New(q->y()));
}

Handle<Value> QPointFWrap::isNull(const Arguments& args) {
  HandleScope scope;

  QPointFWrap* w = ObjectWrap::Unwrap<QPointFWrap>(args.This());
  QPointF* q = w->GetWrapped();

  return scope.Close(Boolean::New(q->isNull()));
}

Handle<Value> QPointFWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPointFWrap* w = ObjectWrap::Unwrap<QPointFWrap>(args.This());
  QPointF* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}