
#include "qprinter.h"

using namespace v8;

Persistent<Function> QPrinterWrap::constructor;

QPrinterWrap::QPrinterWrap(const Arguments& args) : q_(NULL) {
  q_ = new QPrinter(QPrinter::ScreenResolution);
}

QPrinterWrap::~QPrinterWrap() {
  //delete q_;
}

void QPrinterWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPrinter"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods	
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
	ADD_METHOD(newPage)
	ADD_METHOD(setOutputFileName)
	ADD_METHOD(setOutputFormat)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPrinter"), constructor);
}

Handle<Value> QPrinterWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QPrinterWrap* w = new QPrinterWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QPrinterWrap::NewInstance(QPrinter *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPrinterWrap* w = node::ObjectWrap::Unwrap<QPrinterWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// newPage ()
Handle<Value> QPrinterWrap::newPage(const Arguments& args) {
  HandleScope scope;
  
  QPrinterWrap* w = ObjectWrap::Unwrap<QPrinterWrap>(args.This());
  QPrinter* q = w->GetWrapped();
  
  q->newPage();
  
  return scope.Close(Undefined());
}

// setOutputFileName (string)
Handle<Value> QPrinterWrap::setOutputFileName(const Arguments& args) {
  HandleScope scope;
  
  QPrinterWrap* w = ObjectWrap::Unwrap<QPrinterWrap>(args.This());
  QPrinter* q = w->GetWrapped();
  
  q->setOutputFileName(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

// setOutputFormat (int)
Handle<Value> QPrinterWrap::setOutputFormat(const Arguments& args) {
  HandleScope scope;
  
  QPrinterWrap* w = ObjectWrap::Unwrap<QPrinterWrap>(args.This());
  QPrinter* q = w->GetWrapped();
  
  q->setOutputFormat((QPrinter::OutputFormat)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

Handle<Value> QPrinterWrap::destroy(const Arguments& args) {
  HandleScope scope;
  QPrinterWrap* w = ObjectWrap::Unwrap<QPrinterWrap>(args.This());
  QPrinter* q = w->GetWrapped();
	delete q;
  return scope.Close(Undefined());
}

// equals
Handle<Value> QPrinterWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QPrinterWrap* w1 = ObjectWrap::Unwrap<QPrinterWrap>(args.This());
  QPrinterWrap* w2 = ObjectWrap::Unwrap<QPrinterWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w1->GetWrapped()==w2->GetWrapped() ));
}