#include "qtablewidgetitem.h"

using namespace v8;

//TEST

class MyTableWidgetItem : public QTableWidgetItem
{
public:
	MyTableWidgetItem(int n=0) :
	QTableWidgetItem(),
	fstype(n)
	{}

virtual bool operator<(QTableWidgetItem const &other) const
{
	MyTableWidgetItem const *item = dynamic_cast<MyTableWidgetItem const*>(&other);
	if(item) {
		if (fstype && item->fstype && fstype!=item->fstype)
			return fstype < item->fstype;
		else
			return QTableWidgetItem::operator<(other);
	} else {
		return QTableWidgetItem::operator<(other);
	}
}

public:
	int fstype;
};



Persistent<Function> QTableWidgetItemWrap::constructor;

QTableWidgetItemWrap::QTableWidgetItemWrap(const Arguments& args) : q_(NULL) {
  q_ = new MyTableWidgetItem;
}

QTableWidgetItemWrap::~QTableWidgetItemWrap() {
  //delete q_;
}

void QTableWidgetItemWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTableWidgetItem"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  // Wrapped methods  
	ADD_METHOD(flags)
	ADD_METHOD(isSelected)
	ADD_METHOD(setFlags)
	ADD_METHOD(setIcon)
	ADD_METHOD(setSelected)
	ADD_METHOD(setText)
	//ADD_METHOD(setData)
	ADD_METHOD(text)
	ADD_METHOD(column)
	ADD_METHOD(row)
	ADD_METHOD(font)
	ADD_METHOD(setFont)
	ADD_METHOD(setTextAlignment)
		
	ADD_METHOD(data)
	ADD_METHOD(setData)
	
	//CUSTOM
//	ADD_METHOD(setNumber)
//	ADD_METHOD(setDate)
//	ADD_METHOD(getNumber)
//	ADD_METHOD(getDate)
	
	ADD_METHOD(setFolder)
	ADD_METHOD(setFile)
	ADD_METHOD(isFolder)

	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTableWidgetItem"), constructor);
}

Handle<Value> QTableWidgetItemWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = new QTableWidgetItemWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QTableWidgetItemWrap::NewInstance(QTableWidgetItem *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTableWidgetItemWrap* w = node::ObjectWrap::Unwrap<QTableWidgetItemWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

//Handle<Value> QTableWidgetItemWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//	
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//	QTableWidgetItem* q = w->GetWrapped();
//	
//	return args.This(); // return action ?
//}

Handle<Value> QTableWidgetItemWrap::destroy(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();

	delete q;

  return scope.Close(Undefined());
}

// equals
Handle<Value> QTableWidgetItemWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItemWrap* w2 = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}



//- setText (string)
Handle<Value> QTableWidgetItemWrap::setText(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- text () -> string
Handle<Value> QTableWidgetItemWrap::text(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text()));
}

//- flags () -> int
Handle<Value> QTableWidgetItemWrap::flags(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
  return scope.Close( Integer::New(q->flags()) );
}

//- setFlags (int)
Handle<Value> QTableWidgetItemWrap::setFlags(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  q->setFlags((Qt::ItemFlags)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

//- setIcon (string|image icon[, iconFromFile])
Handle<Value> QTableWidgetItemWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
	QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
  	
  	if (args.Length()>1){// TEST: get icon from file
  		QFileInfo fileInfo(qt_v8::ToQString(args[0]->ToString()));
  		bool exists = args[1]->BooleanValue();
			if (exists){
			QFileIconProvider ip;
			ico = ip.icon(fileInfo);
			}else{
		   	// TEST
		   	QPixmap pixmap;
				//QPixmapCache iconCache;
				//if (!instance()->iconCache.find(fileInfo.suffix(), &pixmap)){
		static HRESULT comInit = CoInitialize(NULL);
		Q_UNUSED(comInit);
		SHFILEINFO shFileInfo;
		unsigned long val = 0;
				val = SHGetFileInfo(("foo." + fileInfo.suffix()).toStdString().c_str(), 0, &shFileInfo,
					sizeof(SHFILEINFO), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES);
				// Even if GetFileInfo returns a valid result, hIcon can be empty in some cases
		if (val && shFileInfo.hIcon){
		  pixmap = QPixmap::fromWinHICON(shFileInfo.hIcon);
		  //if (!pixmap.isNull()){
					//	instance()->iconCache.insert(fileInfo.suffix(), pixmap);
		  //}
		  DestroyIcon(shFileInfo.hIcon);
		  ico = QIcon(pixmap);
		}
			}
	  }else{
			ico.addFile(qt_v8::ToQString(args[0]->ToString()));
		}
		 
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}

  if (!ico.isNull()) q->setIcon(ico);
  
  return scope.Close(Undefined());
}

//- isSelected () -> bool
Handle<Value> QTableWidgetItemWrap::isSelected(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
  return scope.Close( Boolean::New(q->isSelected()) );
}

//- setSelected (bool)
Handle<Value> QTableWidgetItemWrap::setSelected(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  q->setSelected(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

//- column () -> int
Handle<Value> QTableWidgetItemWrap::column(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
  return scope.Close( Integer::New(q->column()) );
}

//- row () -> int
Handle<Value> QTableWidgetItemWrap::row(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
  return scope.Close( Integer::New(q->row()) );
}

//- font () -> string
Handle<Value> QTableWidgetItemWrap::font(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  QFont font = q->font();
  
  return scope.Close(qt_v8::FromQString(font.toString()));
}

//- setFont (string fontname*, int pointSize, int weight, bool italic)
Handle<Value> QTableWidgetItemWrap::setFont(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
	
  int pointSize = -1;
  int weight = -1;
  bool italic = false;
  if (args.Length() > 1) pointSize = args[1]->IntegerValue();
  if (args.Length() > 2) weight = args[2]->IntegerValue();
  if (args.Length() > 3) italic = args[3]->BooleanValue();
  q->setFont( QFont(qt_v8::ToQString(args[0]->ToString()), pointSize, weight, italic) );
  
  return scope.Close(Undefined());
}

//- setTextAlignment (int)
Handle<Value> QTableWidgetItemWrap::setTextAlignment(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  q->setTextAlignment((Qt::Alignment)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}




// CUSTOM

//- setNumber (int) 
//Handle<Value> QTableWidgetItemWrap::setNumber(const Arguments& args) {
//  HandleScope scope;
//  
//  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//  QTableWidgetItem* q = w->GetWrapped();
//  
//  q->setData( Qt::EditRole, QVariant( args[0]->IntegerValue() ) );
//  
//  return scope.Close(Undefined());
//}

//- getNumber -> int
//Handle<Value> QTableWidgetItemWrap::getNumber(const Arguments& args) {
//  HandleScope scope;
//  
//  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//  QTableWidgetItem* q = w->GetWrapped();
//  
//  QVariant v = q->data(Qt::EditRole);
//
//  return scope.Close( Integer::New(v.toInt()) );
//}


//- setDate (int y,m,d, h,m,s) 
//Handle<Value> QTableWidgetItemWrap::setDate(const Arguments& args) {
//  HandleScope scope;
//  
//  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//  QTableWidgetItem* q = w->GetWrapped();
//  
//  QDate d = QDate(args[0]->IntegerValue(),args[1]->IntegerValue(),args[2]->IntegerValue());
//  QTime t = QTime(args[3]->IntegerValue(),args[4]->IntegerValue(),args[5]->IntegerValue());
//  QDateTime dt = QDateTime(d,t);
//  
//  q->setData( Qt::EditRole, QVariant(dt) );
//  
//  return scope.Close(Undefined());
//}

//- getDate -> string y,m,d,h,m,s
//Handle<Value> QTableWidgetItemWrap::getDate(const Arguments& args) {
//  HandleScope scope;
//  
//  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//  QTableWidgetItem* q = w->GetWrapped();
//  
//  QVariant v = q->data(Qt::EditRole);
//	QDateTime dt = v.toDateTime();
//	
//  return scope.Close(qt_v8::FromQString(dt.toString("yyyy,M,d,h,m,s")));
//}


//- setFolder (str) 
Handle<Value> QTableWidgetItemWrap::setFolder(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
	MyTableWidgetItem* m = dynamic_cast<MyTableWidgetItem*>(q);
	if (m) m->fstype=1;
  
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- setFile (str) 
Handle<Value> QTableWidgetItemWrap::setFile(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
	MyTableWidgetItem* m = dynamic_cast<MyTableWidgetItem*>(q);
	if (m) m->fstype=2;
  
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

//- isFolder () -> bool
Handle<Value> QTableWidgetItemWrap::isFolder(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
	MyTableWidgetItem* m = dynamic_cast<MyTableWidgetItem*>(q);
	bool isFolder = (m && m->fstype==1);
    
  return scope.Close( Boolean::New(isFolder) );
}




//- data (int role) -> QVariant
Handle<Value> QTableWidgetItemWrap::data(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  QVariant v = q->data(args[0]->IntegerValue()); //Qt::EditRole
			
  return scope.Close(QVariantWrap::NewInstance(v));
}

//- setData (int role, QVariant data) 
Handle<Value> QTableWidgetItemWrap::setData(const Arguments& args) {
  HandleScope scope;
  
  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
  QTableWidgetItem* q = w->GetWrapped();
  
  QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[1]->ToObject());
  QVariant* v = vw->GetWrapped();
  
  q->setData(args[0]->IntegerValue(), *v); // Qt::EditRole
  
  return scope.Close(Undefined());
}

//Handle<Value> QTableWidgetItemWrap::getNumber(const Arguments& args) {
//  HandleScope scope;
//  
//  QTableWidgetItemWrap* w = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args.This());
//  QTableWidgetItem* q = w->GetWrapped();
//  
//  QVariant v = q->data(Qt::EditRole);
//
//  return scope.Close( Integer::New(v.toInt()) );
//}