#include "qtreewidgetitem.h"

using namespace v8;

Persistent<Function> QTreeWidgetItemWrap::constructor;

QTreeWidgetItemWrap::QTreeWidgetItemWrap(const Arguments& args) : q_(NULL) {
  q_ = new QTreeWidgetItem;
}

QTreeWidgetItemWrap::~QTreeWidgetItemWrap() {
  //delete q_;
}

void QTreeWidgetItemWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTreeWidgetItem"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods  
	ADD_METHOD(destroy)
	ADD_METHOD(equals)
	
  ADD_METHOD(addChild)
  ADD_METHOD(child)

	ADD_METHOD(flags)
	ADD_METHOD(isSelected)
	ADD_METHOD(setFlags)
	ADD_METHOD(setIcon)
	ADD_METHOD(setSelected)
	ADD_METHOD(setText)
	ADD_METHOD(text)
	
	ADD_METHOD(font)
	ADD_METHOD(setFont)
	
	ADD_METHOD(removeChild)
	
	ADD_METHOD(childCount)
	
	ADD_METHOD(data)
	ADD_METHOD(setData)
	
	ADD_METHOD(parent)
		
	ADD_METHOD(setExpanded)

	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTreeWidgetItem"), constructor);
}

Handle<Value> QTreeWidgetItemWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = new QTreeWidgetItemWrap(args);// 
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QTreeWidgetItemWrap::NewInstance(QTreeWidgetItem *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTreeWidgetItemWrap* w = node::ObjectWrap::Unwrap<QTreeWidgetItemWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

//Handle<Value> QTreeWidgetItemWrap::on(const Arguments& args) {
//	HandleScope scope;
////	if (!args[1]->IsFunction()) {
////		return ThrowException(Exception::TypeError(String::New("Second argument must be a callback function")));
////	}
//	
//	// There's no ToFunction(), use a Cast instead.
//	Local<Function> callback = Local<Function>::Cast(args[1]);
//
//	QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
//	QTreeWidgetItem* q = w->GetWrapped();
//	
//	return args.This(); // return action ?
//}


// destroy
Handle<Value> QTreeWidgetItemWrap::destroy(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();

	delete q;

  return scope.Close(Undefined());
}

// equals
Handle<Value> QTreeWidgetItemWrap::equals(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItemWrap* w2 = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());
  
  return scope.Close(Boolean::New( w->GetWrapped()==w2->GetWrapped() ));
}





//- addChild (object item)
Handle<Value> QTreeWidgetItemWrap::addChild(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());  
	q->addChild(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- child (int index) -> object item
Handle<Value> QTreeWidgetItemWrap::child(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    	
  QTreeWidgetItem *item = q->child(args[0]->IntegerValue());
  
  return scope.Close(QTreeWidgetItemWrap::NewInstance(item));
}

//- setText (int col, string)
Handle<Value> QTreeWidgetItemWrap::setText(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  q->setText(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
  
  return scope.Close(Undefined());
}

//- text (int col) -> string
Handle<Value> QTreeWidgetItemWrap::text(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text(args[0]->IntegerValue())));
}

//- flags () -> int
Handle<Value> QTreeWidgetItemWrap::flags(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  return scope.Close( Integer::New(q->flags()) );
}

//- setFlags (int)
Handle<Value> QTreeWidgetItemWrap::setFlags(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  q->setFlags((Qt::ItemFlags)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

//- setIcon (int col, string|image icon[, iconFromFile])
Handle<Value> QTreeWidgetItemWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
	QIcon ico;
  if (args[1]->IsString()) { //A: LOAD ICON FROM FILE
  	
  	if (args.Length()>2 && args[2]->BooleanValue()){// TEST: get icon from file
	    QFileInfo info(qt_v8::ToQString(args[1]->ToString()));
	    QFileIconProvider ip;
	    ico = ip.icon(info);
	  }else{
	  	ico.addFile(qt_v8::ToQString(args[1]->ToString()));
		}
    
  } else if (args[1]->IsObject() && qt_v8::ToQString(args[1]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[1]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}

  if (!ico.isNull()) q->setIcon(args[0]->IntegerValue(), ico);
  
  return scope.Close(Undefined());
}

//- isSelected () -> bool
Handle<Value> QTreeWidgetItemWrap::isSelected(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  return scope.Close( Boolean::New(q->isSelected()) );
}

//- setSelected (bool)
Handle<Value> QTreeWidgetItemWrap::setSelected(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  q->setSelected(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

//- font (int column) -> string
Handle<Value> QTreeWidgetItemWrap::font(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  QFont font = q->font(args[0]->IntegerValue());
  
  return scope.Close(qt_v8::FromQString(font.toString()));
}

//- setFont (int column, string fontname*, int pointSize, int weight, bool italic)
Handle<Value> QTreeWidgetItemWrap::setFont(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  int pointSize = -1;
  int weight = -1;
  bool italic = false;
  if (args.Length() > 2) pointSize = args[2]->IntegerValue();
  if (args.Length() > 3) weight = args[3]->IntegerValue();
  if (args.Length() > 4) italic = args[4]->BooleanValue();
  q->setFont(args[0]->IntegerValue(), QFont(qt_v8::ToQString(args[1]->ToString()), pointSize, weight, italic));
  
  return scope.Close(Undefined());
}


//- removeChild (object item)
Handle<Value> QTreeWidgetItemWrap::removeChild(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());  
	q->removeChild(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- childCount () -> int
Handle<Value> QTreeWidgetItemWrap::childCount(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
    
  return scope.Close( Integer::New(q->childCount()) );
}


// data ( int column, int role ) -> QVariant
Handle<Value> QTreeWidgetItemWrap::data(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  QVariant v = q->data(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue()
  );
  
  return scope.Close(QVariantWrap::NewInstance(v));
}

// setData ( int column, int role, QVariant data )
Handle<Value> QTreeWidgetItemWrap::setData(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
  
  QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[2]->ToObject());
  QVariant* v = vw->GetWrapped();
    
  q->setData(
  	args[0]->IntegerValue(), 
  	args[1]->IntegerValue(), 
  	*v //qt_v8::ToQString(args[2]->ToString())
  );
  
  return scope.Close(Undefined());
}

// parent () -> 
Handle<Value> QTreeWidgetItemWrap::parent(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();

  QTreeWidgetItem *item = q->parent();
  
  // ???
  return scope.Close(item ? QTreeWidgetItemWrap::NewInstance(item) : Undefined());
}

// setExpanded ( bool )
Handle<Value> QTreeWidgetItemWrap::setExpanded(const Arguments& args) {
  HandleScope scope;
  
  QTreeWidgetItemWrap* w = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args.This());
  QTreeWidgetItem* q = w->GetWrapped();
   	
  q->setExpanded(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

