#include "qvariant.h"

using namespace v8;

//static
Persistent<Function> QVariantWrap::constructor;

QVariantWrap::QVariantWrap(const Arguments& args) : q_(NULL) {
  if (args[0]->IsBoolean()) { // IsBooleanObject
  	q_ = new QVariant(args[0]->BooleanValue());
  }else if (args[0]->IsInt32()) {
  	q_ = new QVariant(args[0]->IntegerValue());
  }else if (args[0]->IsNumber()) {
  	q_ = new QVariant(args[0]->NumberValue());
  }else if (args[0]->IsString()) {
    q_ = new QVariant( qt_v8::ToQString(args[0]->ToString()) );
  }else if (args[0]->IsDate()) {
  	double millis = v8::Date::Cast(*args[0])->NumberValue();
  	QDateTime dt = QDateTime::fromTime_t(millis/1000);
  	q_ = new QVariant(dt);
  }else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="Buffer") {
  	// buffer to bytearray
		size_t size = node::Buffer::Length(args[0]->ToObject());
		char *data = node::Buffer::Data(args[0]->ToObject());
		q_ = new QVariant(QByteArray(data, size));
	}
  	
}

QVariantWrap::~QVariantWrap() {
  //delete q_;
}

//static
void QVariantWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QVariant"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
	//ADD_METHOD(destroy)
	// ADD_METHOD(equals)
	
	ADD_METHOD(toBool)
	ADD_METHOD(toInt)
	ADD_METHOD(toFloat)
	ADD_METHOD(toString)
	ADD_METHOD(toDateTime)
	
	ADD_METHOD(toBuffer)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QVariant"), constructor);
}

//static
Handle<Value> QVariantWrap::New(const Arguments& args) {
  HandleScope scope;
  QVariantWrap* w = new QVariantWrap(args);
  w->Wrap(args.This());
  return args.This();
}

//static
Handle<Value> QVariantWrap::NewInstance(QVariant q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QVariantWrap* w = node::ObjectWrap::Unwrap<QVariantWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// toBool
Handle<Value> QVariantWrap::toBool(const Arguments& args) {
  HandleScope scope;

  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();
        
  return scope.Close(Boolean::New(q->toBool()));
}

// toInt
Handle<Value> QVariantWrap::toInt(const Arguments& args) {
  HandleScope scope;

  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();

  return scope.Close(Integer::New(q->toInt()));
}

// toFloat
Handle<Value> QVariantWrap::toFloat(const Arguments& args) {
  HandleScope scope;
  
  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();

  return scope.Close(Number::New(q->toFloat()));
}

// toString
Handle<Value> QVariantWrap::toString(const Arguments& args) {
  HandleScope scope;

  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();
        
  return scope.Close(qt_v8::FromQString(q->toString()));
}

// toDateTime
Handle<Value> QVariantWrap::toDateTime(const Arguments& args) {
  HandleScope scope;

  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();
        
  QDateTime dt = q->toDateTime();
  double ms = dt.toTime_t() * 1000;
  
  return scope.Close( v8::Date::New(ms) ); // TEST?
}

// toBuffer
Handle<Value> QVariantWrap::toBuffer(const Arguments& args) {
  HandleScope scope;

  QVariantWrap* w = ObjectWrap::Unwrap<QVariantWrap>(args.This());
  QVariant* q = w->GetWrapped();
  
	QByteArray ba = q->toByteArray();
	
	node::Buffer *buf = node::Buffer::New(ba.size());
	memcpy(node::Buffer::Data(buf), ba.data(), ba.size());
	
//	dataObj->Set(
//		qt_v8::FromQString(format),
//		buf->handle_
//	);
		
  return scope.Close( buf->handle_ ); // TEST?
}	
		