
#include "qcolordialog.h"

#include "../qplaintextedit.h"
#include "../qwidgeteventstpl.h"

#include "../../Types/qcolor.h"
#include "../../Types/qfont.h"
#include "../../Types/qprinter.h"

using namespace v8;

template class QImpl<QColorDialog>;

/*************************************************
 QColorDialogWrap
*************************************************/

Persistent<Function> QColorDialogWrap::constructor;

QColorDialogWrap::QColorDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QColorDialog>(q_parent, fixScope);
	else
  	q_ = new QColorDialog(q_parent);
}

QColorDialogWrap::~QColorDialogWrap() {
  //delete q_;
}

void QColorDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QColorDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
	// OWN METHODS  
  ADD_METHOD(currentColor)
  ADD_METHOD(selectedColor)
  ADD_METHOD(setCurrentColor)

  // STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(getColor)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QColorDialogWrap, QColorDialog)
	
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QColorDialogWrap, QColorDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QColorDialog"), constructor);
}

Handle<Value> QColorDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QColorDialogWrap* w = new QColorDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QColorDialogWrap::NewInstance(QColorDialog *q) {//QColorDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QColorDialogWrap* w = node::ObjectWrap::Unwrap<QColorDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QColorDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QColorDialogWrap* w = ObjectWrap::Unwrap<QColorDialogWrap>(args.This());
	QImpl<QColorDialog>* q = (QImpl<QColorDialog>*)(w->GetWrapped()); 
	//TODO: check if really QColorDialogImpl!!!
	
  // signals
//  gQt.setWidgetCallback(pDialogID, #colorSelected, #colorSelected, me)
//  gQt.setWidgetCallback(pDialogID, #currentColorChanged, #currentColorChanged, me)
//  
//  -- inherited from QColorDialog
//  gQt.setWidgetCallback(pDialogID, #accepted, #accepted, me)
//  gQt.setWidgetCallback(pDialogID, #finished, #finished, me)
//  gQt.setWidgetCallback(pDialogID, #rejected, #rejected, me)
	
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
	else if (sign=="colorSelected")
		QObject::connect(q, SIGNAL(colorSelected(const QColor &)), rec, SLOT(evtC(const QColor &)));
	else if (sign=="currentColorChanged")
		QObject::connect(q, SIGNAL(currentColorChanged(const QColor &)), rec, SLOT(evtC(const QColor &)));
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QColorDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QColorDialogWrap* w = node::ObjectWrap::Unwrap<QColorDialogWrap>(args.This());
  
 	//QColorDialog* q = w->GetWrapped();
 	QImpl<QColorDialog>* q = (QImpl<QColorDialog>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		else if (sign=="colorSelected") 
			QObject::disconnect(q, SIGNAL(colorSelected(const QColor &)), rec, 0);	
		else if (sign=="currentColorChanged") 
			QObject::disconnect(q, SIGNAL(currentColorChanged(const QColor &)), rec, 0);	

		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

// currentColor () -> object QColor
Handle<Value> QColorDialogWrap::currentColor(const Arguments& args) {
  HandleScope scope;

  QColorDialogWrap* w = ObjectWrap::Unwrap<QColorDialogWrap>(args.This());
  QColorDialog* q = w->GetWrapped();

	QColor col = q->currentColor();
	return scope.Close(QColorWrap::NewInstance(col));
}

// selectedColor () -> object QColor
Handle<Value> QColorDialogWrap::selectedColor(const Arguments& args) {
  HandleScope scope;

  QColorDialogWrap* w = ObjectWrap::Unwrap<QColorDialogWrap>(args.This());
  QColorDialog* q = w->GetWrapped();
	
	QColor col = q->selectedColor();
	return scope.Close(QColorWrap::NewInstance(col));
}

// setCurrentColor (object QColor)
Handle<Value> QColorDialogWrap::setCurrentColor(const Arguments& args) {
  HandleScope scope;

  QColorDialogWrap* w = ObjectWrap::Unwrap<QColorDialogWrap>(args.This());
  QColorDialog* q = w->GetWrapped();
  
  QColorWrap* color_wrap = ObjectWrap::Unwrap<QColorWrap>(args[0]->ToObject());  
	q->setCurrentColor(*color_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}


/*************************************************
 STATIC FUNCTIONS
*************************************************/

// NEW: getColor () -> object QColor
Handle<Value> QColorDialogWrap::getColor(const Arguments& args) {
	HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	QColor col = QColorDialog::getColor(Qt::white, win);
	if (col.isValid()){
		return scope.Close(QColorWrap::NewInstance(col));
	}
	return scope.Close(Undefined());
}
