#include "qdialog.h"
#include "../qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDialog>;

/*************************************************
 QDialogWrap
*************************************************/

Persistent<Function> QDialogWrap::constructor;

QDialogWrap::QDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QDialog>(q_parent, fixScope);
	else
  	q_ = new QDialog(q_parent);
}

QDialogWrap::~QDialogWrap() {
  //delete q_;
}

void QDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
//  ADD_METHOD(result)
//  ADD_METHOD(setModal)
//  ADD_METHOD(setResult)
//  ADD_METHOD(setSizeGripEnabled)
//  ADD_METHOD(exec)
//  ADD_METHOD(open)

  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QDialogWrap, QDialog)

	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QDialogWrap, QDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDialog"), constructor);
}

Handle<Value> QDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QDialogWrap* w = new QDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDialogWrap::NewInstance(QDialog *q) {//QDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDialogWrap* w = node::ObjectWrap::Unwrap<QDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QDialogWrap* w = ObjectWrap::Unwrap<QDialogWrap>(args.This());
	QImpl<QDialog>* q = (QImpl<QDialog>*)(w->GetWrapped()); //TODO: check if really QDialogImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QDialogWrap* w = node::ObjectWrap::Unwrap<QDialogWrap>(args.This());
  
 	//QDialog* q = w->GetWrapped();
 	QImpl<QDialog>* q = (QImpl<QDialog>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

