
#include "qfiledialog.h"

#include "../qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QFileDialog>;

/*************************************************
 QFileDialogWrap
*************************************************/

Persistent<Function> QFileDialogWrap::constructor;

QFileDialogWrap::QFileDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QFileDialog>(q_parent, fixScope);
	else
  	q_ = new QFileDialog(q_parent);
}

QFileDialogWrap::~QFileDialogWrap() {
  //delete q_;
}

void QFileDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QFileDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // OWN METHODS
	ADD_METHOD(setAcceptMode)
	ADD_METHOD(setDefaultSuffix)
	ADD_METHOD(setDirectory)
	ADD_METHOD(setFileMode)
	ADD_METHOD(setFilter)
	ADD_METHOD(setLabelText)
	ADD_METHOD(setNameFilters)
	ADD_METHOD(setOption)
	ADD_METHOD(setSidebarUrls)
	ADD_METHOD(setViewMode)

  // STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(getOpenFileName)
  ADD_STATIC_METHOD(getSaveFileName)
  ADD_STATIC_METHOD(getExistingDirectory)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QFileDialogWrap, QFileDialog)
	
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QFileDialogWrap, QFileDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QFileDialog"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QFileDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = new QFileDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QFileDialogWrap::NewInstance(QFileDialog *q) {//QFileDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QFileDialogWrap* w = node::ObjectWrap::Unwrap<QFileDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QFileDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
	QImpl<QFileDialog>* q = (QImpl<QFileDialog>*)(w->GetWrapped()); //TODO: check if really QFileDialogImpl!!!
		
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QFileDialog
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
//- #currentChanged -> string path
//- #directoryEntered -> string directory
//- #fileSelected -> string file
//- #filesSelected -> list selectedFiles
//- #filterSelected -> string filter

	else if (sign=="currentChanged")
		QObject::connect(q, SIGNAL(currentChanged(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="directoryEntered")
		QObject::connect(q, SIGNAL(directoryEntered(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="fileSelected")
		QObject::connect(q, SIGNAL(fileSelected(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="filesSelected")
		QObject::connect(q, SIGNAL(filesSelected(const QStringList &)), rec, SLOT(evtSL(const QStringList &)));
	else if (sign=="filterSelected")
		QObject::connect(q, SIGNAL(filterSelected(const QString &)), rec, SLOT(evtS(const QString &)));
			
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QFileDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QFileDialogWrap* w = node::ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  
 	//QFileDialog* q = w->GetWrapped();
 	QImpl<QFileDialog>* q = (QImpl<QFileDialog>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);
			
		else if (sign=="currentChanged")
			QObject::disconnect(q, SIGNAL(currentChanged(const QString &)), rec, 0);
		else if (sign=="directoryEntered")
			QObject::disconnect(q, SIGNAL(directoryEntered(const QString &)), rec, 0);
		else if (sign=="fileSelected")
			QObject::disconnect(q, SIGNAL(fileSelected(const QString &)), rec, 0);
		else if (sign=="filesSelected")
			QObject::disconnect(q, SIGNAL(filesSelected(const QStringList &)), rec, 0);
		else if (sign=="filterSelected")
			QObject::disconnect(q, SIGNAL(filterSelected(const QString &)), rec, 0);			
			
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

//- setAcceptMode (int)
//- setDefaultSuffix (string)
//- setDirectory (string)
//- setFileMode (int)
//- setFilter (int)
//- setLabelText (int, string)
//- setNameFilters (list)
//- setOption (int option, bool on)
//- setSidebarUrls (list of string|int)
//- setViewMode (int)

//- setAcceptMode (int)
Handle<Value> QFileDialogWrap::setAcceptMode(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setAcceptMode((QFileDialog::AcceptMode)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// setDefaultSuffix (string)
Handle<Value> QFileDialogWrap::setDefaultSuffix(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setDefaultSuffix(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// setDirectory (string)
Handle<Value> QFileDialogWrap::setDirectory(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setDirectory(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setFileMode (int)
Handle<Value> QFileDialogWrap::setFileMode(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setFileMode((QFileDialog::FileMode)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setFilter (int)
Handle<Value> QFileDialogWrap::setFilter(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setFilter((QDir::Filter)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// setLabelText (int, string)
Handle<Value> QFileDialogWrap::setLabelText(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();
				
	q->setLabelText((QFileDialog::DialogLabel)args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setNameFilters (array)
Handle<Value> QFileDialogWrap::setNameFilters(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	QStringList filters;
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++) {
    filters.append(
    	qt_v8::ToQString(obj->Get(i)->ToString())
    );
  }
	q->setNameFilters(filters);
  return scope.Close(Undefined());
}

//- setOption (int option, bool on)
Handle<Value> QFileDialogWrap::setOption(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setOption((QFileDialog::Option)args[0]->IntegerValue(), args[1]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setSidebarUrls (array of string|int)
Handle<Value> QFileDialogWrap::setSidebarUrls(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	QList<QUrl> urls;

  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++) {
		if (obj->Get(i)->IsString())
	    urls.append(
	    	qt_v8::ToQString(obj->Get(i)->ToString())
	    );
    else
	    urls.append(
	    	QUrl::fromLocalFile(
	    		QDesktopServices::storageLocation(
	    			(QDesktopServices::StandardLocation)(obj->Get(i)->IntegerValue())
	    		)
	    	)
	    );
  }
	q->setSidebarUrls(urls);
  return scope.Close(Undefined());
}

//- setViewMode (int)
Handle<Value> QFileDialogWrap::setViewMode(const Arguments& args) {
  HandleScope scope;

  QFileDialogWrap* w = ObjectWrap::Unwrap<QFileDialogWrap>(args.This());
  QFileDialog* q = w->GetWrapped();

	q->setViewMode((QFileDialog::ViewMode)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

/*************************************************
 STATIC FUNCTIONS
*************************************************/

// NEW: getOpenFileName( caption, dir, filter, *options )
Handle<Value> QFileDialogWrap::getOpenFileName(const Arguments& args) {
	HandleScope scope;

	QWidget * win = 0; //args[0]->NumberValue()
	
//	int options = 0; // ???
//	if (args.Length() > 3 && args[3]->IsNumber()){
//		options = args[3]->IntegerValue();
//	}	
	
	// win, caption, dir, filter, 0, (QFileDialog::Options)options
	QString qstr = QFileDialog::getOpenFileName(
		win, 
		args.Length()>0?qt_v8::ToQString(args[0]->ToString()):QString(), 
		args.Length()>1?qt_v8::ToQString(args[1]->ToString()):QString(), 
		args.Length()>2?qt_v8::ToQString(args[2]->ToString()):QString(), 
		0, 
		args.Length()>3?(QFileDialog::Options)args[3]->IntegerValue():0
	);
	return scope.Close(qt_v8::FromQString(qstr));
}



// NEW: getSaveFileName( caption, dir, filter, *options )

	// QWidget * parent = 0, 
	// const QString & caption = QString(), 
	// const QString & dir = QString(), 
	// const QString & filter = QString(),
	
	// QString * selectedFilter = 0, 
	// Options options = 0
Handle<Value> QFileDialogWrap::getSaveFileName(const Arguments& args) {
	HandleScope scope;

	QWidget * win = 0; //args[0]->NumberValue()
	
//	int options = 0; // ???
//	if (args.Length() > 3 && args[3]->IsNumber()){
//		options = args[3]->IntegerValue();
//	}	
		
	// win, caption, dir, filter, 0, (QFileDialog::Options)options);
	QString qstr = QFileDialog::getSaveFileName (
		win, 
		args.Length()>0?qt_v8::ToQString(args[0]->ToString()):QString(), 
		args.Length()>1?qt_v8::ToQString(args[1]->ToString()):QString(), 
		args.Length()>2?qt_v8::ToQString(args[2]->ToString()):QString(), 
		0, 
		args.Length()>3?(QFileDialog::Options)args[3]->IntegerValue():0
	);
	return scope.Close(qt_v8::FromQString(qstr));
}

// NEW: getExistingDirectory (string caption, string dir, *int options) -> string
Handle<Value> QFileDialogWrap::getExistingDirectory(const Arguments& args) {
	HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	
//	int options = QFileDialog::ShowDirsOnly;
//	if (args.Length() > 2 && args[2]->IsNumber()){
//		options = args[2]->IntegerValue();
//	}	
	
	QString qstr = QFileDialog::getExistingDirectory (
		win, 
		args.Length()>0?qt_v8::ToQString(args[0]->ToString()):QString(), //caption
		args.Length()>1?qt_v8::ToQString(args[1]->ToString()):QString(), //dir
		args.Length()>2? (QFileDialog::Options)args[2]->IntegerValue() : QFileDialog::ShowDirsOnly
	);
	return scope.Close(qt_v8::FromQString(qstr));
}
