
#include "qfontdialog.h"

#include "../qwidgeteventstpl.h"
#include "../../Types/qfont.h"

using namespace v8;

template class QImpl<QFontDialog>;

/*************************************************
 QFontDialogWrap
*************************************************/

Persistent<Function> QFontDialogWrap::constructor;

QFontDialogWrap::QFontDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QFontDialog>(q_parent, fixScope);
	else
  	q_ = new QFontDialog(q_parent);
}

QFontDialogWrap::~QFontDialogWrap() {
  //delete q_;
}

void QFontDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QFontDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // OWN METHODS
  ADD_METHOD(setOption)

  // STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(getFont)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QFontDialogWrap, QFontDialog)
  
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QFontDialogWrap, QFontDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QFontDialog"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QFontDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QFontDialogWrap* w = new QFontDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QFontDialogWrap::NewInstance(QFontDialog *q) {//QFontDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QFontDialogWrap* w = ObjectWrap::Unwrap<QFontDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QFontDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QFontDialogWrap* w = ObjectWrap::Unwrap<QFontDialogWrap>(args.This());
	
	QImpl<QFontDialog>* q = (QImpl<QFontDialog>*)(w->GetWrapped()); //TODO: check if really QFontDialogImpl!!!
		
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QDialog
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
//- #currentFontChanged -> string fontDescription
//- #fontSelected -> string fontDescription
  
	else if (sign=="currentFontChanged")
		QObject::connect(q, SIGNAL(currentFontChanged(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="fontSelected")
		QObject::connect(q, SIGNAL(fontSelected(const QString &)), rec, SLOT(evtS(const QString &)));
			
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QFontDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QFontDialogWrap* w = ObjectWrap::Unwrap<QFontDialogWrap>(args.This());
  
 	//QFontDialog* q = w->GetWrapped();
 	QImpl<QFontDialog>* q = (QImpl<QFontDialog>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		else if (sign=="currentFontChanged")
			QObject::disconnect(q, SIGNAL(currentFontChanged(const QString &)), rec, 0);
		else if (sign=="fontSelected")
			QObject::disconnect(q, SIGNAL(fontSelected(const QString &)), rec, 0);
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

// - setOption (int option, bool on)
Handle<Value> QFontDialogWrap::setOption(const Arguments& args) {
  HandleScope scope;

  QFontDialogWrap* w = ObjectWrap::Unwrap<QFontDialogWrap>(args.This());
  QFontDialog* q = w->GetWrapped();

	q->setOption((QFontDialog::FontDialogOption)args[0]->IntegerValue(), args[1]->BooleanValue());
	
  return scope.Close(Undefined());
}

/*************************************************
 STATIC FUNCTIONS
*************************************************/

// NEW: getFont () -> object QFont
Handle<Value> QFontDialogWrap::getFont(const Arguments& args) {
	HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	bool ok;
	QFont font = QFontDialog::getFont(&ok, win);
	if (ok){
		return scope.Close(QFontWrap::NewInstance(font));
	}
	return scope.Close(Undefined());
}
