
#include "qinputdialog.h"

#include "../qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QInputDialog>;

/*************************************************
 QInputDialogWrap
*************************************************/

Persistent<Function> QInputDialogWrap::constructor;

QInputDialogWrap::QInputDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QInputDialog>(q_parent, fixScope);
	else
  	q_ = new QInputDialog(q_parent);
}

QInputDialogWrap::~QInputDialogWrap() {
  //delete q_;
}

void QInputDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QInputDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // OWN METHODS  
  ADD_METHOD(setCancelButtonText)
  ADD_METHOD(setLabelText)
  ADD_METHOD(setOkButtonText)
  ADD_METHOD(setOption)
  ADD_METHOD(setTextValue)
  
  // STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(getText)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QInputDialogWrap, QInputDialog)
	
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QInputDialogWrap, QInputDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QInputDialog"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QInputDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = new QInputDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QInputDialogWrap::NewInstance(QInputDialog *q) {//QInputDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QInputDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
	QImpl<QInputDialog>* q = (QImpl<QInputDialog>*)(w->GetWrapped()); //TODO: check if really QInputDialogImpl!!!
		
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QDialog
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
    
//- #textValueChanged -> string
//- #textValueSelected -> string

	else if (sign=="textValueChanged")
		QObject::connect(q, SIGNAL(textValueChanged(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="textValueSelected")
		QObject::connect(q, SIGNAL(textValueSelected(const QString &)), rec, SLOT(evtS(const QString &)));
			
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QInputDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
 	
 	//QInputDialog* q = w->GetWrapped();
 	QImpl<QInputDialog>* q = (QImpl<QInputDialog>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
//- #textValueChanged -> string
//- #textValueSelected -> string

		else if (sign=="textValueChanged")
			QObject::disconnect(q, SIGNAL(textValueChanged(const QString &)), rec, 0);
		else if (sign=="textValueSelected")
			QObject::disconnect(q, SIGNAL(textValueSelected(const QString &)), rec, 0);
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

//- setCancelButtonText (string)
Handle<Value> QInputDialogWrap::setCancelButtonText(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
  QInputDialog* q = w->GetWrapped();

	q->setCancelButtonText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setLabelText (string)
Handle<Value> QInputDialogWrap::setLabelText(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
  QInputDialog* q = w->GetWrapped();

	q->setLabelText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setOkButtonText (string)
Handle<Value> QInputDialogWrap::setOkButtonText(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
  QInputDialog* q = w->GetWrapped();

	q->setOkButtonText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// - setOption (int option, bool on)
Handle<Value> QInputDialogWrap::setOption(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
  QInputDialog* q = w->GetWrapped();

	q->setOption((QInputDialog::InputDialogOption)args[0]->IntegerValue(), args[1]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setTextValue (string)
Handle<Value> QInputDialogWrap::setTextValue(const Arguments& args) {
  HandleScope scope;

  QInputDialogWrap* w = ObjectWrap::Unwrap<QInputDialogWrap>(args.This());
  QInputDialog* q = w->GetWrapped();

	q->setTextValue(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

/*************************************************
 STATIC FUNCTIONS
*************************************************/

// NEW: getText (title, label, default text)
Handle<Value> QInputDialogWrap::getText(const Arguments& args) {
	HandleScope scope;
	// win, title, label,	QLineEdit::Normal, text, &ok
	bool ok;
	QString qstr = QInputDialog::getText(
		0, 
		qt_v8::ToQString(args[0]->ToString()),
		qt_v8::ToQString(args[1]->ToString()),
		QLineEdit::Normal, 
		qt_v8::ToQString(args[2]->ToString()),
		&ok
	);
	if (ok && !qstr.isEmpty()) return scope.Close(qt_v8::FromQString(qstr));
  return scope.Close(Undefined());
}
