#include "qmessagebox.h"

#include "../qwidgeteventstpl.h"
#include "../../Types/qimage.h"

using namespace v8;

template class QImpl<QMessageBox>;

/*************************************************
 QMessageBoxWrap
*************************************************/

Persistent<Function> QMessageBoxWrap::constructor;

QMessageBoxWrap::QMessageBoxWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QMessageBox>(q_parent, fixScope);
	else
  	q_ = new QMessageBox(q_parent);
}

QMessageBoxWrap::~QMessageBoxWrap() {
  //delete q_;
}

void QMessageBoxWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMessageBox"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // OWN METHODS
  ADD_METHOD(setDetailedText)
  ADD_METHOD(setIcon)
  ADD_METHOD(setStandardButtons)
  ADD_METHOD(setText)
  ADD_METHOD(setTextFormat)

  ADD_METHOD(setInformativeText)
  ADD_METHOD(setDefaultButton)
  ADD_METHOD(exec)

  ADD_METHOD(addButton)
  
  // C) STATIC FUNCTIONS (SYNC)
  ADD_STATIC_METHOD(about)
  ADD_STATIC_METHOD(aboutQt)
  ADD_STATIC_METHOD(critical)
  ADD_STATIC_METHOD(information)
  ADD_STATIC_METHOD(question)
  ADD_STATIC_METHOD(warning)
  
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QMessageBoxWrap, QMessageBox)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMessageBox"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QMessageBoxWrap::New(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = new QMessageBoxWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMessageBoxWrap::NewInstance(QMessageBox *q) {//QMessageBoxImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QMessageBoxWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
	QImpl<QMessageBox>* q = (QImpl<QMessageBox>*)(w->GetWrapped()); //TODO: check if really QMessageBoxImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QMessageBox
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QMessageBoxWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  
 	//QMessageBox* q = w->GetWrapped();
 	QImpl<QMessageBox>* q = (QImpl<QMessageBox>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

// setDetailedText (string)
Handle<Value> QMessageBoxWrap::setDetailedText(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setDetailedText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// setIcon (int icon)
Handle<Value> QMessageBoxWrap::setIcon(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();
  	
  q->setIcon((QMessageBox::Icon)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setStandardButtons (int)
Handle<Value> QMessageBoxWrap::setStandardButtons(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setStandardButtons((QMessageBox::StandardButtons)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// setText (string)
Handle<Value> QMessageBoxWrap::setText(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// setTextFormat (int)
Handle<Value> QMessageBoxWrap::setTextFormat(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setTextFormat((Qt::TextFormat)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}



// setInformativeText (string)
Handle<Value> QMessageBoxWrap::setInformativeText(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setInformativeText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

// setDefaultButton (int)
Handle<Value> QMessageBoxWrap::setDefaultButton(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	q->setDefaultButton((QMessageBox::StandardButton)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// exec () -> int
Handle<Value> QMessageBoxWrap::exec(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();

	int ret = q->exec();
	
  return scope.Close(Integer::New(ret));
}

// addButton (int)
Handle<Value> QMessageBoxWrap::addButton(const Arguments& args) {
  HandleScope scope;

  QMessageBoxWrap* w = ObjectWrap::Unwrap<QMessageBoxWrap>(args.This());
  QMessageBox* q = w->GetWrapped();
  
  if (args[0]->IsObject()){
	  QPushButtonWrap* btn_wrap = ObjectWrap::Unwrap<QPushButtonWrap>(args[0]->ToObject());
	  q->addButton(btn_wrap->GetWrapped(), (QMessageBox::ButtonRole)args[1]->IntegerValue());
	  return scope.Close(Undefined());
	  
	}else if (args[0]->IsString()){
  	QPushButton* p = q->addButton(qt_v8::ToQString(args[0]->ToString()), (QMessageBox::ButtonRole)args[1]->IntegerValue());  
  	return scope.Close(QPushButtonWrap::NewInstance(p));
  	
  }else{
  	QPushButton* p = q->addButton((QMessageBox::StandardButton)args[0]->IntegerValue());
  	return scope.Close(QPushButtonWrap::NewInstance(p));
	}

  return scope.Close(Undefined());
}


/*************************************************
 STATIC FUNCTIONS
*************************************************/

Handle<Value> QMessageBoxWrap::about(const Arguments& args) {
  HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	QMessageBox::about(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()));
  return scope.Close(Undefined());
}
	
Handle<Value> QMessageBoxWrap::aboutQt(const Arguments& args) {
  HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	QMessageBox::aboutQt(win, qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}


Handle<Value> QMessageBoxWrap::critical(const Arguments& args) {
  HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	long ret = (long)QMessageBox::critical(
		win, 
		qt_v8::ToQString(args[0]->ToString()),
		qt_v8::ToQString(args[1]->ToString()),
		args.Length()>2?(QMessageBox::StandardButtons)args[2]->IntegerValue():0x00000400,
		args.Length()>3?(QMessageBox::StandardButton)args[3]->IntegerValue():0x00000000
	);
  return scope.Close(Integer::New(ret));
}

Handle<Value> QMessageBoxWrap::information(const Arguments& args) {
  HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	long ret = (long)QMessageBox::information(
		win, 
		qt_v8::ToQString(args[0]->ToString()), 
		qt_v8::ToQString(args[1]->ToString()),
		args.Length()>2?(QMessageBox::StandardButtons)args[2]->IntegerValue():0x00000400,
		args.Length()>3?(QMessageBox::StandardButton)args[3]->IntegerValue():0x00000000
	);
  return scope.Close(Integer::New(ret));
}

// QWidget * parent, const QString & title, const QString & text, StandardButtons buttons = Ok, StandardButton defaultButton = NoButton
Handle<Value> QMessageBoxWrap::question(const Arguments& args) {
  HandleScope scope;
	QWidget * win = 0; //args[0]->NumberValue()
	long ret = (long)QMessageBox::question(
		win, 
		qt_v8::ToQString(args[0]->ToString()), 
		qt_v8::ToQString(args[1]->ToString()),
		//QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel,
		args.Length()>2?(QMessageBox::StandardButtons)args[2]->IntegerValue():0x00000400,
		args.Length()>3?(QMessageBox::StandardButton)args[3]->IntegerValue():0x00000000
	);
	
  return scope.Close(Integer::New(ret));
}

// QWidget * parent, const QString & title, const QString & text, StandardButtons buttons = Ok, StandardButton defaultButton = NoButto
Handle<Value> QMessageBoxWrap::warning(const Arguments& args) {
  HandleScope scope;
  QWidget * win = 0; //args[0]->NumberValue()
	long ret = (long)QMessageBox::warning(
		win, 
		qt_v8::ToQString(args[0]->ToString()), 
		qt_v8::ToQString(args[1]->ToString()),
		args.Length()>2?(QMessageBox::StandardButtons)args[2]->IntegerValue():0x00000400,
		args.Length()>3?(QMessageBox::StandardButton)args[3]->IntegerValue():0x00000000
	);
  return scope.Close(Integer::New(ret));
}





////QMessageBox::NoIcon	0	the message box does not have any icon.
////QMessageBox::Question	4	an icon indicating that the message is asking a question.
////QMessageBox::Information	1	an icon indicating that the message is nothing out of the ordinary.
////QMessageBox::Warning	2	an icon indicating that the message is a warning, but can be dealt with.
////QMessageBox::Critical	3	an icon indicating that the message represents a critical problem.
//
//// win, title, text[, type=info, buttons, defaultButton]
//
//
//// NEW: MessageBox( title, text[, type=info, buttons, defaultButton] )
//Handle<Value> QDialogWrap::MessageBox(const Arguments& args) {
//  HandleScope scope;
//  
////	if (args.Length() < 1) {
////	  return v8::ThrowException(v8::Exception::Error(v8::String::New("Illegal arguments")));
////	}
//
//  QWidget * win = 0;
//  
//	long type = 0; //kQtMessageBoxTypeInformation;
//	long buttons = 0x00000400; //Ok
//	long defaultButton = 0x00000000; // NoButton
//  int res = 0;
//  
////	if (callPtr->nargs>6){
////		GetArgByIndex(7, &moaValue);
////		pObj->pMmValue->ValueToInteger(&moaValue, &buttons);
////	}
////	if (callPtr->nargs>7){
////		GetArgByIndex(8, &moaValue);
////		pObj->pMmValue->ValueToInteger(&moaValue, &defaultButton);
////	}
//	
//	if (args.Length() > 2) type = args[2]->NumberValue();
//	if (args.Length() > 3) buttons = args[3]->NumberValue();
//	if (args.Length() > 4) defaultButton = args[4]->NumberValue();
//  
//	switch (type){
//	case 0: //kQtMessageBoxTypeInformation 
//		res = QMessageBox::information(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()), (QMessageBox::StandardButtons)buttons, (QMessageBox::StandardButton)defaultButton);
//		return scope.Close(Integer::New(res));
//		break;
//	case 1: //kQtMessageBoxTypeCritical
//		res = QMessageBox::critical(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()), (QMessageBox::StandardButtons)buttons, (QMessageBox::StandardButton)defaultButton);
//		return scope.Close(Integer::New(res));
//		break;
//	case 2: //kQtMessageBoxTypeQuestion
//		res = QMessageBox::question(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()), (QMessageBox::StandardButtons)buttons, (QMessageBox::StandardButton)defaultButton);
//		return scope.Close(Integer::New(res));
//		break;
//	case 3: //kQtMessageBoxTypeWarning
//		res = QMessageBox::warning(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()), (QMessageBox::StandardButtons)buttons, (QMessageBox::StandardButton)defaultButton);
//		return scope.Close(Integer::New(res));
//		break;
//	case 4: //kQtMessageBoxTypeAbout
//		QMessageBox::about(win, qt_v8::ToQString(args[0]->ToString()), qt_v8::ToQString(args[1]->ToString()));
//		break;
//	case 5: //kQtActionAboutQt
//		QMessageBox::aboutQt(win, qt_v8::ToQString(args[0]->ToString()));
//		break;
//	}
//  return scope.Close(Undefined());
//}