#include "qprintpreviewdialog.h"

//#include "../qwidgeteventstpl.h"
#include "../qwidgetevents_printer_tpl.h"

#include "../../Types/qprinter.h"

using namespace v8;

template class QImpl<QPrintPreviewDialog>;

/*************************************************
 QPrintPreviewDialogWrap
*************************************************/

Persistent<Function> QPrintPreviewDialogWrap::constructor;

QPrintPreviewDialogWrap::QPrintPreviewDialogWrap(const Arguments& args) : q_(NULL){

  QPrinter * printer = 0;
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	
	int n = 0;
  if (args.Length() > 0 && args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QPrinter") {
	  QPrinterWrap* printer_wrap = ObjectWrap::Unwrap<QPrinterWrap>(args[1]->ToObject());
	  printer = printer_wrap->GetWrapped();
	  n = 1;
  }
  
	if (args.Length()>n && args[n]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[n]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>n+1?args[n+1]->BooleanValue():false;
	bool fixScope = args.Length()>n+2?args[n+2]->BooleanValue():false;

	if (hasEvents)
		// was: q_ = new QImpl<QPageSetupDialog>(printer);
		q_ = printer ? new QImpl<QPrintPreviewDialog>(printer, q_parent, fixScope) : new QImpl<QPrintPreviewDialog>(q_parent, fixScope);
	else
  	q_ = printer ? new QPrintPreviewDialog(printer, q_parent) : new QPrintPreviewDialog(q_parent);
}

QPrintPreviewDialogWrap::~QPrintPreviewDialogWrap() {
  //delete q_;
}

void QPrintPreviewDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPrintPreviewDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // OWN METHODS
  //ADD_METHOD(setOption)

  // STATIC FUNCTIONS (SYNC)
  //ADD_STATIC_METHOD(getFont)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QPrintPreviewDialogWrap, QPrintPreviewDialog)
  
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QPrintPreviewDialogWrap, QPrintPreviewDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPrintPreviewDialog"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QPrintPreviewDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QPrintPreviewDialogWrap* w = new QPrintPreviewDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPrintPreviewDialogWrap::NewInstance(QPrintPreviewDialog *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPrintPreviewDialogWrap* w = ObjectWrap::Unwrap<QPrintPreviewDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QPrintPreviewDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QPrintPreviewDialogWrap* w = ObjectWrap::Unwrap<QPrintPreviewDialogWrap>(args.This());
	QImpl<QPrintPreviewDialog>* q = (QImpl<QPrintPreviewDialog>*)(w->GetWrapped()); //TODO: check if really QPrintPreviewDialogImpl!!!
		
	// SIGNALS
	
	Receiver* rec = new Receiver(callback, args.This());
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QDialog
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  		
	else if (sign=="paintRequested") 
		QObject::connect(q, SIGNAL(paintRequested(QPrinter *)), rec, SLOT(evtPr(QPrinter *)));	
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QPrintPreviewDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QPrintPreviewDialogWrap* w = ObjectWrap::Unwrap<QPrintPreviewDialogWrap>(args.This());
  
 	//QPrintPreviewDialog* q = w->GetWrapped();
 	QImpl<QPrintPreviewDialog>* q = (QImpl<QPrintPreviewDialog>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		else if (sign=="paintRequested") 
			QObject::disconnect(q, SIGNAL(paintRequested(QPrinter *)), rec, 0);	
			
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

// - setOption (int option, bool on)
//Handle<Value> QPrintPreviewDialogWrap::setOption(const Arguments& args) {
//  HandleScope scope;
//
//  QPrintPreviewDialogWrap* w = ObjectWrap::Unwrap<QPrintPreviewDialogWrap>(args.This());
//  QPrintPreviewDialog* q = w->GetWrapped();
//
//	q->setOption((QPrintPreviewDialog::FontDialogOption)args[0]->IntegerValue(), args[1]->BooleanValue());
//	
//  return scope.Close(Undefined());
//}

/*************************************************
 STATIC FUNCTIONS
*************************************************/

// parent
// printer, parent
// NEW: PageSetupDialog( *object printer )
//Handle<Value> QDialogWrap::PageSetupDialog(const Arguments& args) {
//	HandleScope scope;
//	QWidget * win = 0; //args[0]->NumberValue()	
//	QPrintPreviewDialog * pageSetupDialog;
//	
//	if (args.Length() > 0) {
//  	QPrinterWrap* printer_wrap = ObjectWrap::Unwrap<QPrinterWrap>(args[0]->ToObject());  
//		pageSetupDialog = new QPrintPreviewDialog(printer_wrap->GetWrapped(), win);
//	}else{
//		pageSetupDialog = new QPrintPreviewDialog(win);
//	}
//	
//	pageSetupDialog->open();
//	delete pageSetupDialog; //???
//			
//	return scope.Close(Undefined());
//}


//// parent
//// printer, parent
//// printer, parent, widget
//
//
////NEW: PrintPreviewDialog( printer=0, widget=0, flags )
//Handle<Value> QDialogWrap::PrintPreviewDialog(const Arguments& args) {
//	HandleScope scope;
//	
//	QWidget * win = 0; //args[0]->NumberValue()
//	QPrintPreviewDialog * printPreviewDialog;
//	int flags = 0;
//	
////	// parent?
////	if (!args[0]->IsNumber()){
////		// widget win
////	}
//	
//	// flags?
//	if (args.Length() > 2) {
//		flags = args[2]->IntegerValue();
//	}
//	
//	// printer?  args.Length() > 1 && 
//	if (args[0]->IsObject()){ //!args[1]->IsNumber()){
//  	QPrinterWrap* printer_wrap = ObjectWrap::Unwrap<QPrinterWrap>(args[0]->ToObject());  
//		printPreviewDialog = new QPrintPreviewDialog(printer_wrap->GetWrapped(), win);
//	}else{
//		printPreviewDialog = new QPrintPreviewDialog(win, (Qt::WindowFlags)flags);
//	}
//	
//	// widget?
//	if (args.Length() > 1 && !args[1]->IsNumber()){
//		QPlainTextEditWrap* edit_wrap = ObjectWrap::Unwrap<QPlainTextEditWrap>(args[1]->ToObject());
//		
//		QObject::connect(printPreviewDialog, SIGNAL(paintRequested(QPrinter*)), edit_wrap->GetWrapped(), SLOT(printPreview(QPrinter*)));
//		
//		//test
//		//QObject::connect(printPreviewDialog, SIGNAL(paintRequested(QPrinter*)), edit_wrap->GetWrapped(), SLOT(print(QPrinter*)));
//	}
//	
//	printPreviewDialog->exec();
//	delete printPreviewDialog; //???
//			
//	return scope.Close(Undefined());
//}