
#include "qprogressdialog.h"

#include "../qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QProgressDialog>;

/*************************************************
 QProgressDialogWrap
*************************************************/

Persistent<Function> QProgressDialogWrap::constructor;

QProgressDialogWrap::QProgressDialogWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QProgressDialog>(q_parent, fixScope);
	else
  	q_ = new QProgressDialog(q_parent);
}

QProgressDialogWrap::~QProgressDialogWrap() {
  //delete q_;
}

void QProgressDialogWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QProgressDialog"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
	// OWN METHODS
  ADD_METHOD(setAutoClose)
  ADD_METHOD(setAutoReset)
  ADD_METHOD(setCancelButtonText)
  ADD_METHOD(setLabelText)
  ADD_METHOD(setMaximum)
  ADD_METHOD(setMinimum)
  ADD_METHOD(setValue)
  
  // STATIC FUNCTIONS (SYNC)
  //ADD_STATIC_METHOD(getColor)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QProgressDialogWrap, QProgressDialog)
	
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QProgressDialogWrap, QProgressDialog)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QProgressDialog"), constructor);
}

Handle<Value> QProgressDialogWrap::New(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = new QProgressDialogWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QProgressDialogWrap::NewInstance(QProgressDialog *q) {//QProgressDialogImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QProgressDialogWrap* w = node::ObjectWrap::Unwrap<QProgressDialogWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QProgressDialogWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
	QImpl<QProgressDialog>* q = (QImpl<QProgressDialog>*)(w->GetWrapped()); //TODO: check if really QProgressDialogImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
	else if (sign=="canceled")
		QObject::connect(q, SIGNAL(canceled()), rec, SLOT(evtV()));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QProgressDialogWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QProgressDialogWrap* w = node::ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  
 	//QProgressDialog* q = w->GetWrapped();
 	QImpl<QProgressDialog>* q = (QImpl<QProgressDialog>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		else if (sign=="canceled") 
			QObject::disconnect(q, SIGNAL(canceled()), rec, 0);	

		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 CUSTOM INSTANCE METHODS
*************************************************/

//- setAutoClose (bool)
Handle<Value> QProgressDialogWrap::setAutoClose(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setAutoClose(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setAutoReset (bool)
Handle<Value> QProgressDialogWrap::setAutoReset(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setAutoReset(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setCancelButtonText (string)
Handle<Value> QProgressDialogWrap::setCancelButtonText(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setCancelButtonText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setLabelText (string)
Handle<Value> QProgressDialogWrap::setLabelText(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setLabelText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setMaximum (int)
Handle<Value> QProgressDialogWrap::setMaximum(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setMaximum(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setMinimum (int)
Handle<Value> QProgressDialogWrap::setMinimum(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setMinimum(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setValue (int)
Handle<Value> QProgressDialogWrap::setValue(const Arguments& args) {
  HandleScope scope;

  QProgressDialogWrap* w = ObjectWrap::Unwrap<QProgressDialogWrap>(args.This());
  QProgressDialog* q = w->GetWrapped();

	q->setValue(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

/*************************************************
 STATIC FUNCTIONS
*************************************************/
