#include "qwizard.h"
#include "../qwizardpage.h"
#include "../qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QWizard>;

/*************************************************
 QWizardWrap
*************************************************/

Persistent<Function> QWizardWrap::constructor;

QWizardWrap::QWizardWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QWizard>(q_parent, fixScope);
	else
  	q_ = new QWizard(q_parent);
}

QWizardWrap::~QWizardWrap() {
  //delete q_;
}

void QWizardWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QWizard"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)

  // OWN METHODS 
//- addPage (object QWizardPage) -> int
//- page (int id) -> object QWizardPage
//- removePage (int id)
  ADD_METHOD(addPage)
  ADD_METHOD(page)
  ADD_METHOD(removePage)
  
  // INHERITED FROM QDIALOG
	ADD_DIALOG_METHODS(QWizardWrap, QWizard)
	
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QWizardWrap, QWizard)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QWizard"), constructor);
}

/*************************************************
 CONSTRUCTOR
*************************************************/

Handle<Value> QWizardWrap::New(const Arguments& args) {
  HandleScope scope;

  QWizardWrap* w = new QWizardWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QWizardWrap::NewInstance(QWizard *q) {//QWizardImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QWizardWrap* w = node::ObjectWrap::Unwrap<QWizardWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

/*************************************************
 EVENTLISTENER METHODS ON/OFF
*************************************************/

Handle<Value> QWizardWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

	QWizardWrap* w = ObjectWrap::Unwrap<QWizardWrap>(args.This());
	QImpl<QWizard>* q = (QImpl<QWizard>*)(w->GetWrapped()); //TODO: check if really QWizardImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	
// inherited from QWidget
	if (sign=="customContextMenuRequested") //QWIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
// inherited from QWizard
	else if (sign=="accepted")
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evtV()));
	else if (sign=="finished")
		QObject::connect(q, SIGNAL(finished(int)), rec, SLOT(evtI(int)));
	else if (sign=="rejected ")
		QObject::connect(q, SIGNAL(rejected ()), rec, SLOT(evtV()));
  
	else if (sign=="currentIDChanged")
		QObject::connect(q, SIGNAL(currentIDChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="customButtonClicked")
		QObject::connect(q, SIGNAL(customButtonClicked(int)), rec, SLOT(evtI(int)));
	else if (sign=="helpRequested")
		QObject::connect(q, SIGNAL(helpRequested(int)), rec, SLOT(evtI(int)));
	else if (sign=="pageAdded")
		QObject::connect(q, SIGNAL(pageAdded(int)), rec, SLOT(evtI(int)));
	else if (sign=="pageRemoved")
		QObject::connect(q, SIGNAL(pageRemoved(int)), rec, SLOT(evtI(int)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off()
// off(event)
// off(event, function) TODO
Handle<Value> QWizardWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QWizardWrap* w = node::ObjectWrap::Unwrap<QWizardWrap>(args.This());
  
 	//QWizard* q = w->GetWrapped();
 	QImpl<QWizard>* q = (QImpl<QWizard>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="accepted") 
			QObject::disconnect(q, SIGNAL(accepted()), rec, 0);	
		else if (sign=="finished") 
			QObject::disconnect(q, SIGNAL(finished(int)), rec, 0);	
		else if (sign=="rejected") 
			QObject::disconnect(q, SIGNAL(rejected()), rec, 0);	
			
		else if (sign=="currentIDChanged")
			QObject::disconnect(q, SIGNAL(currentIDChanged(int)), rec, 0);	
		else if (sign=="customButtonClicked")
			QObject::disconnect(q, SIGNAL(customButtonClicked(int)), rec, 0);	
		else if (sign=="helpRequested")
			QObject::disconnect(q, SIGNAL(helpRequested(int)), rec, 0);	
		else if (sign=="pageAdded")
			QObject::disconnect(q, SIGNAL(pageAdded(int)), rec, 0);	
		else if (sign=="pageRemoved")
			QObject::disconnect(q, SIGNAL(pageRemoved(int)), rec, 0);	

		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This(); // return action ?
}

/*************************************************
 INSTANCE METHODS
*************************************************/

// addPage (object QWizardPage) -> int
Handle<Value> QWizardWrap::addPage(const Arguments& args) {
  HandleScope scope;

  QWizardWrap* w = ObjectWrap::Unwrap<QWizardWrap>(args.This());
  QWizard* q = w->GetWrapped();
  
  QWizardPageWrap* page_wrap = ObjectWrap::Unwrap<QWizardPageWrap>(args[0]->ToObject());

	return scope.Close(Integer::New(q->addPage(page_wrap->GetWrapped())));
}

// page (int id) -> object QWizardPage
Handle<Value> QWizardWrap::page(const Arguments& args) {
  HandleScope scope;

  QWizardWrap* w = ObjectWrap::Unwrap<QWizardWrap>(args.This());
  QWizard* q = w->GetWrapped();
	
	QWizardPage * page = q->page(args[0]->IntegerValue());
	return scope.Close(QWizardPageWrap::NewInstance(page));
}

// removePage (int id)
Handle<Value> QWizardWrap::removePage(const Arguments& args) {
  HandleScope scope;

  QWizardWrap* w = ObjectWrap::Unwrap<QWizardWrap>(args.This());
  QWizard* q = w->GetWrapped();
	
	q->removePage(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}
