#include "qcalendarwidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QCalendarWidget>;

/*************************************************
 QCalendarWidgetWrap
*************************************************/

Persistent<Function> QCalendarWidgetWrap::constructor;

QCalendarWidgetWrap::QCalendarWidgetWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QCalendarWidget>(q_parent, fixScope);
	else
  	q_ = new QCalendarWidget(q_parent);
}

QCalendarWidgetWrap::~QCalendarWidgetWrap() {
  //delete q_;
}

void QCalendarWidgetWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QCalendarWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
	
	ADD_METHOD(selectedDate)
	ADD_METHOD(setSelectedDate)
	
	//geerbt
	ADD_WIDGET_METHODS(QCalendarWidgetWrap, QCalendarWidget)
	 
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QCalendarWidget"), constructor);
}

Handle<Value> QCalendarWidgetWrap::New(const Arguments& args) {
  HandleScope scope;
  QCalendarWidgetWrap* w = new QCalendarWidgetWrap(args);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QCalendarWidgetWrap::NewInstance(QCalendarWidget *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QCalendarWidgetWrap* w = node::ObjectWrap::Unwrap<QCalendarWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QCalendarWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QCalendarWidgetWrap* w = ObjectWrap::Unwrap<QCalendarWidgetWrap>(args.This());
  //QCalendarWidget* q = w->GetWrapped();
	QImpl<QCalendarWidget>* q = (QImpl<QCalendarWidget>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
		
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	

	else if (sign=="currentPageChanged") 
		QObject::connect(q, SIGNAL(currentPageChanged(int,int)), rec, SLOT(evtII(int,int)));	
	else if (sign=="selectionChanged") 
		QObject::connect(q, SIGNAL(selectionChanged()), rec, SLOT(evtV()));	
	else{
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
	}
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QCalendarWidgetWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QCalendarWidgetWrap* w = node::ObjectWrap::Unwrap<QCalendarWidgetWrap>(args.This());
  
 	//QCalendarWidget* q = w->GetWrapped();
	QImpl<QCalendarWidget>* q = (QImpl<QCalendarWidget>*)(w->GetWrapped()); 
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else{
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		else if (sign=="currentPageChanged") 
			QObject::disconnect(q, SIGNAL(currentPageChanged(int,int)), rec, 0);	
		else if (sign=="selectionChanged") 
			QObject::disconnect(q, SIGNAL(selectionChanged()), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This(); // return action ?
}

Handle<Value> QCalendarWidgetWrap::selectedDate(const Arguments& args) {
  HandleScope scope;
  
  QCalendarWidgetWrap* w = ObjectWrap::Unwrap<QCalendarWidgetWrap>(args.This());
  QCalendarWidget* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->selectedDate().toString("yyyyMMdd")));
}

Handle<Value> QCalendarWidgetWrap::setSelectedDate(const Arguments& args) {
  HandleScope scope;
  
  QCalendarWidgetWrap* w = ObjectWrap::Unwrap<QCalendarWidgetWrap>(args.This());
  QCalendarWidget* q = w->GetWrapped();
  
  q->setSelectedDate(QDate(args[0]->IntegerValue(), args[1]->IntegerValue(), args[2]->IntegerValue()));//int year, int month, int day
   
  return scope.Close(Undefined());
}
