#include "qcombobox.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QComboBox>;

/*************************************************
 QComboBoxWrap
*************************************************/

Persistent<Function> QComboBoxWrap::constructor;

QComboBoxWrap::QComboBoxWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QComboBox>(q_parent, fixScope);
	else
  	q_ = new QComboBox(q_parent);
}

QComboBoxWrap::~QComboBoxWrap() {
  //delete q_;
}

void QComboBoxWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QComboBox"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(addItem)
	ADD_METHOD(currentIndex)
	ADD_METHOD(currentText)
	ADD_METHOD(itemData)
	ADD_METHOD(itemText)
	ADD_METHOD(removeItem)
	ADD_METHOD(setCurrentIndex)
	ADD_METHOD(setEditable)
	ADD_METHOD(setEditText)
	ADD_METHOD(setItemData)
	ADD_METHOD(setItemText)
	
	ADD_METHOD(setModel)
	ADD_METHOD(count)
	ADD_METHOD(findText)
	ADD_METHOD(clear)
	
	//geerbt
	ADD_WIDGET_METHODS(QComboBoxWrap, QComboBox)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QComboBox"), constructor);
}

Handle<Value> QComboBoxWrap::New(const Arguments& args) {
  HandleScope scope;
  QComboBoxWrap* w = new QComboBoxWrap(args);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QComboBoxWrap::NewInstance(QComboBox *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QComboBoxWrap* w = node::ObjectWrap::Unwrap<QComboBoxWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QComboBoxWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  //QComboBox* q = w->GetWrapped();
  QImpl<QComboBox>* q = (QImpl<QComboBox>*)(w->GetWrapped()); 

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	

	else if (sign=="currentIndexChanged")
		QObject::connect(q, SIGNAL(currentIndexChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="activated")
		QObject::connect(q, SIGNAL(activated(int)), rec, SLOT(evtI(int)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QComboBoxWrap::off(const Arguments& args) {
	HandleScope scope;
  QComboBoxWrap* w = node::ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  
 	//QComboBox* q = w->GetWrapped();
 	QImpl<QComboBox>* q = (QImpl<QComboBox>*)(w->GetWrapped()); 
 		
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
			
		else if (sign=="currentIndexChanged") 
			QObject::disconnect(q, SIGNAL(currentIndexChanged(int)), rec, 0);	
		else if (sign=="activated") 
			QObject::disconnect(q, SIGNAL(currentIndexChanged(int)), rec, 0);	
			
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- currentIndex () -> int
//- currentText () -> string
//- itemData (int) -> string
//- itemText (int) -> string
//- removeItem (int)
//- setCurrentIndex (int)
//- setEditable (bool)
//- setEditText (string)
//- setItemData (int, string)
//- setItemText (int, string)


// addItem (string text, QVariant data)
// addItem (string|image icon, string text, QVariant data)
Handle<Value> QComboBoxWrap::addItem(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

	if (args.Length()>2){
	  QIcon ico;
	  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
			 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
	  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
		  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
		  QImage* img = img_wrap->GetWrapped();
			QPixmap pm = QPixmap::fromImage(*img);
			ico.addPixmap(pm);
		}
		//if (!ico.isNull()) q->setIcon(ico);
		
	  QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[2]->ToObject());
		q->addItem(
			ico,
			qt_v8::ToQString(args[1]->ToString()), 
			*vw->GetWrapped()
		);
	  
	}else if (args.Length()>1){
		QVariantWrap* vw = ObjectWrap::Unwrap<QVariantWrap>(args[1]->ToObject());
		q->addItem(
			qt_v8::ToQString(args[0]->ToString()), 
			*vw->GetWrapped()
		);
	}else{
		q->addItem(
			qt_v8::ToQString(args[0]->ToString())
		);
	}
	
  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::currentIndex(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(Integer::New(q->currentIndex()));
}

Handle<Value> QComboBoxWrap::currentText(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->currentText()));
}

Handle<Value> QComboBoxWrap::itemData(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->itemData(args[0]->IntegerValue()).toString()));
}

Handle<Value> QComboBoxWrap::itemText(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->itemText(args[0]->IntegerValue())));
}

Handle<Value> QComboBoxWrap::removeItem(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

	q->removeItem(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::setCurrentIndex(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

	q->setCurrentIndex(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::setEditable(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

	q->setEditable(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::setEditText(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  q->setEditText(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::setItemData(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  q->setItemData(args[0]->IntegerValue(), QVariant(qt_v8::ToQString(args[1]->ToString())));

  return scope.Close(Undefined());
}

Handle<Value> QComboBoxWrap::setItemText(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  q->setItemText(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));

  return scope.Close(Undefined());
}

// setModel ( QAbstractItemModel * )
Handle<Value> QComboBoxWrap::setModel(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();
	
	QFileSystemModelWrap* file_system_model_wrap = ObjectWrap::Unwrap<QFileSystemModelWrap>(args[0]->ToObject());
	q->setModel(file_system_model_wrap->GetWrapped());
					
  return scope.Close(Undefined());
}

// count () -> int
Handle<Value> QComboBoxWrap::count(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(Integer::New(q->count()));
}

// findText 
Handle<Value> QComboBoxWrap::findText(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  return scope.Close(Integer::New(q->findText(
  	qt_v8::ToQString(args[0]->ToString()),
  	(Qt::MatchFlags)args[1]->IntegerValue()
  )));
}

// clear ()
Handle<Value> QComboBoxWrap::clear(const Arguments& args) {
  HandleScope scope;

  QComboBoxWrap* w = ObjectWrap::Unwrap<QComboBoxWrap>(args.This());
  QComboBox* q = w->GetWrapped();

  q->clear();

  return scope.Close(Undefined());
}