#include "qdateedit.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDateEdit>;

/*************************************************
 QDateEditWrap
*************************************************/

Persistent<Function> QDateEditWrap::constructor;

QDateEditWrap::QDateEditWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QDateEdit>(q_parent, fixScope);
	else
  	q_ = new QDateEdit(q_parent);
}

QDateEditWrap::~QDateEditWrap() {
  //delete q_;
}

void QDateEditWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDateEdit"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(date)
	ADD_METHOD(setDate)
	  
	//geerbt
	ADD_WIDGET_METHODS(QDateEditWrap, QDateEdit)
  
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDateEdit"), constructor);
}

Handle<Value> QDateEditWrap::New(const Arguments& args) {
  HandleScope scope;

  QDateEditWrap* w = new QDateEditWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDateEditWrap::NewInstance(QDateEdit *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDateEditWrap* w = node::ObjectWrap::Unwrap<QDateEditWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDateEditWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDateEditWrap* w = ObjectWrap::Unwrap<QDateEditWrap>(args.This());
  //QDateEdit* q = w->GetWrapped();
	QImpl<QDateEdit>* q = (QImpl<QDateEdit>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
		
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
	else if (sign=="dateTimeChanged")
		QObject::connect(q, SIGNAL(dateTimeChanged(QDateTime)), rec, SLOT(evtDT(QDateTime)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDateEditWrap::off(const Arguments& args) {
	HandleScope scope;
  QDateEditWrap* w = node::ObjectWrap::Unwrap<QDateEditWrap>(args.This());
  
 	//QDateEdit* q = w->GetWrapped();
 	QImpl<QDateEdit>* q = (QImpl<QDateEdit>*)(w->GetWrapped()); 
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else{
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="dateTimeChanged") 
			QObject::disconnect(q, SIGNAL(dateTimeChanged(QDateTime)), rec, 0);	
			
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- date () -> string ("YYYYMMDD")
Handle<Value> QDateEditWrap::date(const Arguments& args) {
  HandleScope scope;

  QDateEditWrap* w = ObjectWrap::Unwrap<QDateEditWrap>(args.This());
  QDateEdit* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->date().toString("yyyyMMdd")));
}

//- setDate (int Y, int M, int D)
Handle<Value> QDateEditWrap::setDate(const Arguments& args) {
  HandleScope scope;

  QDateEditWrap* w = ObjectWrap::Unwrap<QDateEditWrap>(args.This());
  QDateEdit* q = w->GetWrapped();

	q->setDate(QDate(args[0]->IntegerValue(),args[1]->IntegerValue(),args[2]->IntegerValue()));
	
  return scope.Close(Undefined());
}
