
#include "qdatetimeedit.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDateTimeEdit>;

/*************************************************
 QDateTimeEditWrap
*************************************************/

Persistent<Function> QDateTimeEditWrap::constructor;

QDateTimeEditWrap::QDateTimeEditWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QDateTimeEdit>(q_parent, fixScope);
	else
  	q_ = new QDateTimeEdit(q_parent);
}

QDateTimeEditWrap::~QDateTimeEditWrap() {
  //delete q_;
}

void QDateTimeEditWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDateTimeEdit"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(dateTime)
  ADD_METHOD(setDateTime)
    
	//geerbt
	ADD_WIDGET_METHODS(QDateTimeEditWrap, QDateTimeEdit)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDateTimeEdit"), constructor);
}

Handle<Value> QDateTimeEditWrap::New(const Arguments& args) {
  HandleScope scope;

  QDateTimeEditWrap* w = new QDateTimeEditWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDateTimeEditWrap::NewInstance(QDateTimeEdit *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDateTimeEditWrap* w = node::ObjectWrap::Unwrap<QDateTimeEditWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDateTimeEditWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDateTimeEditWrap* w = ObjectWrap::Unwrap<QDateTimeEditWrap>(args.This());
  //QDateTimeEdit* q = w->GetWrapped();
	QImpl<QDateTimeEdit>* q = (QImpl<QDateTimeEdit>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
		
	else if (sign=="dateTimeChanged")
		QObject::connect(q, SIGNAL(dateTimeChanged(QDateTime)), rec, SLOT(evtDT(QDateTime)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDateTimeEditWrap::off(const Arguments& args) {
	HandleScope scope;
  QDateTimeEditWrap* w = node::ObjectWrap::Unwrap<QDateTimeEditWrap>(args.This());
  
 	//QDateTimeEdit* q = w->GetWrapped();
 	QImpl<QDateTimeEdit>* q = (QImpl<QDateTimeEdit>*)(w->GetWrapped()); 
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="dateTimeChanged") 
			QObject::disconnect(q, SIGNAL(dateTimeChanged(QDateTime)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- dateTime () -> string ("YYYYMMDD hh:mm:ss")
Handle<Value> QDateTimeEditWrap::dateTime(const Arguments& args) {
  HandleScope scope;

  QDateTimeEditWrap* w = ObjectWrap::Unwrap<QDateTimeEditWrap>(args.This());
  QDateTimeEdit* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->dateTime().toString("yyyyMMdd hh:mm:ss")));
}

//- setDateTime (int Y, int M, int D, [int h, int m, int s])
Handle<Value> QDateTimeEditWrap::setDateTime(const Arguments& args) {
  HandleScope scope;

  QDateTimeEditWrap* w = ObjectWrap::Unwrap<QDateTimeEditWrap>(args.This());
  QDateTimeEdit* q = w->GetWrapped();

	//((QDateTimeEdit*)w)->setDateTime(QDateTime(QDate(y, m, d), QTime(h, min, s)));
	q->setDateTime(QDateTime(
		QDate(args[0]->IntegerValue(),args[1]->IntegerValue(),args[2]->IntegerValue()),
		QTime(args[3]->IntegerValue(),args[4]->IntegerValue(),args[5]->IntegerValue())
	));
	
  return scope.Close(Undefined());
}
