
#include "qdial.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDateTimeEdit>;

/*************************************************
 QDialWrap
*************************************************/

Persistent<Function> QDialWrap::constructor;

QDialWrap::QDialWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QDial>(q_parent, fixScope);
	else
  	q_ = new QDial(q_parent);
}

QDialWrap::~QDialWrap() {
  //delete q_;
}

void QDialWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDial"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(setValue)
  ADD_METHOD(value)
  
	//geerbt
	ADD_WIDGET_METHODS(QDialWrap, QDial)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDial"), constructor);
}

Handle<Value> QDialWrap::New(const Arguments& args) {
  HandleScope scope;

  QDialWrap* w = new QDialWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDialWrap::NewInstance(QDial *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDialWrap* w = node::ObjectWrap::Unwrap<QDialWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDialWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDialWrap* w = ObjectWrap::Unwrap<QDialWrap>(args.This());
  //QDial* q = w->GetWrapped();
	QImpl<QDial>* q = (QImpl<QDial>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="valueChanged")
		QObject::connect(q, SIGNAL(valueChanged(int)), rec, SLOT(evtI(int)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDialWrap::off(const Arguments& args) {
	HandleScope scope;
  QDialWrap* w = node::ObjectWrap::Unwrap<QDialWrap>(args.This());
  
 	//QDial* q = w->GetWrapped();
	QImpl<QDial>* q = (QImpl<QDial>*)(w->GetWrapped()); 
 	 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="valueChanged") 
			QObject::disconnect(q, SIGNAL(valueChanged(int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

// setValue (int)
Handle<Value> QDialWrap::setValue(const Arguments& args) {
  HandleScope scope;

  QDialWrap* w = ObjectWrap::Unwrap<QDialWrap>(args.This());
  QDial* q = w->GetWrapped();

	q->setValue(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QDialWrap::value(const Arguments& args) {
  HandleScope scope;

  QDialWrap* w = ObjectWrap::Unwrap<QDialWrap>(args.This());
  QDial* q = w->GetWrapped();

  return scope.Close(Integer::New(q->value()));
}
