
#include "qdialogbuttonbox.h"
#include "qpushbutton.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDialogButtonBox>;

/*************************************************
 QDialogButtonBoxWrap
*************************************************/

Persistent<Function> QDialogButtonBoxWrap::constructor;

QDialogButtonBoxWrap::QDialogButtonBoxWrap(const Arguments& args) : q_(NULL){
	if (hasEvents)
		q_ = new QImpl<QDialogButtonBox>;
	else
  	q_ = new QDialogButtonBox;

}

QDialogButtonBoxWrap::~QDialogButtonBoxWrap() {
  //delete q_;
}

void QDialogButtonBoxWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDialogButtonBox"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
  ADD_METHOD(addButton)
  ADD_METHOD(button)
  
	//geerbt
	ADD_WIDGET_METHODS(QDialogButtonBoxWrap, QDialogButtonBox)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDialogButtonBox"), constructor);
}

Handle<Value> QDialogButtonBoxWrap::New(const Arguments& args) {
  HandleScope scope;
  
	QDialogButtonBoxWrap* w = new QDialogButtonBoxWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDialogButtonBoxWrap::NewInstance(QDialogButtonBox *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDialogButtonBoxWrap* w = node::ObjectWrap::Unwrap<QDialogButtonBoxWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDialogButtonBoxWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDialogButtonBoxWrap* w = ObjectWrap::Unwrap<QDialogButtonBoxWrap>(args.This());
  //QDialogButtonBox* q = w->GetWrapped();
	QImpl<QDialogButtonBox>* q = (QImpl<QDialogButtonBox>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), 0, 0);	

	else if (sign=="accepted") 
		QObject::connect(q, SIGNAL(accepted()), rec, SLOT(evt()));	
//	else if (sign=="clicked") 
//		QObject::connect(q, SIGNAL(clicked()), ec, SLOT(evtV()));	
	else if (sign=="helpRequested") 
		QObject::connect(q, SIGNAL(helpRequested()), rec, SLOT(evtV()));	
	else if (sign=="rejected") 
		QObject::connect(q, SIGNAL(rejected()), rec, SLOT(evt()));	
		
//void 	accepted ()
//void 	clicked ( QAbstractButton * button )
//void 	helpRequested ()
//void 	rejected ()

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDialogButtonBoxWrap::off(const Arguments& args) {
	HandleScope scope;
  QDialogButtonBoxWrap* w = node::ObjectWrap::Unwrap<QDialogButtonBoxWrap>(args.This());
  
 	//QDialogButtonBox* q = w->GetWrapped();
 	QImpl<QDialogButtonBox>* q = (QImpl<QDialogButtonBox>*)(w->GetWrapped());

	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This();
}

// button (int QDialogButtonBox::StandardButton) -> object QPushButton
Handle<Value> QDialogButtonBoxWrap::button(const Arguments& args) {
  HandleScope scope;

  QDialogButtonBoxWrap* w = ObjectWrap::Unwrap<QDialogButtonBoxWrap>(args.This());
  QDialogButtonBox* q = w->GetWrapped();
  
  QPushButton* btn = q->addButton( (QDialogButtonBox::StandardButton)args[0]->IntegerValue() );
	  
  return scope.Close(btn ? QPushButtonWrap::NewInstance(btn) : Undefined()); // ???
}

// addButton (int QDialogButtonBox::StandardButton) -> object QPushButton
Handle<Value> QDialogButtonBoxWrap::addButton(const Arguments& args) {
  HandleScope scope;

  QDialogButtonBoxWrap* w = ObjectWrap::Unwrap<QDialogButtonBoxWrap>(args.This());
  QDialogButtonBox* q = w->GetWrapped();
  
  QPushButton* btn;
  
  if (args.Length()>1){
  	btn = q->addButton( 
  		qt_v8::ToQString(args[0]->ToString()),
  		(QDialogButtonBox::ButtonRole)args[1]->IntegerValue()
  	);
  }else{
  	btn = q->addButton( (QDialogButtonBox::StandardButton)args[0]->IntegerValue() );
  }
	
  return scope.Close(btn ? QPushButtonWrap::NewInstance(btn) : Undefined()); // ???
}