#include "qdockwidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QDockWidget>;

/*************************************************
 QTimeEditWrap
*************************************************/

Persistent<Function> QDockWidgetWrap::constructor;

QDockWidgetWrap::QDockWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QDockWidget>(q_parent, fixScope);
	else
  	q_ = new QDockWidget(q_parent);
}

QDockWidgetWrap::~QDockWidgetWrap() {
  //delete q_;
}

void QDockWidgetWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QDockWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)

  ADD_METHOD(setAllowedAreas)
  ADD_METHOD(setFeatures)
  ADD_METHOD(setFloating)
  ADD_METHOD(setWidget)
  ADD_METHOD(setWindowTitle)
  ADD_METHOD(widget)
  
	//geerbt
	ADD_WIDGET_METHODS(QDockWidgetWrap, QDockWidget)
	
	//from QWidget
	//tpl->PrototypeTemplate()->Set(String::NewSymbol("setWindowTitle"), FunctionTemplate::New(SetWindowTitle)->GetFunction());
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QDockWidget"), constructor);
}

Handle<Value> QDockWidgetWrap::New(const Arguments& args) {
  HandleScope scope;

  QDockWidgetWrap* w = new QDockWidgetWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QDockWidgetWrap::NewInstance(QDockWidget *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QDockWidgetWrap* w = node::ObjectWrap::Unwrap<QDockWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QDockWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  //QDockWidget* q = w->GetWrapped();
	QImpl<QDockWidget>* q = (QImpl<QDockWidget>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS	
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		
	else if (sign=="topLevelChanged") 
		QObject::connect(q, SIGNAL(topLevelChanged(bool)), rec, SLOT(evtB(bool)));	
	else if (sign=="visibilityChanged") 
		QObject::connect(q, SIGNAL(visibilityChanged(bool)), rec, SLOT(evtB(bool)));	
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QDockWidgetWrap::off(const Arguments& args) {
	HandleScope scope;
  QDockWidgetWrap* w = node::ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  
 	//QDockWidget* q = w->GetWrapped();
	QImpl<QDockWidget>* q = (QImpl<QDockWidget>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="topLevelChanged") 
			QObject::disconnect(q, SIGNAL(topLevelChanged(bool)), rec, 0);	
		else if (sign=="visibilityChanged") 
			QObject::disconnect(q, SIGNAL(visibilityChanged(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

// setAllowedAreas
Handle<Value> QDockWidgetWrap::setAllowedAreas(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();

	q->setAllowedAreas((Qt::DockWidgetArea)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// setFeatures (int)
Handle<Value> QDockWidgetWrap::setFeatures(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();

	q->setFeatures((QDockWidget::DockWidgetFeature)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// setFloating (bool)
Handle<Value> QDockWidgetWrap::setFloating(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();

	q->setFloating(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

// setWidget (widget)
Handle<Value> QDockWidgetWrap::setWidget(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	q->setWidget(widget_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

// setWindowTitle
Handle<Value> QDockWidgetWrap::setWindowTitle(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();

	q->setWindowTitle( qt_v8::ToQString(args[0]->ToString()) );

  return scope.Close(Undefined());
}

// widget () -> widget
Handle<Value> QDockWidgetWrap::widget(const Arguments& args) {
  HandleScope scope;
  QDockWidgetWrap* w = ObjectWrap::Unwrap<QDockWidgetWrap>(args.This());
  QDockWidget* q = w->GetWrapped();
	
  QWidget* w2 = q->widget();

  return scope.Close(QWidgetWrap::NewInstance(w2));
}