
#include "qframe.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QFrame>;

/*************************************************
 QFrameWrap
*************************************************/

Persistent<Function> QFrameWrap::constructor;

QFrameWrap::QFrameWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QFrame>(q_parent, fixScope);
	else
  	q_ = new QFrame(q_parent);
}

QFrameWrap::~QFrameWrap() {
  //delete q_;
}

void QFrameWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QFrame"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(setFrameRect)
	ADD_METHOD(setFrameShadow)
	ADD_METHOD(setFrameShape)
	ADD_METHOD(setFrameStyle)
	ADD_METHOD(setLineWidth)
	ADD_METHOD(setMidLineWidth)
  
	//geerbt
	ADD_WIDGET_METHODS(QFrameWrap, QFrame)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QFrame"), constructor);
}

Handle<Value> QFrameWrap::New(const Arguments& args) {
  HandleScope scope;
  
	QFrameWrap* w = new QFrameWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QFrameWrap::NewInstance(QFrame *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QFrameWrap* w = node::ObjectWrap::Unwrap<QFrameWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QFrameWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  //QFrame* q = w->GetWrapped();
	QImpl<QFrame>* q = (QImpl<QFrame>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QFrameWrap::off(const Arguments& args) {
	HandleScope scope;
  QFrameWrap* w = node::ObjectWrap::Unwrap<QFrameWrap>(args.This());
  
 	//QFrame* q = w->GetWrapped();
	QImpl<QFrame>* q = (QImpl<QFrame>*)(w->GetWrapped()); 
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else{
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- setFrameRect (rect)
//- setFrameShadow (int shadow)
//- setFrameShape (int shape)
//- setFrameStyle (int style)
//- setLineWidth (int)
//- setMidLineWidth (int)

Handle<Value> QFrameWrap::setFrameRect(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setFrameRect(QRect(
	args[0]->IntegerValue(), // left
	args[1]->IntegerValue(), // top
	args[2]->IntegerValue(), // w
	args[3]->IntegerValue()  // h
	));

  return scope.Close(Undefined());
}

Handle<Value> QFrameWrap::setFrameShadow(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setFrameShadow((QFrame::Shadow)(args[0]->IntegerValue()));

  return scope.Close(Undefined());
}

Handle<Value> QFrameWrap::setFrameShape(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setFrameShape((QFrame::Shape)(args[0]->IntegerValue()));

  return scope.Close(Undefined());
}

Handle<Value> QFrameWrap::setFrameStyle(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setFrameStyle(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QFrameWrap::setLineWidth(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setLineWidth(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QFrameWrap::setMidLineWidth(const Arguments& args) {
  HandleScope scope;

  QFrameWrap* w = ObjectWrap::Unwrap<QFrameWrap>(args.This());
  QFrame* q = w->GetWrapped();

	q->setMidLineWidth(args[0]->IntegerValue());

  return scope.Close(Undefined());
}
