
#include "qlabel.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QLabel>;

/*************************************************
 QLabelWrap
*************************************************/

Persistent<Function> QLabelWrap::constructor;

QLabelWrap::QLabelWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QLabel>(q_parent, fixScope);
	else
  	q_ = new QLabel(q_parent);
}

QLabelWrap::~QLabelWrap() {
  //delete q_;
}

void QLabelWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QLabel"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(setAlignment)
	ADD_METHOD(setImage)
	ADD_METHOD(setIndent)
	ADD_METHOD(setMargin)
	ADD_METHOD(setOpenExternalLinks)
	ADD_METHOD(setText)
	ADD_METHOD(setTextFormat)
	ADD_METHOD(setWordWrap)
	ADD_METHOD(text)

	//geerbt
	ADD_WIDGET_METHODS(QLabelWrap, QLabel)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QLabel"), constructor);
}

Handle<Value> QLabelWrap::New(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = new QLabelWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QLabelWrap::NewInstance(QLabel *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QLabelWrap* w = node::ObjectWrap::Unwrap<QLabelWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QLabelWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  //QLabel* q = w->GetWrapped();
	QImpl<QLabel>* q = (QImpl<QLabel>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="linkActivated")
		QObject::connect(q, SIGNAL(linkActivated(const QString &)), rec, SLOT(evtS(const QString &)));
	else if (sign=="linkHovered")
		QObject::connect(q, SIGNAL(linkHovered(const QString &)), rec, SLOT(evtS(const QString &)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QLabelWrap::off(const Arguments& args) {
	HandleScope scope;
  QLabelWrap* w = node::ObjectWrap::Unwrap<QLabelWrap>(args.This());
  
 	//QLabel* q = w->GetWrapped();
	QImpl<QLabel>* q = (QImpl<QLabel>*)(w->GetWrapped()); 
	
	if (args.Length()==0){
		
		QObject::disconnect(q, 0, 0, 0);
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="linkActivated") 
			QObject::disconnect(q, SIGNAL(linkActivated(const QString &)), rec, 0);
		else if (sign=="linkHovered") 
			QObject::disconnect(q, SIGNAL(linkHovered(const QString &)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- setAlignment (int)
//- setImage (int objectID|image|filename)  NEW: STRING fn | BUFFER data ???
//- setIndent (int)
//- setMargin (int)
//- setOpenExternalLinks (bool)
//- setText (string)
//- setTextFormat (int)
//- text () -> string
//- setWordWrap (bool)

Handle<Value> QLabelWrap::setAlignment(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setAlignment((Qt::Alignment)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setImage(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

//	}else if (varType==kMoaMmValueType_String){
//		const char * fn;
//		pObj->pMmValue->ValueToStringPtr(&moaValue, &fn);
//		((QLabel*)w)->setPixmap(QPixmap::fromImage(QImage(fn)));
//		pObj->pMmValue->ValueReleaseStringPtr(&moaValue);
//	}else if (varType==kMoaMmValueType_Other){//image
//		MoaMmImageInfo imgInfo;
//		pObj->pMmImage->GetImageInfo(&moaValue, &imgInfo);
//		if (imgInfo.iTotalDepth!=32) RETURN_INT_AND_EXIT(0); // FOR NOW ONLY 32-BIT
//		QImage img(imgInfo.iWidth, imgInfo.iHeight, QImage::Format_ARGB32);
//		PMoaVoid pPixels;
//		pObj->pMmImage->LockPixels (&moaValue, &pPixels);
//		memcpy(img.bits(), (char *)pPixels, imgInfo.iWidth * imgInfo.iHeight * 4); //imgInfo.iTotalDepth/8;
//		pObj->pMmImage->UnlockPixels (&moaValue);
//		((QLabel*)w)->setPixmap( QPixmap::fromImage(img.mirrored()));

	q->setPixmap(QPixmap::fromImage(QImage(  qt_v8::ToQString(args[0]->ToString())  )));

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setIndent(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setIndent(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setMargin(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setMargin(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setOpenExternalLinks(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setOpenExternalLinks(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setText(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setText(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::setTextFormat(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setTextFormat((Qt::TextFormat)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QLabelWrap::text(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->text()));
}

Handle<Value> QLabelWrap::setWordWrap(const Arguments& args) {
  HandleScope scope;

  QLabelWrap* w = ObjectWrap::Unwrap<QLabelWrap>(args.This());
  QLabel* q = w->GetWrapped();

	q->setWordWrap(args[0]->BooleanValue());

  return scope.Close(Undefined());
}
