
#include "qlineedit.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QLineEdit>;

/*************************************************
 QLineEditWrap
*************************************************/

Persistent<Function> QLineEditWrap::constructor;

QLineEditWrap::QLineEditWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QLineEdit>(q_parent, fixScope);
	else
  	q_ = new QLineEdit(q_parent);
}

QLineEditWrap::~QLineEditWrap() {
  //delete q_;
}

void QLineEditWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QLineEdit"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)

  ADD_METHOD(setText)
  ADD_METHOD(text)

	ADD_METHOD(setEchoMode)

	//geerbt
	ADD_WIDGET_METHODS(QLineEditWrap, QLineEdit)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QLineEdit"), constructor);
}

Handle<Value> QLineEditWrap::New(const Arguments& args) {
  HandleScope scope;

  QLineEditWrap* w = new QLineEditWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QLineEditWrap::NewInstance(QLineEdit *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QLineEditWrap* w = node::ObjectWrap::Unwrap<QLineEditWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QLineEditWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QLineEditWrap* w = ObjectWrap::Unwrap<QLineEditWrap>(args.This());
  //QLineEdit* q = w->GetWrapped();
	QImpl<QLineEdit>* q = (QImpl<QLineEdit>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));

	else if (sign=="returnPressed")
		QObject::connect(q, SIGNAL(returnPressed()), rec, SLOT(evtV()));
	else if (sign=="cursorPositionChanged")
		QObject::connect(q, SIGNAL(cursorPositionChanged(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="selectionChanged")
		QObject::connect(q, SIGNAL(selectionChanged()), rec, SLOT(evtV()));
	else if (sign=="textEdited")
		QObject::connect(q, SIGNAL(textEdited(const QString &)), rec, SLOT(evtS(const QString &)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QLineEditWrap::off(const Arguments& args) {
	HandleScope scope;
  QLineEditWrap* w = node::ObjectWrap::Unwrap<QLineEditWrap>(args.This());
  
 	//QLineEdit* q = w->GetWrapped();
	QImpl<QLineEdit>* q = (QImpl<QLineEdit>*)(w->GetWrapped()); 
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);

		else if (sign=="returnPressed")
			QObject::disconnect(q, SIGNAL(returnPressed()), rec, 0);
		else if (sign=="cursorPositionChanged")
			QObject::disconnect(q, SIGNAL(cursorPositionChanged(int,int)), rec, 0);
		else if (sign=="selectionChanged")
			QObject::disconnect(q, SIGNAL(selectionChanged()), rec, 0);
		else if (sign=="textEdited")
			QObject::disconnect(q, SIGNAL(textEdited(const QString &)), rec, 0);
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QLineEditWrap::setText(const Arguments& args) {
  HandleScope scope;

  QLineEditWrap* w = ObjectWrap::Unwrap<QLineEditWrap>(args.This());
  QLineEdit* q = w->GetWrapped();

  q->setText(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QLineEditWrap::text(const Arguments& args) {
  HandleScope scope;

  QLineEditWrap* w = ObjectWrap::Unwrap<QLineEditWrap>(args.This());
  QLineEdit* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->text()));
}

Handle<Value> QLineEditWrap::setEchoMode(const Arguments& args) {
  HandleScope scope;

  QLineEditWrap* w = ObjectWrap::Unwrap<QLineEditWrap>(args.This());
  QLineEdit* q = w->GetWrapped();

  q->setEchoMode((QLineEdit::EchoMode) args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}
