#include "qlistwidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QListWidget>;

/*************************************************
 QListWidgetWrap
*************************************************/

Persistent<Function> QListWidgetWrap::constructor;

QListWidgetWrap::QListWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QListWidget>(q_parent, fixScope);
	else
  	q_ = new QListWidget(q_parent);
}

QListWidgetWrap::~QListWidgetWrap() {
  //delete q_;
}

void QListWidgetWrap::Initialize(Handle<Object> target) {
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QListWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(addItem)
  ADD_METHOD(clear)
  ADD_METHOD(count)
  ADD_METHOD(currentRow)
  ADD_METHOD(insertItem)
  ADD_METHOD(isSortingEnabled)
  ADD_METHOD(item)
  ADD_METHOD(selectedItems)
  ADD_METHOD(setCurrentRow)
  ADD_METHOD(setSortingEnabled)
  ADD_METHOD(sortItems)
  
  ADD_METHOD(currentItem)
  ADD_METHOD(setCurrentItem)
  
  ADD_METHOD(findItems)
  ADD_METHOD(itemAt)
  
	//geerbt
	ADD_WIDGET_METHODS(QListWidgetWrap, QListWidget)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QListWidget"), constructor);
}

Handle<Value> QListWidgetWrap::New(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = new QListWidgetWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QListWidgetWrap::NewInstance(QListWidget *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QListWidgetWrap* w = node::ObjectWrap::Unwrap<QListWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QListWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  
  //QListWidget* q = w->GetWrapped();
  QImpl<QListWidget>* q = (QImpl<QListWidget>*)(w->GetWrapped()); //TODO: check if really QTreeWidgetImpl!!!
  
  Receiver* rec = new Receiver(callback, args.This());
  
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="currentRowChanged")
		QObject::connect(q, SIGNAL(currentRowChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="itemActivated")
		QObject::connect(q, SIGNAL(itemActivated(QListWidgetItem*)), rec, SLOT(evtL(QListWidgetItem*)));
	else if (sign=="itemChanged")
		QObject::connect(q, SIGNAL(itemChanged(QListWidgetItem*)), rec, SLOT(evtL(QListWidgetItem*)));
	else if (sign=="itemClicked")
		QObject::connect(q, SIGNAL(itemClicked(QListWidgetItem*)), rec, SLOT(evtL(QListWidgetItem*)));
	else if (sign=="itemDoubleClicked")
		QObject::connect(q, SIGNAL(itemDoubleClicked(QListWidgetItem*)), rec, SLOT(evtL(QListWidgetItem*)));
	else if (sign=="itemEntered")
		QObject::connect(q, SIGNAL(itemEntered(QListWidgetItem*,int)), rec, SLOT(evtL(QListWidgetItem*,int)));
	else if (sign=="itemPressed")
		QObject::connect(q, SIGNAL(itemPressed(QListWidgetItem*)), rec, SLOT(evtL(QListWidgetItem*)));
	else if (sign=="itemSelectionChanged")
		QObject::connect(q, SIGNAL(itemSelectionChanged()), rec, SLOT(evtV()));
  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QListWidgetWrap::off(const Arguments& args) {
	HandleScope scope;
  QListWidgetWrap* w = node::ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  
 	//QListWidget* q = w->GetWrapped();
	QImpl<QListWidget>* q = (QImpl<QListWidget>*)(w->GetWrapped()); 
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="currentRowChanged") 
			QObject::disconnect(q, SIGNAL(currentRowChanged(int)), rec, 0);
		else if (sign=="itemActivated") 
			QObject::disconnect(q, SIGNAL(itemActivated(QListWidgetItem*)), rec, 0);
		else if (sign=="itemChanged") 
			QObject::disconnect(q, SIGNAL(itemChanged(QListWidgetItem*)), rec, 0);
		else if (sign=="itemClicked") 
			QObject::disconnect(q, SIGNAL(itemClicked(QListWidgetItem*)), rec, 0);
		else if (sign=="itemDoubleClicked") 
			QObject::disconnect(q, SIGNAL(itemDoubleClicked(QListWidgetItem*)), rec, 0);
		else if (sign=="itemEntered") 
			QObject::disconnect(q, SIGNAL(itemEntered(QListWidgetItem*)), rec, 0);
		else if (sign=="itemPressed") 
			QObject::disconnect(q, SIGNAL(itemPressed(QListWidgetItem*)), rec, 0);
		else if (sign=="itemSelectionChanged") 
			QObject::disconnect(q, SIGNAL(itemSelectionChanged()), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- addItem (string)
//- clear ()
//- currentRow () -> int
//- insertItem (int, string)
//- item (int) -> string
		//- selectedItems () -> list
//- setCurrentRow (int)
//- setSortingEnabled (bool)
//- sortingEnabled () -> bool
//- sortItems ()

Handle<Value> QListWidgetWrap::addItem(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

  q->addItem(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QListWidgetWrap::clear(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

	q->clear();
	
  return scope.Close(Undefined());
}

Handle<Value> QListWidgetWrap::count(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

  return scope.Close(Integer::New(q->count()));
}

Handle<Value> QListWidgetWrap::currentRow(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

  return scope.Close(Integer::New(q->currentRow()));
}

Handle<Value> QListWidgetWrap::insertItem(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

  q->insertItem(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QListWidgetWrap::item(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

	QListWidgetItem *item = q->item(args[0]->IntegerValue());
	  
  return scope.Close(item ? QListWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

// selectedItems () -> []
Handle<Value> QListWidgetWrap::selectedItems(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
  
  QList<QListWidgetItem *> items = q->selectedItems();
  
  // create array
  Handle<Array> itemsArray = v8::Array::New(items.count());
  for (int i=0;i<items.count();i++){  	
  	itemsArray->Set(i, QListWidgetItemWrap::NewInstance(items[i]));
  }
  
  return scope.Close(itemsArray);
}

Handle<Value> QListWidgetWrap::setCurrentRow(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

	q->setCurrentRow(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QListWidgetWrap::setSortingEnabled(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();

	q->setSortingEnabled(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QListWidgetWrap::isSortingEnabled(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
	
  return scope.Close(Boolean::New(q->isSortingEnabled()));
}

Handle<Value> QListWidgetWrap::sortItems(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
	
	q->sortItems();
	
  return scope.Close(Undefined());
}

//- currentItem () -> QListWidgetItem *
Handle<Value> QListWidgetWrap::currentItem(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
  	
  QListWidgetItem *item = q->currentItem();
  
  return scope.Close(QListWidgetItemWrap::NewInstance(item));
}

// setCurrentItem ( QListWidgetItem * item[, QItemSelectionModel::SelectionFlags command] ) 
Handle<Value> QListWidgetWrap::setCurrentItem(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
	
	QListWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QListWidgetItemWrap>(args[0]->ToObject());
	
	if (args.Length()>1){
		q->setCurrentItem(item_wrap->GetWrapped(),(QItemSelectionModel::SelectionFlags)args[1]->IntegerValue());
	}else{
		q->setCurrentItem(item_wrap->GetWrapped());
	}
	
  return scope.Close(Undefined());
}

// findItems (str, flags) -> []
Handle<Value> QListWidgetWrap::findItems(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
  
  QList<QListWidgetItem *> items = q->findItems(qt_v8::ToQString(args[0]->ToString()), (Qt::MatchFlags)args[1]->IntegerValue());
  
  // create array
  Handle<Array> itemsArray = v8::Array::New(items.count());
  for (int i=0;i<items.count();i++){  	
  	itemsArray->Set(i, QListWidgetItemWrap::NewInstance(items[i]));
  }
  
  return scope.Close(itemsArray);
}

// itemAt (QPoint) -> QListWidgetItem *
// itemAt (int x, int y) -> QListWidgetItem *
Handle<Value> QListWidgetWrap::itemAt(const Arguments& args) {
  HandleScope scope;

  QListWidgetWrap* w = ObjectWrap::Unwrap<QListWidgetWrap>(args.This());
  QListWidget* q = w->GetWrapped();
  //QImpl<QListWidget>* q = (QImpl<QListWidget>*)(w->GetWrapped());
  
  QListWidgetItem *item;
  if (args[0]->IsObject()){
	  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
	  QPoint* point = point_wrap->GetWrapped();
	  item = q->itemAt(*point);
	}else{//if (args[0]->IsString()) { 
		item = q->itemAt(args[0]->IntegerValue(), args[1]->IntegerValue());
	}
  return scope.Close(item ? QListWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}