
#include "qmainwindow.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QMainWindow>;

/*************************************************
 QMainWindowWrap
*************************************************/

Persistent<Function> QMainWindowWrap::constructor;

QMainWindowWrap::QMainWindowWrap(const Arguments& args) : q_(NULL){
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QMainWindow>(q_parent, fixScope);
	else
  	q_ = new QMainWindow(q_parent);
}

QMainWindowWrap::~QMainWindowWrap() {
  //delete q_;
}

//static
void QMainWindowWrap::Initialize(Handle<Object> target) {
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMainWindow"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  //ADD_METHOD(listeners)
	//ADD_METHOD(addListener)
	
	ADD_METHOD(addDockWidget)
  ADD_METHOD(addToolBar)
  ADD_METHOD(setCentralWidget)
  ADD_METHOD(setMenuBar)
  ADD_METHOD(setStatusBar)
  
 	ADD_METHOD(menuBar)
 	ADD_METHOD(statusBar)
 	
 	//eigtl: QWidget!
 	ADD_METHOD(activateWindow)
 	
 	
  // geerbt
  ADD_WIDGET_METHODS(QMainWindowWrap, QMainWindow)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMainWindow"), constructor);
}

Handle<Value> QMainWindowWrap::New(const Arguments& args) {
  HandleScope scope;

  QMainWindowWrap* w = new QMainWindowWrap(args);

  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMainWindowWrap::NewInstance(QMainWindow *q) {//QMainWindowImpl
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}


///////////////////////////////////////
// OBJECT METHODS
///////////////////////////////////////

// kopie von qwidget
Handle<Value> QMainWindowWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
			
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
	QImpl<QMainWindow>* q = (QImpl<QMainWindow>*)(w->GetWrapped()); //TODO: check if really QMainWindowImpl!!!
	 
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off
Handle<Value> QMainWindowWrap::off(const Arguments& args) {
	HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  
 	//QMainWindow* q = w->GetWrapped();
	QImpl<QMainWindow>* q = (QImpl<QMainWindow>*)(w->GetWrapped()); //TODO: check if really QMainWindowImpl!!!
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else{
		
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested"){ //QWIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		// delete receivers?
	  }
	  
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}	
	
	
	//else HANDLE_WIDGET_EVENTS_OFF_BY_VALUE
	
//	if (args.Length()==2 && w->hasEvents){
//		QString sign = qt_v8::ToQString(args[0]->ToString());
//		Local<Function> cb = Local<Function>::Cast(args[1]);
//		Persistent<Function> cbp = Persistent<Function>::New(cb); // ???
//				
//		// Dispose?
//		QMultiHash<QString, v8::Persistent<v8::Value>>::iterator i = q->eventCallbacks.find(sign, cbp);
//		if (i != q->eventCallbacks.end()) i.value().Dispose();
//
// 		// remove
//		q->eventCallbacks.remove(sign, cbp);
//	
//	}

	return args.This(); // return action ?
}

// addDockWidget
Handle<Value> QMainWindowWrap::addDockWidget(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
  
  int area = args[0]->IntegerValue();
  QDockWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QDockWidgetWrap>(args[1]->ToObject());

	if (args.Length()>2){
		q->addDockWidget((Qt::DockWidgetArea)area, widget_wrap->GetWrapped(),(Qt::Orientation)args[2]->IntegerValue());
	}else{
		q->addDockWidget((Qt::DockWidgetArea)area, widget_wrap->GetWrapped());
	}
  return scope.Close(Undefined());
}

// addToolBar
Handle<Value> QMainWindowWrap::addToolBar(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
  
  if (args[0]->IsObject()){
	  QToolBarWrap* widget_wrap = ObjectWrap::Unwrap<QToolBarWrap>(args[0]->ToObject());
	  q->addToolBar(widget_wrap->GetWrapped());
	}else if (args[0]->IsString()){
	  QToolBar* t = q->addToolBar(qt_v8::ToQString(args[0]->ToString()));
  	return scope.Close(QToolBarWrap::NewInstance(t));
	}
  return scope.Close(Undefined());
}

Handle<Value> QMainWindowWrap::setCentralWidget(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
  
  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  q->setCentralWidget(widget_wrap->GetWrapped());
    
  return scope.Close(Undefined());
}

Handle<Value> QMainWindowWrap::setMenuBar(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
  
  QMenuBarWrap* widget_wrap = ObjectWrap::Unwrap<QMenuBarWrap>(args[0]->ToObject());
  q->setMenuBar(widget_wrap->GetWrapped());
  
  return scope.Close(Undefined());
}

Handle<Value> QMainWindowWrap::setStatusBar(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
 
  QStatusBarWrap* widget_wrap = ObjectWrap::Unwrap<QStatusBarWrap>(args[0]->ToObject());
  q->setStatusBar(widget_wrap->GetWrapped());
  
  return scope.Close(Undefined());
}

Handle<Value> QMainWindowWrap::menuBar(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
    
  QMenuBar* m = q->menuBar();

  return scope.Close(QMenuBarWrap::NewInstance(m));
}

Handle<Value> QMainWindowWrap::statusBar(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
    
  QStatusBar* s = q->statusBar();

  return scope.Close(QStatusBarWrap::NewInstance(s));
}

Handle<Value> QMainWindowWrap::activateWindow(const Arguments& args) {
  HandleScope scope;
  QMainWindowWrap* w = node::ObjectWrap::Unwrap<QMainWindowWrap>(args.This());
  QMainWindow* q = w->GetWrapped();
 
  q->activateWindow();
  
  return scope.Close(Undefined());
}

//aliases
//Handle<Value> QMainWindowWrap::addListener(const Arguments& args) {return on(args);}
//Handle<Value> QMainWindowWrap::removeListener(const Arguments& args) {
//	
//}
//Handle<Value> QMainWindowWrap::removeAllListeners(const Arguments& args) {
//	
//}

//template<class TW, class TI>
//Handle<Value> QWidgetTpl<TW,TI>::addListener(const Arguments& args) {
//  return on(args);
//}