#include "qmenu.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QMenu>;

/*************************************************
 QListWidgetWrap
*************************************************/

Persistent<Function> QMenuWrap::constructor;

QMenuWrap::QMenuWrap(const Arguments& args) : q_(NULL) {
//	if (args.Length() > 1) {
//  	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[1]->ToObject());
//		QWidget* widget = widget_wrap->GetWrapped();
//		if (hasEvents)
//			q_ = new QImpl<QMenu>;
//		else
//			q_ = new QMenu(widget); // TEST ONLY
//	}else{

	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
		if (hasEvents)
			q_ = new QImpl<QMenu>(q_parent, fixScope);
		else
	  	q_ = new QMenu(q_parent);
	  	
//	}

}

QMenuWrap::~QMenuWrap() {
  //delete q_;
}

void QMenuWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMenu"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(addAction)
  ADD_METHOD(addSeparator)
  ADD_METHOD(exec)
  
  // eigtl. von QWidget:
  ADD_METHOD(insertAction)
  ADD_METHOD(insertSeparator)
  ADD_METHOD(removeAction)
  
  ADD_METHOD(clear)
  
  ADD_METHOD(addMenu)
  ADD_METHOD(setIcon)
  
	//geerbt
	ADD_WIDGET_METHODS(QMenuWrap, QMenu)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMenu"), constructor);
}

Handle<Value> QMenuWrap::New(const Arguments& args) {
  HandleScope scope;

  
// 	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
//	QWidget* widget = widget_wrap->GetWrapped();

  QMenuWrap* w = new QMenuWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMenuWrap::NewInstance(QMenu *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMenuWrap* w = node::ObjectWrap::Unwrap<QMenuWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QMenuWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  //QMenu* q = w->GetWrapped();
	QImpl<QMenu>* q = (QImpl<QMenu>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="hovered") 
		QObject::connect(q, SIGNAL(hovered(QAction *)), rec, SLOT(evtA(QAction *)));	
	else if (sign=="triggered") 
		QObject::connect(q, SIGNAL(triggered(QAction *)), rec, SLOT(evtA(QAction *)));	

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QMenuWrap::off(const Arguments& args) {
	HandleScope scope;
  QMenuWrap* w = node::ObjectWrap::Unwrap<QMenuWrap>(args.This());
  
 	//QMenu* q = w->GetWrapped();
	QImpl<QMenu>* q = (QImpl<QMenu>*)(w->GetWrapped()); 
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		else if (sign=="hovered") 
			QObject::disconnect(q, SIGNAL(hovered(QAction *)), rec, 0);	
		else if (sign=="triggered") 
			QObject::disconnect(q, SIGNAL(triggered(QAction *)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

// addAction ( QAction * action )
// addAction ( QString text ) -> obj action
Handle<Value> QMenuWrap::addAction(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
  
  //QAction* a = q->addAction(qt_v8::ToQString(args[0]->ToString()));
  
  if (args[0]->IsObject()){
	  QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
	  q->addAction(action_wrap->GetWrapped());  
	}else if (args[0]->IsString()){
  	QAction* a = q->addAction(qt_v8::ToQString(args[0]->ToString()));  
  	return scope.Close(QActionWrap::NewInstance(a));
	}
	return scope.Close(Undefined());
}

Handle<Value> QMenuWrap::addSeparator(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
  
  QAction* a = q->addSeparator();
  
  return scope.Close(QActionWrap::NewInstance(a));
}

Handle<Value> QMenuWrap::exec(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
  
  if (args.Length()==0){
  	q->exec();
  }else{
  	//if (args[0]->IsObject())
	  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
	  QPoint* point = point_wrap->GetWrapped();
		if (args.Length() > 1){
  		//if (args[1]->IsObject())
	  	QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[1]->ToObject());
	  	q->exec(*point, action_wrap->GetWrapped());  
		}else{
  		q->exec(*point);
  	}
  }
  
//  if (args.Length() > 1)
//  	q->exec(QPoint(args[0]->IntegerValue(), args[1]->IntegerValue()));  	
//  else
//  	q->exec();
  	
  return scope.Close(Undefined());
}

// void QWidget::insertAction ( QAction * before, QAction * action )
Handle<Value> QMenuWrap::insertAction(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
    
  QActionWrap* before_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
  QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[1]->ToObject());
  
  q->insertAction(before_wrap->GetWrapped(), action_wrap->GetWrapped());
    
  return scope.Close(Undefined());
}

// void QWidget::insertSeparator ( QAction * before )
Handle<Value> QMenuWrap::insertSeparator(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
    
  QActionWrap* before_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
 
  q->insertSeparator(before_wrap->GetWrapped());
    
  return scope.Close(Undefined());
}

// void QWidget::removeAction ( QAction * action )
Handle<Value> QMenuWrap::removeAction(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
    
  QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
  
  q->removeAction(action_wrap->GetWrapped());
    
  return scope.Close(Undefined());
}

// clear ()
Handle<Value> QMenuWrap::clear(const Arguments& args) {
  HandleScope scope;
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
  q->clear();
  return scope.Close(Undefined());
}

Handle<Value> QMenuWrap::addMenu(const Arguments& args) {
  HandleScope scope;

  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();

  if (args[0]->IsObject()){
	  QMenuWrap* menu_wrap = ObjectWrap::Unwrap<QMenuWrap>(args[0]->ToObject());
	  QAction *a = q->addMenu(menu_wrap->GetWrapped());
	  return scope.Close(QActionWrap::NewInstance(a));
	  
	}else if (args[0]->IsString()){
  	QMenu *m = q->addMenu(qt_v8::ToQString(args[0]->ToString()));
  	return scope.Close(QMenuWrap::NewInstance(m));
	}

  return scope.Close(Undefined());
}

// setIcon (string|QImage)
Handle<Value> QMenuWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  
  QMenuWrap* w = ObjectWrap::Unwrap<QMenuWrap>(args.This());
  QMenu* q = w->GetWrapped();
    
  QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
		 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}
  if (!ico.isNull()) q->setIcon(ico);
  
  return scope.Close(Undefined());
}