
#include "qmenubar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QMainWindow>;

/*************************************************
 QMenuBarWrap
*************************************************/

Persistent<Function> QMenuBarWrap::constructor;

QMenuBarWrap::QMenuBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QMenuBar>(q_parent, fixScope);
	else
  	q_ = new QMenuBar(q_parent);
}

QMenuBarWrap::~QMenuBarWrap() {
  //delete q_;
}

void QMenuBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QMenuBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Prototype
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(addMenu)
  
  ADD_METHOD(addSeparator)
  
	//geerbt
	ADD_WIDGET_METHODS(QMenuBarWrap, QMenuBar)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QMenuBar"), constructor);
}

Handle<Value> QMenuBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QMenuBarWrap* w = new QMenuBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QMenuBarWrap::NewInstance(QMenuBar *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QMenuBarWrap* w = node::ObjectWrap::Unwrap<QMenuBarWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QMenuBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QMenuBarWrap* w = ObjectWrap::Unwrap<QMenuBarWrap>(args.This());
  //QMenuBar* q = w->GetWrapped();
	QImpl<QMenuBar>* q = (QImpl<QMenuBar>*)(w->GetWrapped()); 
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="hovered") 
		QObject::connect(q, SIGNAL(hovered(QAction *)), rec, SLOT(evtA(QAction *)));	
	else if (sign=="triggered") 
		QObject::connect(q, SIGNAL(triggered(QAction *)), rec, SLOT(evtA(QAction *)));	

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QMenuBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QMenuBarWrap* w = node::ObjectWrap::Unwrap<QMenuBarWrap>(args.This());
  
 	//QMenuBar* q = w->GetWrapped();
	QImpl<QMenuBar>* q = (QImpl<QMenuBar>*)(w->GetWrapped()); 
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="hovered") 
			QObject::disconnect(q, SIGNAL(hovered(QAction *)), rec, 0);	
		else if (sign=="triggered") 
			QObject::disconnect(q, SIGNAL(triggered(QAction *)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QMenuBarWrap::addMenu(const Arguments& args) {
  HandleScope scope;

  QMenuBarWrap* w = ObjectWrap::Unwrap<QMenuBarWrap>(args.This());
  QMenuBar* q = w->GetWrapped();

//  QMenu *m = q->addMenu(qt_v8::ToQString(args[0]->ToString()));
//  return scope.Close(QMenuWrap::NewInstance(m));

  if (args[0]->IsObject()){
	  QMenuWrap* menu_wrap = ObjectWrap::Unwrap<QMenuWrap>(args[0]->ToObject());
	  QAction *a = q->addMenu(menu_wrap->GetWrapped());
	  return scope.Close(QActionWrap::NewInstance(a));
	  
	}else if (args[0]->IsString()){
  	QMenu *m = q->addMenu(qt_v8::ToQString(args[0]->ToString()));
  	return scope.Close(QMenuWrap::NewInstance(m));
	}
	
  return scope.Close(Undefined());
}

Handle<Value> QMenuBarWrap::addSeparator(const Arguments& args) {
  HandleScope scope;
  
  QMenuBarWrap* w = ObjectWrap::Unwrap<QMenuBarWrap>(args.This());
  QMenuBar* q = w->GetWrapped();
  
  QAction* a = q->addSeparator();
  
  return scope.Close(QActionWrap::NewInstance(a));
}