
#include "qprogressbar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QMenu>;

/*************************************************
 QProgressBarWrap
*************************************************/

Persistent<Function> QProgressBarWrap::constructor;

QProgressBarWrap::QProgressBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QProgressBar>(q_parent, fixScope);
	else
  	q_ = new QProgressBar(q_parent);
}

QProgressBarWrap::~QProgressBarWrap() {
  //delete q_;
}

void QProgressBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QProgressBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(setFormat)
  ADD_METHOD(setMaximum)
  ADD_METHOD(setMinimum)
  ADD_METHOD(setValue)
  ADD_METHOD(text)
  ADD_METHOD(value)

//- setFormat (string)
//- setMaximum (int)
//- setMinimum (int)
//- setValue (int)
//- text () -> string
//- value () -> int

	//geerbt
	ADD_WIDGET_METHODS(QProgressBarWrap, QProgressBar)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QProgressBar"), constructor);
}

Handle<Value> QProgressBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = new QProgressBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QProgressBarWrap::NewInstance(QProgressBar *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QProgressBarWrap* w = node::ObjectWrap::Unwrap<QProgressBarWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QProgressBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  //QProgressBar* q = w->GetWrapped();
	QImpl<QProgressBar>* q = (QImpl<QProgressBar>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="valueChanged")
		QObject::connect(q, SIGNAL(valueChanged(int)), rec, SLOT(evtI(int)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QProgressBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QProgressBarWrap* w = node::ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  
 	//QProgressBar* q = w->GetWrapped();
	QImpl<QProgressBar>* q = (QImpl<QProgressBar>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="valueChanged") 
			QObject::disconnect(q, SIGNAL(valueChanged(int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}


//- setFormat (string)
//- setMaximum (int)
//- setMinimum (int)
//- setValue (int)
//- text () -> string

Handle<Value> QProgressBarWrap::setFormat(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  q->setFormat(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QProgressBarWrap::setMaximum(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  q->setMaximum(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QProgressBarWrap::setMinimum(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  q->setMinimum(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QProgressBarWrap::setValue(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  q->setValue(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QProgressBarWrap::text(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->text()));
}

Handle<Value> QProgressBarWrap::value(const Arguments& args) {
  HandleScope scope;

  QProgressBarWrap* w = ObjectWrap::Unwrap<QProgressBarWrap>(args.This());
  QProgressBar* q = w->GetWrapped();

  return scope.Close(Integer::New(q->value()));
}
