#include "qpushbutton.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QPushButton>;

/*************************************************
 QPushButtonWrap
*************************************************/

Persistent<Function> QPushButtonWrap::constructor;

QPushButtonWrap::QPushButtonWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QPushButton>(q_parent, fixScope);
	else
  	q_ = new QPushButton(q_parent);
}

QPushButtonWrap::~QPushButtonWrap() {
  //delete q_;
}

void QPushButtonWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QPushButton"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
    
  // eigtl. geerbt von QAbstractButton
  ADD_METHOD(setText)
  ADD_METHOD(text)
  //ADD_METHOD(setIcon)
  //setCheckable ( bool )
  //setChecked ( bool )
  //isChecked 
  //setShortcut ( const QKeySequence & key )
      
  ADD_METHOD(setDefault)
  
	//geerbt
	ADD_WIDGET_METHODS(QPushButtonWrap, QPushButton)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QPushButton"), constructor);  
}

Handle<Value> QPushButtonWrap::New(const Arguments& args) {
  HandleScope scope;

  QPushButtonWrap* w = new QPushButtonWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QPushButtonWrap::NewInstance(QPushButton *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QPushButtonWrap* w = node::ObjectWrap::Unwrap<QPushButtonWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QPushButtonWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QPushButtonWrap* w = ObjectWrap::Unwrap<QPushButtonWrap>(args.This());
  //QPushButton* q = w->GetWrapped();
	QImpl<QPushButton>* q = (QImpl<QPushButton>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS	
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="clicked")
		QObject::connect(q, SIGNAL(clicked(bool)), rec, SLOT(evtB(bool)));	
	else if (sign=="pressed")
		QObject::connect(q, SIGNAL(pressed()), rec, SLOT(evtV()));	
	else if (sign=="released")
		QObject::connect(q, SIGNAL(released()), rec, SLOT(evtV()));	
	else if (sign=="toggled")
		QObject::connect(q, SIGNAL(toggled(bool)), rec, SLOT(evtB(bool)));	
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QPushButtonWrap::off(const Arguments& args) {
	HandleScope scope;
  QPushButtonWrap* w = node::ObjectWrap::Unwrap<QPushButtonWrap>(args.This());
  
	//QPushButton* q = w->GetWrapped();
	QImpl<QPushButton>* q = (QImpl<QPushButton>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="clicked") 
			QObject::disconnect(q, SIGNAL(clicked(bool)), rec, 0);
		else if (sign=="pressed")
			QObject::disconnect(q, SIGNAL(pressed()), rec, 0);
		else if (sign=="released")
			QObject::disconnect(q, SIGNAL(released()), rec, 0);
		else if (sign=="toggled") 
			QObject::disconnect(q, SIGNAL(toggled(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
	
}

Handle<Value> QPushButtonWrap::setText(const Arguments& args) {
  HandleScope scope;

  QPushButtonWrap* w = ObjectWrap::Unwrap<QPushButtonWrap>(args.This());
  QPushButton* q = w->GetWrapped();

  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

Handle<Value> QPushButtonWrap::text(const Arguments& args) {
  HandleScope scope;

  QPushButtonWrap* w = ObjectWrap::Unwrap<QPushButtonWrap>(args.This());
  QPushButton* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text()));
}

// setDefault (bool)
Handle<Value> QPushButtonWrap::setDefault(const Arguments& args) {
  HandleScope scope;

  QPushButtonWrap* w = ObjectWrap::Unwrap<QPushButtonWrap>(args.This());
  QPushButton* q = w->GetWrapped();

  q->setDefault(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}
