
#include "qradiobutton.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QRadioButton>;

/*************************************************
 QRadioButtonWrap
*************************************************/

Persistent<Function> QRadioButtonWrap::constructor;

QRadioButtonWrap::QRadioButtonWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QRadioButton>(q_parent, fixScope);
	else
  	q_ = new QRadioButton(q_parent);
}

QRadioButtonWrap::~QRadioButtonWrap() {
  //delete q_;
}

void QRadioButtonWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QRadioButton"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(isChecked)
  ADD_METHOD(setChecked)
  ADD_METHOD(setText)
  ADD_METHOD(text)
  ADD_METHOD(toggle)
  
	//geerbt
	ADD_WIDGET_METHODS(QRadioButtonWrap, QRadioButton)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QRadioButton"), constructor);
}

Handle<Value> QRadioButtonWrap::New(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = new QRadioButtonWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QRadioButtonWrap::NewInstance(QRadioButton *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QRadioButtonWrap* w = node::ObjectWrap::Unwrap<QRadioButtonWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QRadioButtonWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  //QRadioButton* q = w->GetWrapped();
  QImpl<QRadioButton>* q = (QImpl<QRadioButton>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="clicked")
		QObject::connect(q, SIGNAL(clicked(bool)), rec, SLOT(evtB(bool)));
	else if (sign=="pressed")
		QObject::connect(q, SIGNAL(pressed()), rec, SLOT(evtV()));
	else if (sign=="released")
		QObject::connect(q, SIGNAL(released()), rec, SLOT(evtV()));
	else if (sign=="stateChanged")
		QObject::connect(q, SIGNAL(stateChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="toggled")
		QObject::connect(q, SIGNAL(toggled(bool)), rec, SLOT(evtB(bool)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QRadioButtonWrap::off(const Arguments& args) {
	HandleScope scope;
  QRadioButtonWrap* w = node::ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  
 	//QRadioButton* q = w->GetWrapped();
	QImpl<QRadioButton>* q = (QImpl<QRadioButton>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="clicked") 
			QObject::disconnect(q, SIGNAL(clicked(toggled)), rec, 0);
		else if (sign=="pressed")
			QObject::disconnect(q, SIGNAL(pressed()), rec, 0);
		else if (sign=="released")
			QObject::disconnect(q, SIGNAL(released()), rec, 0);
		else if (sign=="stateChanged")
			QObject::disconnect(q, SIGNAL(stateChanged(int)), rec, 0);
		else if (sign=="toggled") 
			QObject::disconnect(q, SIGNAL(toggled(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QRadioButtonWrap::isChecked(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  QRadioButton* q = w->GetWrapped();

  return scope.Close(Boolean::New(q->isChecked()));
}

Handle<Value> QRadioButtonWrap::setChecked(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  QRadioButton* q = w->GetWrapped();

  q->setChecked(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

Handle<Value> QRadioButtonWrap::setText(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  QRadioButton* q = w->GetWrapped();

  q->setText(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

Handle<Value> QRadioButtonWrap::text(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  QRadioButton* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->text()));
}

Handle<Value> QRadioButtonWrap::toggle(const Arguments& args) {
  HandleScope scope;

  QRadioButtonWrap* w = ObjectWrap::Unwrap<QRadioButtonWrap>(args.This());
  QRadioButton* q = w->GetWrapped();

  q->toggle();

  return scope.Close(Undefined());
}
