#include "qscrollarea.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QScrollArea>;

/*************************************************
 QScrollAreaWrap
*************************************************/

Persistent<Function> QScrollAreaWrap::constructor;

QScrollAreaWrap::QScrollAreaWrap(const v8::Arguments& args) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QScrollArea>(q_parent, fixScope);
	else
  	q_ = new QScrollArea(q_parent);
}

QScrollAreaWrap::~QScrollAreaWrap() {
  //delete q_;
}

void QScrollAreaWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QScrollArea"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  // QScrollArea-specific
  ADD_METHOD(horizontalScrollBar)
  ADD_METHOD(setFrameShape)
  ADD_METHOD(setHorizontalScrollBarPolicy)
  ADD_METHOD(setVerticalScrollBarPolicy)
  ADD_METHOD(setWidget)
  ADD_METHOD(verticalScrollBar)
  ADD_METHOD(widget)
  
	// INHERITED FROM QWIDGET
	ADD_WIDGET_METHODS(QScrollAreaWrap, QScrollArea)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QScrollArea"), constructor);
}

Handle<Value> QScrollAreaWrap::New(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = new QScrollAreaWrap(args);
  w->Wrap(args.This());

  return args.This();
}

// on (event, function)
Handle<Value> QScrollAreaWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
			
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QImpl<QScrollArea>* q = (QImpl<QScrollArea>*)(w->GetWrapped()); //TODO: check if really QImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested")
		
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}


//################################################
// NEW
// http://doc.qt.digia.com/qt/qobject.html#disconnect-2
//################################################
// off()
// off(event)
// off(event, function) TODO
Handle<Value> QScrollAreaWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  
 	//QScrollArea* q = w->GetWrapped();
 	QImpl<QScrollArea>* q = (QImpl<QScrollArea>*)(w->GetWrapped()); //TODO: check if really QImpl!!!
	
	if (args.Length()==0){
		
		// QObject * sender, char * signal, QObject * receiver, char * method
		QObject::disconnect(q, 0, 0, 0);

		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested")
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		HANDLE_WIDGET_EVENTS_OFF
				
	}
		
	return args.This(); // return action ?
}

Handle<Value> QScrollAreaWrap::setWidget(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  QString arg0_constructor;
  if (args[0]->IsObject()) {
    arg0_constructor = 
        qt_v8::ToQString(args[0]->ToObject()->GetConstructorName());
  }

  if (arg0_constructor != "QWidget")
    return ThrowException(Exception::TypeError(
      String::New("QScrollArea::SetWidget: bad argument")));

  // Unwrap obj
  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(
      args[0]->ToObject());
  QWidget* widget = widget_wrap->GetWrapped();

  q->setWidget(widget);

  return scope.Close(Undefined());
}

// QUIRK:
// Does not return QWidget. Returns 1 if child widget exists, 0 otherwise
Handle<Value> QScrollAreaWrap::widget(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  int retvalue = q->widget() ? 1 : 0;

  return scope.Close(Integer::New(retvalue));
}

Handle<Value> QScrollAreaWrap::setFrameShape(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  q->setFrameShape((QFrame::Shape)(args[0]->IntegerValue()));

  return scope.Close(Undefined());
}

Handle<Value> QScrollAreaWrap::setVerticalScrollBarPolicy(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  q->setVerticalScrollBarPolicy((Qt::ScrollBarPolicy)(args[0]->IntegerValue()));

  return scope.Close(Undefined());
}

Handle<Value> QScrollAreaWrap::setHorizontalScrollBarPolicy(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  q->setHorizontalScrollBarPolicy((Qt::ScrollBarPolicy)
      (args[0]->IntegerValue()));

  return scope.Close(Undefined());
}

Handle<Value> QScrollAreaWrap::horizontalScrollBar(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  return scope.Close( 
    QScrollBarWrap::NewInstance(q->horizontalScrollBar()) );
}

Handle<Value> QScrollAreaWrap::verticalScrollBar(const Arguments& args) {
  HandleScope scope;

  QScrollAreaWrap* w = node::ObjectWrap::Unwrap<QScrollAreaWrap>(args.This());
  QScrollArea* q = w->GetWrapped();

  return scope.Close( 
      QScrollBarWrap::NewInstance(q->verticalScrollBar()) );
}
