
#include "qscrollbar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QRadioButton>;

/*************************************************
 QScrollBarWrap
*************************************************/

Persistent<Function> QScrollBarWrap::constructor;

QScrollBarWrap::QScrollBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QScrollBar>(q_parent, fixScope);
	else
  	q_ = new QScrollBar(q_parent);
}

QScrollBarWrap::~QScrollBarWrap() {
  //delete q_;
}

void QScrollBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QScrollBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

//- setOrientation (int)
//- setRange (int min, int max)
//- setTracking (bool enable)
//- setValue (int)
//- value () -> int

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(setOrientation)
  ADD_METHOD(setRange)
  ADD_METHOD(setTracking)
  ADD_METHOD(setValue)
  ADD_METHOD(value)
    
	//geerbt
	ADD_WIDGET_METHODS(QScrollBarWrap, QScrollBar)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QScrollBar"), constructor);
}

Handle<Value> QScrollBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = new QScrollBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QScrollBarWrap::NewInstance(QScrollBar *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QScrollBarWrap* w = node::ObjectWrap::Unwrap<QScrollBarWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QScrollBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  //QScrollBar* q = w->GetWrapped();
	QImpl<QScrollBar>* q = (QImpl<QScrollBar>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="sliderMoved") 
		QObject::connect(q, SIGNAL(sliderMoved(int)), rec, SLOT(evtI(int)));	
	else if (sign=="valueChanged") 
		QObject::connect(q, SIGNAL(valueChanged(int)), rec, SLOT(evtI(int)));	

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QScrollBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QScrollBarWrap* w = node::ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  
 	//QScrollBar* q = w->GetWrapped();
	QImpl<QScrollBar>* q = (QImpl<QScrollBar>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="sliderMoved") 
			QObject::disconnect(q, SIGNAL(sliderMoved(int)), rec, 0);	
		else if (sign=="valueChanged") 
			QObject::disconnect(q, SIGNAL(valueChanged(int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- setOrientation (int)
//- setRange (int min, int max)
//- setTracking (bool enable)
//- setValue (int)
//- value () -> int

Handle<Value> QScrollBarWrap::setOrientation(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  QScrollBar* q = w->GetWrapped();

  q->setOrientation((Qt::Orientation)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QScrollBarWrap::setRange(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  QScrollBar* q = w->GetWrapped();

  q->setRange(args[0]->IntegerValue(),args[1]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QScrollBarWrap::setTracking(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  QScrollBar* q = w->GetWrapped();

  q->setTracking(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

Handle<Value> QScrollBarWrap::setValue(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  QScrollBar* q = w->GetWrapped();

  q->setValue(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QScrollBarWrap::value(const Arguments& args) {
  HandleScope scope;

  QScrollBarWrap* w = ObjectWrap::Unwrap<QScrollBarWrap>(args.This());
  QScrollBar* q = w->GetWrapped();

  return scope.Close(Integer::New(q->value()));
}
