
#include "qspinbox.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QRadioButton>;

/*************************************************
 QScrollBarWrap
*************************************************/

Persistent<Function> QSpinBoxWrap::constructor;

QSpinBoxWrap::QSpinBoxWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QSpinBox>(q_parent, fixScope);
	else
  	q_ = new QSpinBox(q_parent);
}

QSpinBoxWrap::~QSpinBoxWrap() {
  //delete q_;
}

void QSpinBoxWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QSpinBox"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

//- setRange (int min, int max)
//- setValue (int)
//- value () -> int

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(setRange)
  ADD_METHOD(setValue)
  ADD_METHOD(value)

	//geerbt
	ADD_WIDGET_METHODS(QSpinBoxWrap, QSpinBox)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QSpinBox"), constructor);
}

Handle<Value> QSpinBoxWrap::New(const Arguments& args) {
  HandleScope scope;

  QSpinBoxWrap* w = new QSpinBoxWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QSpinBoxWrap::NewInstance(QSpinBox *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QSpinBoxWrap* w = node::ObjectWrap::Unwrap<QSpinBoxWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QSpinBoxWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QSpinBoxWrap* w = ObjectWrap::Unwrap<QSpinBoxWrap>(args.This());
  //QSpinBox* q = w->GetWrapped();
	QImpl<QSpinBox>* q = (QImpl<QSpinBox>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="valueChanged")
		QObject::connect(q, SIGNAL(valueChanged(int)), rec, SLOT(evtI(int)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QSpinBoxWrap::off(const Arguments& args) {
	HandleScope scope;
  QSpinBoxWrap* w = node::ObjectWrap::Unwrap<QSpinBoxWrap>(args.This());
  
 	//QSpinBox* q = w->GetWrapped();
	QImpl<QSpinBox>* q = (QImpl<QSpinBox>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="valueChanged") 
			QObject::disconnect(q, SIGNAL(valueChanged(int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QSpinBoxWrap::setRange(const Arguments& args) {
  HandleScope scope;

  QSpinBoxWrap* w = ObjectWrap::Unwrap<QSpinBoxWrap>(args.This());
  QSpinBox* q = w->GetWrapped();

	q->setRange(args[0]->IntegerValue(), args[1]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QSpinBoxWrap::setValue(const Arguments& args) {
  HandleScope scope;

  QSpinBoxWrap* w = ObjectWrap::Unwrap<QSpinBoxWrap>(args.This());
  QSpinBox* q = w->GetWrapped();

	q->setValue(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

Handle<Value> QSpinBoxWrap::value(const Arguments& args) {
  HandleScope scope;

  QSpinBoxWrap* w = ObjectWrap::Unwrap<QSpinBoxWrap>(args.This());
  QSpinBox* q = w->GetWrapped();

  return scope.Close(Integer::New(q->value()));
}
