
#include "qsplitter.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QRadioButton>;

/*************************************************
 QSplitterWrap
*************************************************/

Persistent<Function> QSplitterWrap::constructor;

QSplitterWrap::QSplitterWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QSplitter>(q_parent, fixScope);
	else
  	q_ = new QSplitter(q_parent);
	
	
//  if (args[0]->IsNumber()) {
//    q_ = new QSplitter((Qt::Orientation)args[0]->IntegerValue());
//  } else {
//    q_ = new QSplitter();;
//  }
}

QSplitterWrap::~QSplitterWrap() {
  //delete q_;
}

void QSplitterWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QSplitter"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
	
	//addWidget ( QWidget * widget )
	//insertWidget ( int index, QWidget * widget )
	
	ADD_METHOD(addWidget)
	ADD_METHOD(insertWidget)
  
	ADD_METHOD(orientation)
	ADD_METHOD(setOrientation)
	
	ADD_METHOD(setSizes)
	
	ADD_METHOD(setCollapsible)
	ADD_METHOD(setHandleWidth)
	ADD_METHOD(setStretchFactor)
	
	//geerbt
	ADD_WIDGET_METHODS(QSplitterWrap, QSplitter)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QSplitter"), constructor);
}

Handle<Value> QSplitterWrap::New(const Arguments& args) {
  HandleScope scope;
  QSplitterWrap* w = new QSplitterWrap(args);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QSplitterWrap::NewInstance(QSplitter *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QSplitterWrap* w = node::ObjectWrap::Unwrap<QSplitterWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QSplitterWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  //QSplitter* q = w->GetWrapped();
	QImpl<QSplitter>* q = (QImpl<QSplitter>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="splitterMoved")
		QObject::connect(q, SIGNAL(splitterMoved(int,int)), rec, SLOT(evtII(int,int)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QSplitterWrap::off(const Arguments& args) {
	HandleScope scope;
  QSplitterWrap* w = node::ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  
 	//QSplitter* q = w->GetWrapped();
	QImpl<QSplitter>* q = (QImpl<QSplitter>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="splitterMoved") 
			QObject::disconnect(q, SIGNAL(splitterMoved(int,int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

// addWidget ( QWidget * widget )
Handle<Value> QSplitterWrap::addWidget(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	//ASSERT_OBJECT_TYPE(args[0], "QWidget")
	if (!args[0]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	
	q->addWidget(widget_wrap->GetWrapped());

  return scope.Close(Undefined());
}

// insertWidget ( int index, QWidget * widget )
Handle<Value> QSplitterWrap::insertWidget(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(1)
	if (!args[0]->IsNumber()||!args[1]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	
	q->insertWidget(args[0]->IntegerValue(), widget_wrap->GetWrapped());

  return scope.Close(Undefined());
}

// orientation () -> int
Handle<Value> QSplitterWrap::orientation(const Arguments& args) {
  HandleScope scope;
		
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	return scope.Close(Integer::New(q->orientation()));
}

// setOrientation (int orientation)
Handle<Value> QSplitterWrap::setOrientation(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(1)
	if (!args[0]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	q->setOrientation((Qt::Orientation)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// setSizes (array)
Handle<Value> QSplitterWrap::setSizes(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(1)
	if (!args[0]->IsArray()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	//JS ARRAY TO QLIST
	QList<int> widths;
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++) {
    widths.append(obj->Get(i)->IntegerValue());
    //DBGI(obj->Get(i)->IntegerValue());
  }

	q->setSizes(widths);

  return scope.Close(Undefined());
}

//setCollapsible ( int index, bool collapse )
Handle<Value> QSplitterWrap::setCollapsible(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(2)
	//if (!args[0]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	q->setCollapsible(args[0]->IntegerValue(), args[1]->BooleanValue());

  return scope.Close(Undefined());
}

// setHandleWidth (int)
Handle<Value> QSplitterWrap::setHandleWidth(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(1)
	if (!args[0]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	q->setHandleWidth(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// setStretchFactor ( int index, int stretch )
Handle<Value> QSplitterWrap::setStretchFactor(const Arguments& args) {
  HandleScope scope;
	
	ASSERT_NARGS(2)
	//if (!args[0]->IsNumber()) THROW_ERROR_BAD_TYPES
	
  QSplitterWrap* w = ObjectWrap::Unwrap<QSplitterWrap>(args.This());
  QSplitter* q = w->GetWrapped();
	
	q->setStretchFactor(args[0]->IntegerValue(), args[1]->IntegerValue());

  return scope.Close(Undefined());
}
