
#include "qstatusbar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QStatusBar>;

/*************************************************
 QStatusBarWrap
*************************************************/

Persistent<Function> QStatusBarWrap::constructor;

QStatusBarWrap::QStatusBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QStatusBar>(q_parent, fixScope);
	else
  	q_ = new QStatusBar(q_parent);
}

QStatusBarWrap::~QStatusBarWrap() {
  //delete q_;
}

void QStatusBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QStatusBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

//- isVisible () -> bool
//- setSizeGripEnabled (bool)
//- setVisible (bool)
//- showMessage (string, *int timeout)

	// Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(isVisible)
  ADD_METHOD(setSizeGripEnabled)
  ADD_METHOD(setVisible)
  ADD_METHOD(showMessage)
  
  ADD_METHOD(addWidget)
  ADD_METHOD(addPermanentWidget)
  
	//geerbt
	ADD_WIDGET_METHODS(QStatusBarWrap, QStatusBar)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QStatusBar"), constructor);
}

Handle<Value> QStatusBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QStatusBarWrap* w = new QStatusBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QStatusBarWrap::NewInstance(QStatusBar *q) {
  HandleScope scope;
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QStatusBarWrap* w = node::ObjectWrap::Unwrap<QStatusBarWrap>(instance);
  w->SetWrapped(q);
  return scope.Close(instance);
}

Handle<Value> QStatusBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  //QStatusBar* q = w->GetWrapped();
	QImpl<QStatusBar>* q = (QImpl<QStatusBar>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="messageChanged")
		QObject::connect(q, SIGNAL(messageChanged(const QString &)), rec, SLOT(evtS(const QString &)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QStatusBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QStatusBarWrap* w = node::ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  
 	//QStatusBar* q = w->GetWrapped();
	QImpl<QStatusBar>* q = (QImpl<QStatusBar>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="messageChanged") 
			QObject::disconnect(q, SIGNAL(messageChanged(const QString &)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}


//- isVisible () -> bool
//- setSizeGripEnabled (bool)
//- setVisible (bool)
//- showMessage (string, *int timeout)

Handle<Value> QStatusBarWrap::isVisible(const Arguments& args) {
  HandleScope scope;
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
  return scope.Close(Boolean::New(q->isVisible()));
}

Handle<Value> QStatusBarWrap::setSizeGripEnabled(const Arguments& args) {
  HandleScope scope;
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
  q->setSizeGripEnabled(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QStatusBarWrap::setVisible(const Arguments& args) {
  HandleScope scope;
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
  q->setVisible(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QStatusBarWrap::showMessage(const Arguments& args) {
  HandleScope scope;
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
  int timeout = 0;
  if (args.Length() > 1) timeout = args[1]->NumberValue();
  if (q) q->showMessage(qt_v8::ToQString(args[0]->ToString()), timeout);
  return scope.Close(Undefined());
}



// addWidget ( QWidget * widget, int stretch = 0 )
Handle<Value> QStatusBarWrap::addWidget(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	//ASSERT_OBJECT_TYPE(args[0], "QWidget")
	if (!args[0]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
	
	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  int stretch = 0;
  if (args.Length() > 1) stretch = args[1]->IntegerValue();
  
	q->addWidget(widget_wrap->GetWrapped(), stretch);

  return scope.Close(Undefined());
}

// addPermanentWidget ( QWidget * widget, int stretch = 0 )
Handle<Value> QStatusBarWrap::addPermanentWidget(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	//ASSERT_OBJECT_TYPE(args[0], "QWidget")
	if (!args[0]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QStatusBarWrap* w = ObjectWrap::Unwrap<QStatusBarWrap>(args.This());
  QStatusBar* q = w->GetWrapped();
	
	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  int stretch = 0;
  if (args.Length() > 1) stretch = args[1]->IntegerValue();
  
	q->addPermanentWidget(widget_wrap->GetWrapped(), stretch);

  return scope.Close(Undefined());
}