
#include "qtabbar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTabBar>;

/*************************************************
 QTabBarWrap
*************************************************/

Persistent<Function> QTabBarWrap::constructor;

QTabBarWrap::QTabBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTabBar>(q_parent, fixScope);
	else
  	q_ = new QTabBar(q_parent);
}

QTabBarWrap::~QTabBarWrap() {
  //delete q_;
}

void QTabBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTabBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
//- addTab (string text, *string|image icon) -> int
//- count () -> int
//- currentIndex () -> int
//- insertTab (int index, string text, *string|image icon) -> int
//- removeTab (int index)
//- setMovable (bool)
//- setShape (int)
//- setTabEnabled (int index, bool enabled)
//- setTabsClosable (bool)
//- setTabText (int index, string text)
//- setTabTextColor (int index, color)
//- setTabToolTip int index, string text)
//- setTabWhatsThis (int index, string text)
//- setUsesScrollButtons (bool)

	ADD_METHOD(addTab)
	ADD_METHOD(count)
	ADD_METHOD(currentIndex)
	ADD_METHOD(insertTab)
	ADD_METHOD(removeTab)
	ADD_METHOD(setMovable)
	ADD_METHOD(setShape)
	ADD_METHOD(setTabEnabled)
	ADD_METHOD(setTabsClosable)
	ADD_METHOD(setTabText)
	ADD_METHOD(setTabTextColor)
	ADD_METHOD(setTabToolTip)
	ADD_METHOD(setTabWhatsThis)
	ADD_METHOD(setUsesScrollButtons)

	//geerbt
	ADD_WIDGET_METHODS(QTabBarWrap, QTabBar)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTabBar"), constructor);
}

Handle<Value> QTabBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = new QTabBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTabBarWrap::NewInstance(QTabBar *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTabBarWrap* w = node::ObjectWrap::Unwrap<QTabBarWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTabBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  //QTabBar* q = w->GetWrapped();
	QImpl<QTabBar>* q = (QImpl<QTabBar>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="currentChanged")
		QObject::connect(q, SIGNAL(currentChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="tabCloseRequested")
		QObject::connect(q, SIGNAL(tabCloseRequested(int)), rec, SLOT(evtI(int)));
	else if (sign=="tabMoved")
		QObject::connect(q, SIGNAL(tabMoved(int,int)), rec, SLOT(evtII(int,int)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTabBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QTabBarWrap* w = node::ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  
 	//QTabBar* q = w->GetWrapped();
	QImpl<QTabBar>* q = (QImpl<QTabBar>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="currentChanged") 
			QObject::disconnect(q, SIGNAL(currentChanged(int)), rec, 0);	
		else if (sign=="tabCloseRequested") 
			QObject::disconnect(q, SIGNAL(tabCloseRequested(int)), rec, 0);	
		else if (sign=="tabMoved") 
			QObject::disconnect(q, SIGNAL(tabMoved(int,int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- addTab (string text, *string|image icon) -> int
Handle<Value> QTabBarWrap::addTab(const Arguments& args) {//TODO: icon: string | buffer
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	if (args.Length() > 1) {
		QIcon ico;
		ico.addFile( qt_v8::ToQString(args[1]->ToString()) );
  	q->addTab(ico, qt_v8::ToQString(args[0]->ToString()));
	}else{
		q->addTab(qt_v8::ToQString(args[0]->ToString()));
	}
	
  return scope.Close(Undefined());
}

//- count () -> int
Handle<Value> QTabBarWrap::count(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

  return scope.Close(Integer::New(q->count()));
}

//- currentIndex () -> int
Handle<Value> QTabBarWrap::currentIndex(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

  return scope.Close(Integer::New(q->currentIndex()));
}

//- insertTab (int index, string text, *string|image icon) -> int
Handle<Value> QTabBarWrap::insertTab(const Arguments& args) {//TODO: icon: string | buffer
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	if (args.Length() > 2) {
		QIcon ico;
		ico.addFile( qt_v8::ToQString(args[2]->ToString()) );
  	q->insertTab(
  		args[0]->IntegerValue(),
  		ico,
  		qt_v8::ToQString(args[1]->ToString()) 
  	);
	}else{
		q->insertTab(
			args[0]->IntegerValue(), 
			qt_v8::ToQString(args[1]->ToString())
		);
	}
	
  return scope.Close(Undefined());
}

//- removeTab (int index)
Handle<Value> QTabBarWrap::removeTab(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->removeTab(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// setMovable (bool)
Handle<Value> QTabBarWrap::setMovable(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setMovable(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setShape (int)
Handle<Value> QTabBarWrap::setShape(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setShape((QTabBar::Shape)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setTabEnabled (int index, bool enabled)
Handle<Value> QTabBarWrap::setTabEnabled(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabEnabled(args[0]->IntegerValue(), args[1]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setTabsClosable (bool)
Handle<Value> QTabBarWrap::setTabsClosable(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabsClosable(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setTabText (int index, string text)
Handle<Value> QTabBarWrap::setTabText(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabText(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setTabTextColor (int index, int r, int g, int b)
Handle<Value> QTabBarWrap::setTabTextColor(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabTextColor(args[0]->IntegerValue(), QColor(
		args[1]->IntegerValue(),
		args[2]->IntegerValue(),
		args[3]->IntegerValue()
	));
	
  return scope.Close(Undefined());
}

//- setTabToolTip int index, string text)
Handle<Value> QTabBarWrap::setTabToolTip(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabToolTip(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setTabWhatsThis (int index, string text)
Handle<Value> QTabBarWrap::setTabWhatsThis(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setTabWhatsThis(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setUsesScrollButtons (bool)
Handle<Value> QTabBarWrap::setUsesScrollButtons(const Arguments& args) {
  HandleScope scope;

  QTabBarWrap* w = ObjectWrap::Unwrap<QTabBarWrap>(args.This());
  QTabBar* q = w->GetWrapped();

	q->setUsesScrollButtons(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}
