
#include "qtableview.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTableView>;

/*************************************************
 QTableView
*************************************************/

Persistent<Function> QTableViewWrap::constructor;

QTableViewWrap::QTableViewWrap(const Arguments& args) : q_(NULL) {	
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTableView>(q_parent, fixScope);
	else
  	q_ = new QTableView(q_parent);
}

QTableViewWrap::~QTableViewWrap() {
  //delete q_;
}

void QTableViewWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTableView"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(setModel)
	ADD_METHOD(setRootIndex)
			
	ADD_METHOD(resizeColumnsToContents)
	ADD_METHOD(resizeRowsToContents)
	ADD_METHOD(scrollToBottom)
	ADD_METHOD(scrollToTop)

	ADD_METHOD(setAlternatingRowColors)
	ADD_METHOD(setColumnWidth)
	ADD_METHOD(setDragDropMode)
	ADD_METHOD(setEditTriggers)
	ADD_METHOD(setRowDefaultHeight)
	ADD_METHOD(setRowHeight)
	ADD_METHOD(setSelectionBehavior)
	ADD_METHOD(setSelectionMode)
	ADD_METHOD(setShowGrid)
	
	ADD_METHOD(setColumnHidden)
	ADD_METHOD(setVerticalHeaderVisible)
		
	ADD_METHOD(sortByColumn)
	ADD_METHOD(setSortingEnabled)
	
	ADD_METHOD(setHorizontalStretchLastSection)
	ADD_METHOD(setVerticalStretchLastSection)
	ADD_METHOD(setHorizontalHeaderDefaultAlignment)
	
	ADD_METHOD(setDragEnabled)
	
	ADD_METHOD(setCurrentIndex)

	ADD_METHOD(selectedIndexes)
	ADD_METHOD(setSelectedIndexes)
	ADD_METHOD(indexAt)
	
	//geerbt
	ADD_WIDGET_METHODS(QTableViewWrap, QTableView)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTableView"), constructor);
}

Handle<Value> QTableViewWrap::New(const Arguments& args) {
  HandleScope scope;
  QTableViewWrap* w = new QTableViewWrap(args);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QTableViewWrap::NewInstance(QTableView *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTableViewWrap* w = node::ObjectWrap::Unwrap<QTableViewWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// on
Handle<Value> QTableViewWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  //QTableView* q = w->GetWrapped();
  QImpl<QTableView>* q = (QImpl<QTableView>*)(w->GetWrapped()); //TODO: check if really QTableViewImpl!!!
  
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="activated")
		QObject::connect(q, SIGNAL(activated(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="clicked")
		QObject::connect(q, SIGNAL(clicked(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="doubleClicked")
		QObject::connect(q, SIGNAL(doubleClicked(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="entered")
		QObject::connect(q, SIGNAL(entered(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="pressed")
		QObject::connect(q, SIGNAL(pressed(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
				
	else if (sign=="currentChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));
	else if (sign=="currentColumnChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentColumnChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));
	else if (sign=="currentRowChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

// off
Handle<Value> QTableViewWrap::off(const Arguments& args) {
	HandleScope scope;
  QTableViewWrap* w = node::ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  
 	//QTableView* q = w->GetWrapped();
	QImpl<QTableView>* q = (QImpl<QTableView>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
			
		else if (sign=="activated")
			QObject::disconnect(q, SIGNAL(activated(const QModelIndex &)), rec, 0);
		else if (sign=="clicked")
			QObject::disconnect(q, SIGNAL(clicked(const QModelIndex &)), rec, 0);
		else if (sign=="doubleClicked")
			QObject::disconnect(q, SIGNAL(doubleClicked(const QModelIndex &)), rec, 0);
		else if (sign=="entered")
			QObject::disconnect(q, SIGNAL(entered(const QModelIndex &)), rec, 0);
		else if (sign=="pressed")
			QObject::disconnect(q, SIGNAL(pressed(const QModelIndex &)), rec, 0);
			
		else if (sign=="currentChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentChanged(const QModelIndex &)), rec, 0);
		else if (sign=="currentColumnChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentColumnChanged(const QModelIndex &)), rec, 0);
		else if (sign=="currentRowChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &)), rec, 0);
			
			
	  HANDLE_WIDGET_EVENTS_OFF
	  
		// delete receivers?
	}
	return args.This();
}

// setRootIndex (object QModelIndex)
Handle<Value> QTableViewWrap::setRootIndex(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
	
	QModelIndexWrap* item_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
		
	q->setRootIndex(*item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

// setModel ( QAbstractItemModel * )
Handle<Value> QTableViewWrap::setModel(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
	
	QFileSystemModelWrap* file_system_model_wrap = ObjectWrap::Unwrap<QFileSystemModelWrap>(args[0]->ToObject());
	q->setModel(file_system_model_wrap->GetWrapped());
					
  return scope.Close(Undefined());
}

// resizeColumnsToContents ()
Handle<Value> QTableViewWrap::resizeColumnsToContents(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->resizeColumnsToContents();
	return scope.Close(Undefined());
}

// resizeRowsToContents ()
Handle<Value> QTableViewWrap::resizeRowsToContents(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->resizeRowsToContents();
	return scope.Close(Undefined());
}

// scrollToBottom ()
Handle<Value> QTableViewWrap::scrollToBottom(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->scrollToBottom();

	return scope.Close(Undefined());
}

// scrollToTop ()
Handle<Value> QTableViewWrap::scrollToTop(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->scrollToTop();

	return scope.Close(Undefined());
}

// setAlternatingRowColors (bool)
Handle<Value> QTableViewWrap::setAlternatingRowColors(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setAlternatingRowColors(args[0]->BooleanValue());

	return scope.Close(Undefined());
}

// setColumnWidth (int col, int width)
Handle<Value> QTableViewWrap::setColumnWidth(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setColumnWidth(args[0]->IntegerValue(), args[1]->IntegerValue());
	return scope.Close(Undefined());
}

// setDragDropMode (int)
Handle<Value> QTableViewWrap::setDragDropMode(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setDragDropMode((QAbstractItemView::DragDropMode)args[0]->IntegerValue());
	return scope.Close(Undefined());
}

// setEditTriggers (int)
Handle<Value> QTableViewWrap::setEditTriggers(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setEditTriggers((QAbstractItemView::EditTriggers)args[0]->IntegerValue());
	return scope.Close(Undefined());
}

// setRowHeight (int row, int height)
Handle<Value> QTableViewWrap::setRowHeight(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setRowHeight(args[0]->IntegerValue(), args[1]->IntegerValue());

	return scope.Close(Undefined());
}

// setSelectionBehavior (int)
Handle<Value> QTableViewWrap::setSelectionBehavior(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setSelectionBehavior((QAbstractItemView::SelectionBehavior)args[0]->IntegerValue());
	return scope.Close(Undefined());
}

// setSelectionMode (int)
Handle<Value> QTableViewWrap::setSelectionMode(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setSelectionMode((QAbstractItemView::SelectionMode)args[0]->IntegerValue());
	return scope.Close(Undefined());
}

// setShowGrid (bool)
Handle<Value> QTableViewWrap::setShowGrid(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();

	q->setShowGrid(args[0]->BooleanValue());

	return scope.Close(Undefined());
}

//- setColumnHidden (int column, bool hide)
Handle<Value> QTableViewWrap::setColumnHidden(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

  q->setColumnHidden(args[0]->IntegerValue(), args[1]->BooleanValue());

  return scope.Close(Undefined());
}

// sortByColumn ( int column, Qt::SortOrder order )
// Qt::AscendingOrder	0, Qt::DescendingOrder	1
Handle<Value> QTableViewWrap::sortByColumn(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

  q->sortByColumn(args[0]->IntegerValue(), (Qt::SortOrder)args[1]->IntegerValue());

  return scope.Close(Undefined());
}

// setSortingEnabled ( bool enable )
Handle<Value> QTableViewWrap::setSortingEnabled(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

  q->setSortingEnabled(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

// setDragEnabled (bool)
Handle<Value> QTableViewWrap::setDragEnabled(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

	q->setDragEnabled(args[0]->BooleanValue());
  return scope.Close(Undefined());
}


// HEADER METHODS

// setHorizontalHeaderDefaultAlignment (int)
Handle<Value> QTableViewWrap::setHorizontalHeaderDefaultAlignment(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

	q->horizontalHeader()->setDefaultAlignment((Qt::Alignment)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

// setHorizontalStretchLastSection (bool)
Handle<Value> QTableViewWrap::setHorizontalStretchLastSection(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

	q->horizontalHeader()->setStretchLastSection(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableViewWrap::setRowDefaultHeight(const Arguments& args) {
	HandleScope scope;

	QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
	QTableView* q = w->GetWrapped();
	
	q->verticalHeader()->setDefaultSectionSize(args[0]->IntegerValue());

	return scope.Close(Undefined());
}

// setVerticalHeaderVisible (bool)
Handle<Value> QTableViewWrap::setVerticalHeaderVisible(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

	q->verticalHeader()->setVisible(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

// setVerticalStretchLastSection (bool)
Handle<Value> QTableViewWrap::setVerticalStretchLastSection(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();

	q->verticalHeader()->setStretchLastSection(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

// setCurrentIndex ( const QModelIndex & index ) 
Handle<Value> QTableViewWrap::setCurrentIndex(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
	
	QModelIndexWrap* item_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
		
	q->setCurrentIndex(*item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

// selectedIndexes
Handle<Value> QTableViewWrap::selectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
			
	QList<QModelIndex> indexes = q->selectionModel()->selectedIndexes();
  
  // create array
  Handle<Array> indexArray = v8::Array::New(indexes.count());
  for (int i=0;i<indexes.count();i++){
  	indexArray->Set(i, QModelIndexWrap::NewInstance(indexes[i]));
  }
  
  return scope.Close(indexArray);
}

//NEW
// setSelectedIndexes
Handle<Value> QTableViewWrap::setSelectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
	
	QItemSelection *selection = new QItemSelection(); // QList<QModelIndex>
	
	//JS ARRAY TO QLIST
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++){
  	QModelIndexWrap* index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(obj->Get(i)->ToObject());
  	QItemSelectionRange* range = new QItemSelectionRange(*index_wrap->GetWrapped());
    selection->append( *range );
  }
	
	int flags = args[1]->IntegerValue();
	
	// const QItemSelection & selection, QItemSelectionModel::SelectionFlags command 
	q->selectionModel()->select(
		*selection,
		(QItemSelectionModel::SelectionFlags)flags
	);
	  
  return scope.Close(Undefined());
}

// indexAt
Handle<Value> QTableViewWrap::indexAt(const Arguments& args) {
  HandleScope scope;

  QTableViewWrap* w = ObjectWrap::Unwrap<QTableViewWrap>(args.This());
  QTableView* q = w->GetWrapped();
			  
  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
  QPoint* point = point_wrap->GetWrapped();
  QModelIndex index = q->indexAt(*point);

  return scope.Close(QModelIndexWrap::NewInstance(index));
}