#include "qtablewidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTableWidget>;

class MyTableWidget : public QTableWidget
{
public:
MyTableWidget(QWidget* parent=0) :	QTableWidget(parent)	{}

QStringList MyTableWidget::mimeTypes() const{
	return QTableWidget::mimeTypes() << QString("text/plain"); // text/table-cells
}

//bool MyTableWidget::dropMimeData(int row, int column, const QMimeData *data, Qt::DropAction action){
//	if(data->hasText()){
//		QTableWidgetItem *item = this->item(row, column);
//		if(item != 0)
//			item->setText(data->text());
//		return false;
//	}else{
//		return QTableWidget::dropMimeData(row, column, data, action);
//	}
//}

QMimeData *MyTableWidget::mimeData(const QList<QTableWidgetItem *> items) const{
	QString text;

	//if(items.count() > 0){
	//	QTableWidgetItem *lastItem = items.last();
	//	foreach(QTableWidgetItem *item, items){
	//		if(item->column()==0&&!item->text().isEmpty()){
	//			text.append(item->text());
	//			if(item != lastItem){
	//				text.append(QString("\r\n"));
	//			}
	//		}
	//	}
	//}

	QMimeData *mimeData = QTableWidget::mimeData(items);

	/////
	
//	QString ** cells; //[x][y]
//	cells = new QString*[colCnt];
//	for(int i=0; i<colCnt; i++) cells[i] = new QString[rowCnt];

	// make sure data is in correct order: c1r1 c1r2... c2r1 c2r2 ...
	int row, col;
	int colCnt = columnCount();
	QString * cols;
	cols = new QString[colCnt];
	QByteArray encoded = mimeData->data("application/x-qabstractitemmodeldatalist");
	QDataStream stream(&encoded, QIODevice::ReadOnly);
	while (!stream.atEnd()){
		QMap<int,  QVariant> roleDataMap;
		stream >> row >> col >> roleDataMap;
		cols[col].append(roleDataMap.value(0).toString());
		cols[col].append(QString("\r\n"));
	}
	for (col=0;col<colCnt;col++){
		text.append(cols[col]);
	}
			
	/////

	// a) as text/plain
	//mimeData->setText(text); // as text/plain
	
	// b) as custom mime-type
	QByteArray data;
	data.append(text);
	mimeData->setData("text/table-cells", data); // test
	
	return mimeData;
}

//public:
	//int fstype;

};


/*************************************************
 QTableWidget
*************************************************/

Persistent<Function> QTableWidgetWrap::constructor;

QTableWidgetWrap::QTableWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<MyTableWidget>(q_parent, fixScope);
	else
  	q_ = new MyTableWidget(q_parent);
}

QTableWidgetWrap::~QTableWidgetWrap() {
  //delete q_;
}

void QTableWidgetWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTableWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
//- item (int row, int col) -> string
//- setAlternatingRowColors (bool)
//- setColumnCount (int)
//- setCurrentCell (int row, int col)
//- setItem (int row, int col, string)
//- setRowCount (int)
//- sortItems (int column, int order)
//- setHorizontalHeaderLabels (list)
//- setVerticalHeaderLabels (list)

	ADD_METHOD(item)
	ADD_METHOD(setAlternatingRowColors)
	ADD_METHOD(setColumnCount)
	ADD_METHOD(setCurrentCell)
	ADD_METHOD(setItem)
	ADD_METHOD(setRowCount)
	
	ADD_METHOD(sortByColumn)
	ADD_METHOD(sortItems)
	
	ADD_METHOD(setHorizontalHeaderLabels)
	ADD_METHOD(setVerticalHeaderLabels)

	ADD_METHOD(setHorizontalHeaderVisible)
	ADD_METHOD(setVerticalHeaderVisible)
	
	ADD_METHOD(resizeColumnsToContents)
	ADD_METHOD(resizeRowsToContents)

	ADD_METHOD(setColumnWidth)
	
	ADD_METHOD(setRowHeight)
	ADD_METHOD(setRowDefaultHeight)
	
	ADD_METHOD(setHorizontalHeaderResizeMode)
	ADD_METHOD(setVerticalHeaderResizeMode)
	ADD_METHOD(setHorizontalStretchLastSection)
	ADD_METHOD(setVerticalStretchLastSection)
	
	ADD_METHOD(setHorizontalHeaderDefaultAlignment)
	ADD_METHOD(setVerticalHeaderDefaultAlignment)
	
	ADD_METHOD(horizontalHeaderItem)
	ADD_METHOD(verticalHeaderItem)
	
	ADD_METHOD(setHorizontalHeaderMovable)
	ADD_METHOD(setVerticalHeaderMovable)
	
	ADD_METHOD(insertRow)
	ADD_METHOD(removeRow)
			
	ADD_METHOD(scrollToTop)
	ADD_METHOD(scrollToBottom)
	ADD_METHOD(setShowGrid)
	
	ADD_METHOD(setDragEnabled)
	
	ADD_METHOD(columnCount)
	ADD_METHOD(rowCount)
	
	ADD_METHOD(selectedItems)
	
	ADD_METHOD(findItems)
	
	//test
	//Local<FunctionTemplate> x = FunctionTemplate::New(test);
	//x->Set("length", Number::New(3));
	//x->Set("name", String::NewSymbol("test"));
	//x->Set("func_property", v8::Number::New(1)); // funktioniert (als einziges)
	//tpl->PrototypeTemplate()->Set(String::NewSymbol("test"), x->GetFunction());

	//Local<Template> proto_t = x->PrototypeTemplate();
	//proto_t->Set("length", v8::Number::New(2));
	//proto_t->Set("name", String::NewSymbol("test"));

	// QAbstractItemView ////////////////////////////////////////////
	
//QAbstractItemView::SelectionBehavior

//QAbstractItemView::SelectItems	0	Selecting single items.
//QAbstractItemView::SelectRows	1	Selecting only rows.
//QAbstractItemView::SelectColumns	2	Selecting only columns.
	ADD_METHOD(setSelectionBehavior)
	
//QAbstractItemView::SelectionMode

//QAbstractItemView::SingleSelection	1	When the user selects an item, any already-selected item becomes unselected, and the user cannot unselect the selected item by clicking on it.
//QAbstractItemView::ContiguousSelection	4	When the user selects an item in the usual way, the selection is cleared and the new item selected. However, if the user presses the Shift key while clicking on an item, all items between the current item and the clicked item are selected or unselected, depending on the state of the clicked item.
//QAbstractItemView::ExtendedSelection	3	When the user selects an item in the usual way, the selection is cleared and the new item selected. However, if the user presses the Ctrl key when clicking on an item, the clicked item gets toggled and all other items are left untouched. If the user presses the Shift key while clicking on an item, all items between the current item and the clicked item are selected or unselected, depending on the state of the clicked item. Multiple items can be selected by dragging the mouse over them.
//QAbstractItemView::MultiSelection	2	When the user selects an item in the usual way, the selection status of that item is toggled and the other items are left alone. Multiple items can be toggled by dragging the mouse over them.
//QAbstractItemView::NoSelection	0	Items cannot be selected.
	ADD_METHOD(setSelectionMode)
	
// QAbstractItemView::EditTrigger

//QAbstractItemView::NoEditTriggers	0	No editing possible.
//QAbstractItemView::CurrentChanged	1	Editing start whenever current item changes.
//QAbstractItemView::DoubleClicked	2	Editing starts when an item is double clicked.
//QAbstractItemView::SelectedClicked	4	Editing starts when clicking on an already selected item.
//QAbstractItemView::EditKeyPressed	8	Editing starts when the platform edit key has been pressed over an item.
//QAbstractItemView::AnyKeyPressed	16	Editing starts when any key is pressed over an item.
//QAbstractItemView::AllEditTriggers	31	Editing starts for all above actions.
	ADD_METHOD(setEditTriggers)
	
//QAbstractItemView::DragDropMode
//QAbstractItemView::NoDragDrop	0	Does not support dragging or dropping.
//QAbstractItemView::DragOnly	1	The view supports dragging of its own items
//QAbstractItemView::DropOnly	2	The view accepts drops
//QAbstractItemView::DragDrop	3	The view supports both dragging and dropping
//QAbstractItemView::InternalMove	4	The view accepts move (not copy) operations only from itself.
	ADD_METHOD(setDragDropMode)
	
	ADD_METHOD(setCellWidget)
	ADD_METHOD(cellWidget)
	ADD_METHOD(removeCellWidget)
	
	ADD_METHOD(setItemPrototype)
	
	ADD_METHOD(currentRow)
	
	ADD_METHOD(setRootIndex)

	ADD_METHOD(setSortingEnabled)

	ADD_METHOD(itemAt)
	
	ADD_METHOD(currentItem)
	ADD_METHOD(setCurrentItem)
	
	ADD_METHOD(selectedIndexes)
	ADD_METHOD(setSelectedIndexes)

	//geerbt
	ADD_WIDGET_METHODS(QTableWidgetWrap, QTableWidget)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTableWidget"), constructor);
}

Handle<Value> QTableWidgetWrap::New(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = new QTableWidgetWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTableWidgetWrap::NewInstance(QTableWidget *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTableWidgetWrap* w = node::ObjectWrap::Unwrap<QTableWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTableWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  
  //QTableWidget* q = w->GetWrapped();
  QImpl<QTableWidget>* q = (QImpl<QTableWidget>*)(w->GetWrapped()); //TODO: check if really QTableWidgetImpl!!!

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="cellActivated")
		QObject::connect(q, SIGNAL(cellActivated(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="cellChanged")
		QObject::connect(q, SIGNAL(cellChanged(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="cellClicked")
		QObject::connect(q, SIGNAL(cellClicked(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="cellDoubleClicked")
		QObject::connect(q, SIGNAL(cellDoubleClicked(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="cellEntered")
		QObject::connect(q, SIGNAL(cellEntered(int,int)), rec, SLOT(evtII(int,int)));
	else if (sign=="cellPressed")
		QObject::connect(q, SIGNAL(cellPressed(int,int)), rec, SLOT(evtII(int,int)));

	else if (sign=="currentItemChanged")
		QObject::connect(q, SIGNAL(currentItemChanged(QTableWidgetItem*,QTableWidgetItem*)), rec, SLOT(evtTwTw(QTableWidgetItem*,QTableWidgetItem*)));
	else if (sign=="itemSelectionChanged")
		QObject::connect(q, SIGNAL(itemSelectionChanged()), rec, SLOT(evtV()));
				
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTableWidgetWrap::off(const Arguments& args) {
	HandleScope scope;
  QTableWidgetWrap* w = node::ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  
 	//QTableWidget* q = w->GetWrapped();
	QImpl<QTableWidget>* q = (QImpl<QTableWidget>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="cellActivated")
			QObject::disconnect(q, SIGNAL(cellActivated(int,int)), rec, 0);	
		else if (sign=="cellChanged")
			QObject::disconnect(q, SIGNAL(cellChanged(int,int)), rec, 0);	
		else if (sign=="cellClicked")
			QObject::disconnect(q, SIGNAL(cellClicked(int,int)), rec, 0);	
		else if (sign=="cellDoubleClicked")
			QObject::disconnect(q, SIGNAL(cellDoubleClicked(int,int)), rec, 0);	
		else if (sign=="cellEntered")
			QObject::disconnect(q, SIGNAL(cellEntered(int,int)), rec, 0);	
		else if (sign=="cellPressed")
			QObject::disconnect(q, SIGNAL(cellPressed(int,int)), rec, 0);
			
		else if (sign=="currentItemChanged")
			QObject::disconnect(q, SIGNAL(currentItemChanged(QTableWidgetItem*,QTableWidgetItem*)), rec, 0);	
		else if (sign=="itemSelectionChanged")
			QObject::disconnect(q, SIGNAL(itemSelectionChanged()), rec, 0);
			
	  HANDLE_WIDGET_EVENTS_OFF
	  
		// delete receivers?
	}
	return args.This();
}

//- item (int row, int col) -> QTableWidgetItem *    string
Handle<Value> QTableWidgetWrap::item(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

//QTableWidgetItem* widgetItem = ((QTableWidget*)w)->item(row, col);
//QVariant var = widgetItem->data(QTableWidgetItem::Type);
//RETURN_QSTR(var.toString())
	
//  return scope.Close(qt_v8::FromQString(
//  	q->item(args[0]->IntegerValue(), args[1]->IntegerValue())->data(QTableWidgetItem::Type).toString()
//  ));
  
	QTableWidgetItem *item = q->item(args[0]->IntegerValue(), args[1]->IntegerValue());
	  
  return scope.Close(item ? QTableWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

//- setAlternatingRowColors (bool)
Handle<Value> QTableWidgetWrap::setAlternatingRowColors(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setAlternatingRowColors(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setColumnCount (int)
Handle<Value> QTableWidgetWrap::setColumnCount(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setColumnCount(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setCurrentCell (int row, int col)
Handle<Value> QTableWidgetWrap::setCurrentCell(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setCurrentCell(args[0]->IntegerValue(), args[1]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setItem (int row, int col, object QTableWidgetItem)
Handle<Value> QTableWidgetWrap::setItem(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QTableWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args[2]->ToObject());
		
	q->setItem(
		args[0]->IntegerValue(), 
		args[1]->IntegerValue(), 
		//new QTableWidgetItem(qt_v8::ToQString(args[2]->ToString()))
		item_wrap->GetWrapped()
	);
	
  return scope.Close(Undefined());
}

//- setRowCount (int)
Handle<Value> QTableWidgetWrap::setRowCount(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setRowCount(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// sortByColumn ( int column, Qt::SortOrder order )
// Qt::AscendingOrder	0, Qt::DescendingOrder	1
Handle<Value> QTableWidgetWrap::sortByColumn(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

  q->sortByColumn(args[0]->IntegerValue(), (Qt::SortOrder)args[1]->IntegerValue());

  return scope.Close(Undefined());
}

//- sortItems (int column, int order)
Handle<Value> QTableWidgetWrap::sortItems(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->sortItems(args[0]->IntegerValue(), (Qt::SortOrder)args[1]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setHorizontalHeaderLabels (array labels)
Handle<Value> QTableWidgetWrap::setHorizontalHeaderLabels(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	if (args[0]->IsArray()){
		QStringList labels;
    Local<Object> obj = args[0]->ToObject();
    Local<Array> props = obj->GetPropertyNames();
    for(unsigned int i = 0; i < props->Length(); i++) {
      labels.append(
      	qt_v8::ToQString(obj->Get(i)->ToString())
      );
    }
    q->setHorizontalHeaderLabels(labels);
	}
			
  return scope.Close(Undefined());
}

//- setVerticalHeaderLabels (list)
Handle<Value> QTableWidgetWrap::setVerticalHeaderLabels(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	if (args[0]->IsArray()){
		QStringList labels;
    Local<Object> obj = args[0]->ToObject();
    Local<Array> props = obj->GetPropertyNames();
    for(unsigned int i = 0; i < props->Length(); i++) {
      labels.append(
      	qt_v8::ToQString(obj->Get(i)->ToString())
      );
    }
    q->setVerticalHeaderLabels(labels);
	}
  return scope.Close(Undefined());
}

//- setHorizontalHeaderVisible (bool)
Handle<Value> QTableWidgetWrap::setHorizontalHeaderVisible(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->horizontalHeader()->setVisible(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

//- setVerticalHeaderVisible (bool)
Handle<Value> QTableWidgetWrap::setVerticalHeaderVisible(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setVisible(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::resizeColumnsToContents(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->resizeColumnsToContents();
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::resizeRowsToContents(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->resizeRowsToContents();
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setColumnWidth(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setColumnWidth(args[0]->IntegerValue(), args[1]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setRowHeight(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setRowHeight(args[0]->IntegerValue(), args[1]->IntegerValue());
	
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setRowDefaultHeight(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setDefaultSectionSize(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//QHeaderView::Interactive	0	The user can resize the section. The section can also be resized programmatically using resizeSection(). The section size defaults to defaultSectionSize. (See also cascadingSectionResizes.)
//QHeaderView::Fixed	2	The user cannot resize the section. The section can only be resized programmatically using resizeSection(). The section size defaults to defaultSectionSize.
//QHeaderView::Stretch	1	QHeaderView will automatically resize the section to fill the available space. The size cannot be changed by the user or programmatically.
//QHeaderView::ResizeToContents	3	QHeaderView will automatically resize the section to its optimal size based on the contents of the entire column or row. The size cannot be changed by the user or programmatically. (This value was introduced in 4.2)


//- setHorizontalHeaderResizeMode (int)
Handle<Value> QTableWidgetWrap::setHorizontalHeaderResizeMode(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->horizontalHeader()->setResizeMode((QHeaderView::ResizeMode)args[0]->IntegerValue());
  return scope.Close(Undefined());
}
//- setVerticalHeaderResizeMode (int)
Handle<Value> QTableWidgetWrap::setVerticalHeaderResizeMode(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setResizeMode((QHeaderView::ResizeMode)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

//- setHorizontalStretchLastSection (bool)
Handle<Value> QTableWidgetWrap::setHorizontalStretchLastSection(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->horizontalHeader()->setStretchLastSection(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

//- setVerticalStretchLastSection (bool)
Handle<Value> QTableWidgetWrap::setVerticalStretchLastSection(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setStretchLastSection(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

//- setHorizontalHeaderDefaultAlignment (int)
Handle<Value> QTableWidgetWrap::setHorizontalHeaderDefaultAlignment(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->horizontalHeader()->setDefaultAlignment((Qt::Alignment)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

//- setVerticalHeaderDefaultAlignment (int)
Handle<Value> QTableWidgetWrap::setVerticalHeaderDefaultAlignment(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setDefaultAlignment((Qt::Alignment)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

//- horizontalHeaderItem (int col) -> QTableWidgetItem *    string
Handle<Value> QTableWidgetWrap::horizontalHeaderItem(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	QTableWidgetItem *item = q->horizontalHeaderItem(args[0]->IntegerValue());
	  
  return scope.Close(item ? QTableWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

//- verticalHeaderItem (int row) -> QTableWidgetItem *    string
Handle<Value> QTableWidgetWrap::verticalHeaderItem(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	QTableWidgetItem *item = q->verticalHeaderItem(args[0]->IntegerValue());
	  
  return scope.Close(item ? QTableWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

//- insertRow (int row)
Handle<Value> QTableWidgetWrap::insertRow(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	q->insertRow(args[0]->IntegerValue());
	  
  return scope.Close(Undefined());
}

//- removeRow (int row)
Handle<Value> QTableWidgetWrap::removeRow(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	q->removeRow(args[0]->IntegerValue());
	  
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setSelectionBehavior(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setSelectionBehavior((QAbstractItemView::SelectionBehavior)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setSelectionMode(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setSelectionMode((QAbstractItemView::SelectionMode)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setEditTriggers(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setEditTriggers((QAbstractItemView::EditTriggers)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QTableWidgetWrap::setDragDropMode(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setDragDropMode((QAbstractItemView::DragDropMode)args[0]->IntegerValue());
  return scope.Close(Undefined());
}

//- setCellWidget (int row, int col, QWidget * widget)
Handle<Value> QTableWidgetWrap::setCellWidget(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(3)
	//ASSERT_OBJECT_TYPE(args[2], "QWidget")
	if (!args[2]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[2]->ToObject());
	
	q->setCellWidget(
		args[0]->IntegerValue(), 
		args[1]->IntegerValue(), 
		widget_wrap->GetWrapped()
	);
	
  return scope.Close(Undefined());
}

//- cellWidget (int row, int col) ->
Handle<Value> QTableWidgetWrap::cellWidget(const Arguments& args) {
  HandleScope scope;
  
  ASSERT_NARGS(2)
  
  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	QWidget * widget = q->cellWidget(args[0]->IntegerValue(), args[1]->IntegerValue());
		
  return scope.Close(QWidgetWrap::NewInstance(widget));
}

//- removeCellWidget (int row, int col) ->
Handle<Value> QTableWidgetWrap::removeCellWidget(const Arguments& args) {
  HandleScope scope;
  
  ASSERT_NARGS(2)
  
  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	q->removeCellWidget(args[0]->IntegerValue(), args[1]->IntegerValue());
		
  return scope.Close(Undefined());
}

//- setItemPrototype (QTableWidgetItem item)
Handle<Value> QTableWidgetWrap::setItemPrototype(const Arguments& args) {
  HandleScope scope;

	ASSERT_NARGS(1)
	//ASSERT_OBJECT_TYPE(args[2], "QWidget")
	if (!args[0]->IsObject()) THROW_ERROR_BAD_TYPES
	
  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QTableWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args[0]->ToObject());
	
	q->setItemPrototype(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- currentRow () -> int
Handle<Value> QTableWidgetWrap::currentRow(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  return scope.Close(Integer::New(q->currentRow()));
}

//- scrollToTop ()
Handle<Value> QTableWidgetWrap::scrollToTop(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  q->scrollToTop();
  
  return scope.Close(Undefined());
}

//- scrollToBottom ()
Handle<Value> QTableWidgetWrap::scrollToBottom(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  q->scrollToBottom();
  
  return scope.Close(Undefined());
}

// setShowGrid (bool)
Handle<Value> QTableWidgetWrap::setShowGrid(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
	q->setShowGrid(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

//- setRootIndex (object QModelIndex)
Handle<Value> QTableWidgetWrap::setRootIndex(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QModelIndexWrap* item_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
		
	q->setRootIndex(*item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

// setSortingEnabled ( bool enable )
Handle<Value> QTableWidgetWrap::setSortingEnabled(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

  q->setSortingEnabled(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

//- setDragEnabled (bool)
Handle<Value> QTableWidgetWrap::setDragEnabled(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->setDragEnabled(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

// columnCount () -> int
Handle<Value> QTableWidgetWrap::columnCount(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  return scope.Close(Integer::New(q->columnCount()));
}

// rowCount () -> int
Handle<Value> QTableWidgetWrap::rowCount(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  return scope.Close(Integer::New(q->rowCount()));
}

// selectedItems () -> []
Handle<Value> QTableWidgetWrap::selectedItems(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  QList<QTableWidgetItem *> items = q->selectedItems();
  
  // create array
  Handle<Array> itemsArray = v8::Array::New(items.count());
  for (int i=0;i<items.count();i++){  	
  	itemsArray->Set(i, QTableWidgetItemWrap::NewInstance(items[i]));
  }
  
  return scope.Close(itemsArray);
}


//Qt::MatchExactly	0	Performs QVariant-based matching.
//Qt::MatchFixedString	8	Performs string-based matching. String-based comparisons are case-insensitive unless the MatchCaseSensitive flag is also specified.
//Qt::MatchContains	1	The search term is contained in the item.
//Qt::MatchStartsWith	2	The search term matches the start of the item.
//Qt::MatchEndsWith	3	The search term matches the end of the item.
//Qt::MatchCaseSensitive	16	The search is case sensitive.
//Qt::MatchRegExp	4	Performs string-based matching using a regular expression as the search term.
//Qt::MatchWildcard	5	Performs string-based matching using a string with wildcards as the search term.
//Qt::MatchWrap	32	Perform a search that wraps around, so that when the search reaches the last item in the model, it begins again at the first item and continues until all items have been examined.
//Qt::MatchRecursive	64	Searches the entire hierarchy.

// findItems (str, flags) -> []
Handle<Value> QTableWidgetWrap::findItems(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  
  QList<QTableWidgetItem *> items = q->findItems(qt_v8::ToQString(args[0]->ToString()), (Qt::MatchFlags)args[1]->IntegerValue());
  
  // create array
  Handle<Array> itemsArray = v8::Array::New(items.count());
  for (int i=0;i<items.count();i++){  	
  	itemsArray->Set(i, QTableWidgetItemWrap::NewInstance(items[i]));
  }
  
  return scope.Close(itemsArray);
}

// itemAt (QPoint) -> QTableWidgetItem *
// itemAt (int x, int y) -> QTableWidgetItem *
Handle<Value> QTableWidgetWrap::itemAt(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  
  //QTableWidget* q = w->GetWrapped();
  QImpl<QTableWidget>* q = (QImpl<QTableWidget>*)(w->GetWrapped());
  
  QTableWidgetItem *item;
  if (args[0]->IsObject()){
	  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
	  QPoint* point = point_wrap->GetWrapped();
	  item = q->itemAt(*point);
	}else{//if (args[0]->IsString()) { 
		item = q->itemAt(args[0]->IntegerValue(), args[1]->IntegerValue());
	}
  return scope.Close(item ? QTableWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

//- currentItem () -> QTreeWidgetItem *
Handle<Value> QTableWidgetWrap::currentItem(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
  	
  QTableWidgetItem *item = q->currentItem();
  
  return scope.Close(QTableWidgetItemWrap::NewInstance(item));
}

// setCurrentItem ( QTableWidgetItem * item[, QItemSelectionModel::SelectionFlags command] ) 
Handle<Value> QTableWidgetWrap::setCurrentItem(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QTableWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTableWidgetItemWrap>(args[0]->ToObject());
	
	if (args.Length()>1){
		q->setCurrentItem(item_wrap->GetWrapped(),(QItemSelectionModel::SelectionFlags)args[1]->IntegerValue());
	}else{
		q->setCurrentItem(item_wrap->GetWrapped());
	}
	
  return scope.Close(Undefined());
}

// selectedIndexes
Handle<Value> QTableWidgetWrap::selectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
			
	QList<QModelIndex> indexes = q->selectionModel()->selectedIndexes();
  
  // create array
  Handle<Array> indexArray = v8::Array::New(indexes.count());
  for (int i=0;i<indexes.count();i++){
  	indexArray->Set(i, QModelIndexWrap::NewInstance(indexes[i]));
  }
  
  return scope.Close(indexArray);
}

// setSelectedIndexes
Handle<Value> QTableWidgetWrap::setSelectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();
	
	QItemSelection *selection = new QItemSelection(); // QList<QModelIndex>
	
	//JS ARRAY TO QLIST
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++){
  	QModelIndexWrap* index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(obj->Get(i)->ToObject());
  	QItemSelectionRange* range = new QItemSelectionRange(*index_wrap->GetWrapped());
    selection->append( *range );
  }
	
	int flags = args[1]->IntegerValue();
	
	// const QItemSelection & selection, QItemSelectionModel::SelectionFlags command 
	q->selectionModel()->select(
		*selection,
		(QItemSelectionModel::SelectionFlags)flags
	);
	  
  return scope.Close(Undefined());
}

//- setHorizontalHeaderMovable (bool)
Handle<Value> QTableWidgetWrap::setHorizontalHeaderMovable(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->horizontalHeader()->setMovable(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

//- setVerticalHeaderMovable (bool)
Handle<Value> QTableWidgetWrap::setVerticalHeaderMovable(const Arguments& args) {
  HandleScope scope;

  QTableWidgetWrap* w = ObjectWrap::Unwrap<QTableWidgetWrap>(args.This());
  QTableWidget* q = w->GetWrapped();

	q->verticalHeader()->setMovable(args[0]->BooleanValue());
  return scope.Close(Undefined());
}