
#include "qtabwidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTabWidget>;

/*************************************************
 QTabWidgetWrap
*************************************************/

Persistent<Function> QTabWidgetWrap::constructor;

QTabWidgetWrap::QTabWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTabWidget>(q_parent, fixScope);
	else
  	q_ = new QTabWidget(q_parent);
}

QTabWidgetWrap::~QTabWidgetWrap() {
  //delete q_;
}

void QTabWidgetWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTabWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
//- addTab ( object widget, string ico, string text || widget, text )
//- count () -> int
//- currentIndex () -> int
//- insertTab (int index, object widget, string ico, string text || index, widget, text) -> int
//- removeTab (int index)
//- setMovable (bool)
//- setShape (int)
//- setTabEnabled (int index, bool enabled)
//- setTabsClosable (bool)
//- setTabText (int index, string text)
	//- setTabTextColor (int index, color)
//- setTabToolTip int index, string text)
//- setTabWhatsThis (int index, string text)
//- setUsesScrollButtons (bool)

	ADD_METHOD(addTab)
	ADD_METHOD(count)
	ADD_METHOD(currentIndex)
	ADD_METHOD(insertTab)
	ADD_METHOD(removeTab)
	ADD_METHOD(setMovable)
	ADD_METHOD(setTabShape)
	ADD_METHOD(setTabEnabled)
	ADD_METHOD(setTabsClosable)
	ADD_METHOD(setTabText)
	//ADD_METHOD(setTabTextColor)
	ADD_METHOD(setTabToolTip)
	ADD_METHOD(setTabWhatsThis)
	ADD_METHOD(setUsesScrollButtons)
	
	ADD_METHOD(setTabPosition)
	
	//ADD_METHOD(tabBar)

	//geerbt
	ADD_WIDGET_METHODS(QTabWidgetWrap, QTabWidget)
		
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTabWidget"), constructor);
}

Handle<Value> QTabWidgetWrap::New(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = new QTabWidgetWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTabWidgetWrap::NewInstance(QTabWidget *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTabWidgetWrap* w = node::ObjectWrap::Unwrap<QTabWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTabWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  //QTabWidget* q = w->GetWrapped();
	QImpl<QTabWidget>* q = (QImpl<QTabWidget>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		
	else if (sign=="currentChanged")
		QObject::connect(q, SIGNAL(currentChanged(int)), rec, SLOT(evtI(int)));
	else if (sign=="tabCloseRequested")
		QObject::connect(q, SIGNAL(tabCloseRequested(int)), rec, SLOT(evtI(int)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTabWidgetWrap::off(const Arguments& args) {
	HandleScope scope;
  QTabWidgetWrap* w = node::ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  
 	//QTabWidget* q = w->GetWrapped();
	QImpl<QTabWidget>* q = (QImpl<QTabWidget>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="currentChanged") 
			QObject::disconnect(q, SIGNAL(currentChanged(int)), rec, 0);	
		else if (sign=="tabCloseRequested") 
			QObject::disconnect(q, SIGNAL(tabCloseRequested(int)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- addTab ( object widget, string ico, string text || widget, text )
Handle<Value> QTabWidgetWrap::addTab(const Arguments& args) {//TODO: icon: string | buffer
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
  
	if (args.Length() > 2) {
		QIcon ico;
		ico.addFile( qt_v8::ToQString(args[1]->ToString()) );
  	q->addTab(
  		widget_wrap->GetWrapped(),
  		ico, 
  		qt_v8::ToQString(args[2]->ToString())
  	);
	}else{
		q->addTab(widget_wrap->GetWrapped(), qt_v8::ToQString(args[1]->ToString()));
	}
	
  return scope.Close(Undefined());
}

//- count () -> int
Handle<Value> QTabWidgetWrap::count(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

  return scope.Close(Integer::New(q->count()));
}

//- currentIndex () -> int
Handle<Value> QTabWidgetWrap::currentIndex(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

  return scope.Close(Integer::New(q->currentIndex()));
}

//- insertTab (int index, object widget, string ico, string text || index, widget, text) -> ???
Handle<Value> QTabWidgetWrap::insertTab(const Arguments& args) {//TODO: icon: string | buffer
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

  QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[1]->ToObject());
  
	if (args.Length() > 3) {
		QIcon ico;
		ico.addFile( qt_v8::ToQString(args[2]->ToString()) );
  	q->insertTab(
  		args[0]->IntegerValue(),
  		widget_wrap->GetWrapped(),
  		ico,
  		qt_v8::ToQString(args[3]->ToString()) 
  	);
	}else{
		q->insertTab(
			args[0]->IntegerValue(),
			widget_wrap->GetWrapped(),
			qt_v8::ToQString(args[2]->ToString())
		);
	}
	
  return scope.Close(Undefined());
}

//- removeTab (int index)
Handle<Value> QTabWidgetWrap::removeTab(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->removeTab(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

// setMovable (bool)
Handle<Value> QTabWidgetWrap::setMovable(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setMovable(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setShape (int)
Handle<Value> QTabWidgetWrap::setTabShape(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabShape((QTabWidget::TabShape)args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setTabEnabled (int index, bool enabled)
Handle<Value> QTabWidgetWrap::setTabEnabled(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabEnabled(args[0]->IntegerValue(), args[1]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setTabsClosable (bool)
Handle<Value> QTabWidgetWrap::setTabsClosable(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabsClosable(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setTabText (int index, string text)
Handle<Value> QTabWidgetWrap::setTabText(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabText(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setTabTextColor (int index, int r, int g, int b)
//Handle<Value> QTabWidgetWrap::setTabTextColor(const Arguments& args) {
//  HandleScope scope;
//
//  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
//  QTabWidget* q = w->GetWrapped();
//
//	q->setTabTextColor(args[0]->IntegerValue(), QColor(
//		args[1]->IntegerValue(),
//		args[2]->IntegerValue(),
//		args[3]->IntegerValue()
//	));
//	
//  return scope.Close(Undefined());
//}

//- setTabToolTip int index, string text)
Handle<Value> QTabWidgetWrap::setTabToolTip(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabToolTip(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setTabWhatsThis (int index, string text)
Handle<Value> QTabWidgetWrap::setTabWhatsThis(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabWhatsThis(args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));
	
  return scope.Close(Undefined());
}

//- setUsesScrollButtons (bool)
Handle<Value> QTabWidgetWrap::setUsesScrollButtons(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setUsesScrollButtons(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

// setTabPosition (int)
Handle<Value> QTabWidgetWrap::setTabPosition(const Arguments& args) {
  HandleScope scope;

  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
  QTabWidget* q = w->GetWrapped();

	q->setTabPosition((QTabWidget::TabPosition) args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//tabBar [protected]
//Handle<Value> QTabWidgetWrap::tabBar(const Arguments& args) {
//  HandleScope scope;
//
//  QTabWidgetWrap* w = ObjectWrap::Unwrap<QTabWidgetWrap>(args.This());
//  QTabWidget* q = w->GetWrapped();
//
//  QTabBar *tb = q->tabBar();
//  
//  return scope.Close(QTabBarWrap::NewInstance(tb));
//}