
#include "qtextbrowser.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTextBrowser>;

/*************************************************
 QTextBrowserWrap
*************************************************/

Persistent<Function> QTextBrowserWrap::constructor;

QTextBrowserWrap::QTextBrowserWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTextBrowser>(q_parent, fixScope);
	else
  	q_ = new QTextBrowser(q_parent);
}

QTextBrowserWrap::~QTextBrowserWrap() {
  //delete q_;
}

void QTextBrowserWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTextBrowser"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
//- backward ()
//- forward ()
//- home ()
//- reload ()
//- setOpenLinks (bool)
//- setOpenExternalLinks (bool)
//- setSource (string)
//- source () -> string
//- setHtml (string)
//- setText (string)

	ADD_METHOD(backward)
	ADD_METHOD(forward)
	ADD_METHOD(home)
	ADD_METHOD(reload)
	ADD_METHOD(setOpenLinks)
	ADD_METHOD(setOpenExternalLinks)
	ADD_METHOD(setSource)
	ADD_METHOD(source)
	ADD_METHOD(setHtml)
	ADD_METHOD(setText)

	//geerbt
	ADD_WIDGET_METHODS(QTextBrowserWrap, QTextBrowser)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTextBrowser"), constructor);
}

Handle<Value> QTextBrowserWrap::New(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = new QTextBrowserWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTextBrowserWrap::NewInstance(QTextBrowser *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTextBrowserWrap* w = node::ObjectWrap::Unwrap<QTextBrowserWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTextBrowserWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  //QTextBrowser* q = w->GetWrapped();
	QImpl<QTextBrowser>* q = (QImpl<QTextBrowser>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="anchorClicked")
		QObject::connect(q, SIGNAL(anchorClicked(const QUrl &)), rec, SLOT(evtU(const QUrl &)));
	else if (sign=="backwardAvailable")
		QObject::connect(q, SIGNAL(backwardAvailable(bool)), rec, SLOT(evtB(bool)));
	else if (sign=="forwardAvailable")
		QObject::connect(q, SIGNAL(forwardAvailable(bool)), rec, SLOT(evtB(bool)));
	else if (sign=="highlighted")
		QObject::connect(q, SIGNAL(highlighted(const QUrl &)), rec, SLOT(evtU(const QUrl &)));
	else if (sign=="historyChanged")
		QObject::connect(q, SIGNAL(historyChanged()), rec, SLOT(evtV()));
	else if (sign=="sourceChanged")
		QObject::connect(q, SIGNAL(sourceChanged(const QUrl &)), rec, SLOT(evtU(const QUrl &)));	
	
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTextBrowserWrap::off(const Arguments& args) {
	HandleScope scope;
  QTextBrowserWrap* w = node::ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  
 	//QTextBrowser* q = w->GetWrapped();
	QImpl<QTextBrowser>* q = (QImpl<QTextBrowser>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="anchorClicked")
			QObject::disconnect(q, SIGNAL(anchorClicked(const QUrl &)), rec, 0);	
		else if (sign=="backwardAvailable")
			QObject::disconnect(q, SIGNAL(backwardAvailable(bool)), rec, 0);	
		else if (sign=="forwardAvailable")
			QObject::disconnect(q, SIGNAL(forwardAvailable(bool)), rec, 0);	
		else if (sign=="highlighted")
			QObject::disconnect(q, SIGNAL(highlighted(const QUrl &)), rec, 0);	
		else if (sign=="historyChanged")
			QObject::disconnect(q, SIGNAL(historyChanged()), rec, 0);	
		else if (sign=="sourceChanged")
			QObject::disconnect(q, SIGNAL(sourceChanged(const QUrl &)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

//- backward ()
Handle<Value> QTextBrowserWrap::backward(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->backward();
	
  return scope.Close(Undefined());
}

//- forward ()
Handle<Value> QTextBrowserWrap::forward(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->forward();
	
  return scope.Close(Undefined());
}

//- home ()
Handle<Value> QTextBrowserWrap::home(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->home();
	
  return scope.Close(Undefined());
}

//- reload ()
Handle<Value> QTextBrowserWrap::reload(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->reload();
	
  return scope.Close(Undefined());
}

//- setOpenLinks (bool)
Handle<Value> QTextBrowserWrap::setOpenLinks(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->setOpenLinks(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setOpenExternalLinks (bool)
Handle<Value> QTextBrowserWrap::setOpenExternalLinks(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->setOpenExternalLinks(args[0]->BooleanValue());
	
  return scope.Close(Undefined());
}

//- setSource (string)
Handle<Value> QTextBrowserWrap::setSource(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->setSource(QUrl(qt_v8::ToQString(args[0]->ToString())));
	
  return scope.Close(Undefined());
}

//- source () -> string
Handle<Value> QTextBrowserWrap::source(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	return scope.Close(qt_v8::FromQString(q->source().toString()));
}

//- setHtml (string)
Handle<Value> QTextBrowserWrap::setHtml(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->setHtml(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}

//- setText (string)
Handle<Value> QTextBrowserWrap::setText(const Arguments& args) {
  HandleScope scope;

  QTextBrowserWrap* w = ObjectWrap::Unwrap<QTextBrowserWrap>(args.This());
  QTextBrowser* q = w->GetWrapped();

	q->setText(qt_v8::ToQString(args[0]->ToString()));
	
  return scope.Close(Undefined());
}
