
#include "qtextedit.h"
#include "qwidgeteventstpl.h"

#include "../Types/qprinter.h"
#include "../Types/qtablewidgetitem.h"


using namespace v8;

template class QImpl<QTextEdit>;

		// QWidgetImpl()
		//
		//QTextEditImpl::QTextEditImpl() : QWidget() {
		//  // Initialize callbacks as boolean values so we can test if the callback
		//  // has been set via ->IsFunction() below
		//  //closeEventCallback_ = Persistent<Boolean>::New(Boolean::New(false));
		//}
		//
		//QTextEditImpl::~QTextEditImpl() {
		//  //closeEventCallback_.Dispose();
		//}

//void QTextEditImpl::printPreview(QPrinter *printer){
//	print(printer);
//}

/*************************************************
 QTextEditWrap
*************************************************/

Persistent<Function> QTextEditWrap::constructor;

QTextEditWrap::QTextEditWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTextEdit>(q_parent, fixScope); //QTextEditImpl
	else
  	q_ = new QTextEdit(q_parent); //QTextEditImpl
}

QTextEditWrap::~QTextEditWrap() {
  //delete q_;
}

void QTextEditWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTextEdit"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(append)
	ADD_METHOD(clear)
	ADD_METHOD(copy)
	ADD_METHOD(cut)
	ADD_METHOD(find)
	ADD_METHOD(paste)
	ADD_METHOD(print)
	ADD_METHOD(selectAll)
	ADD_METHOD(setHtml)
	ADD_METHOD(setPlainText)
	ADD_METHOD(setTabStopWidth)
	ADD_METHOD(setText)
	ADD_METHOD(toHtml)
	ADD_METHOD(toPlainText)
	ADD_METHOD(undo)

	//geerbt
	ADD_WIDGET_METHODS(QTextEditWrap, QTextEdit)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTextEdit"), constructor);
}

Handle<Value> QTextEditWrap::New(const Arguments& args) {
  HandleScope scope;

  QTextEditWrap* w = new QTextEditWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTextEditWrap::NewInstance(QTextEdit *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTextEditWrap* w = node::ObjectWrap::Unwrap<QTextEditWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTextEditWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  //QTextEdit* q = w->GetWrapped();
	QImpl<QTextEdit>* q = (QImpl<QTextEdit>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS	
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		
	else if (sign=="copyAvailable") 
		QObject::connect(q, SIGNAL(copyAvailable(bool)), rec, SLOT(evtB(bool)));
//	else if (sign=="currentCharFormatChanged") 
//		QObject::connect(q, SIGNAL(currentCharFormatChanged()), rec, SLOT(evtV()));
	else if (sign=="cursorPositionChanged") 
		QObject::connect(q, SIGNAL(cursorPositionChanged()), rec, SLOT(evtV()));	
	else if (sign=="redoAvailable") 
		QObject::connect(q, SIGNAL(redoAvailable(bool)), rec, SLOT(evtB(bool)));
	else if (sign=="selectionChanged") 
		QObject::connect(q, SIGNAL(selectionChanged()), rec, SLOT(evtV()));	
	else if (sign=="textChanged") 
		QObject::connect(q, SIGNAL(textChanged()), rec, SLOT(evtV()));	
	else if (sign=="undoAvailable") 
		QObject::connect(q, SIGNAL(undoAvailable(bool)), rec, SLOT(evtB(bool)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTextEditWrap::off(const Arguments& args) {
	HandleScope scope;
  QTextEditWrap* w = node::ObjectWrap::Unwrap<QTextEditWrap>(args.This());
 	
 	//QTextEdit* q = w->GetWrapped();
	QImpl<QTextEdit>* q = (QImpl<QTextEdit>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="copyAvailable") 
			QObject::disconnect(q, SIGNAL(copyAvailable(bool)), rec, 0);	
	//	else if (sign=="currentCharFormatChanged") 
	//		QObject::disconnect(q, SIGNAL(currentCharFormatChanged()), rec, 0);	
		else if (sign=="cursorPositionChanged") 
			QObject::disconnect(q, SIGNAL(cursorPositionChanged()), rec, 0);	
		else if (sign=="redoAvailable") 
			QObject::disconnect(q, SIGNAL(redoAvailable(bool)), rec, 0);	
		else if (sign=="selectionChanged") 
			QObject::disconnect(q, SIGNAL(selectionChanged()), rec, 0);	
		else if (sign=="textChanged") 
			QObject::disconnect(q, SIGNAL(textChanged()), rec, 0);	
		else if (sign=="undoAvailable") 
			QObject::disconnect(q, SIGNAL(undoAvailable(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QTextEditWrap::append(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->append(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::clear(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->clear();
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::copy(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->copy();
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::cut(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->cut();
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::find(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  return scope.Close(Boolean::New( q->find(qt_v8::ToQString(args[0]->ToString())) ));
}

Handle<Value> QTextEditWrap::paste(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->paste();
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::print(const Arguments& args) {
	if (args.Length() < 1) return v8::ThrowException(v8::Exception::Error(v8::String::New("Illegal arguments")));
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  QPrinterWrap* printer_wrap = ObjectWrap::Unwrap<QPrinterWrap>(args[0]->ToObject()); 
  q->print( printer_wrap->GetWrapped() );
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::selectAll(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->selectAll();
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::setHtml(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->setHtml(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::setPlainText(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->setPlainText(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::setTabStopWidth(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->setTabStopWidth(args[0]->IntegerValue());
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::setText(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->setText(qt_v8::ToQString(args[0]->ToString()));
  return scope.Close(Undefined());
}

Handle<Value> QTextEditWrap::toHtml(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  return scope.Close(qt_v8::FromQString(q->toHtml()));
}

Handle<Value> QTextEditWrap::toPlainText(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  return scope.Close(qt_v8::FromQString(q->toPlainText()));
}

Handle<Value> QTextEditWrap::undo(const Arguments& args) {
  HandleScope scope;
  QTextEditWrap* w = ObjectWrap::Unwrap<QTextEditWrap>(args.This());
  QTextEdit* q = w->GetWrapped();
  q->undo();
  return scope.Close(Undefined());
}
