#include "qtoolbar.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QToolBar>;

/*************************************************
 QTimeEditWrap
*************************************************/

Persistent<Function> QToolBarWrap::constructor;

QToolBarWrap::QToolBarWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QToolBar>(q_parent, fixScope);
	else
  	q_ = new QToolBar(q_parent);
}

QToolBarWrap::~QToolBarWrap() {
  //delete q_;
}

void QToolBarWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QToolBar"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_METHOD(addAction)
  ADD_METHOD(addSeparator)
	
  ADD_METHOD(setIconSize)
  
  ADD_METHOD(addWidget)
  
  ADD_METHOD(setOrientation)
  ADD_METHOD(setAllowedAreas)
  
  //ADD_METHOD(setFloatable)
  //ADD_METHOD(setMovable)
  
	//geerbt
	ADD_WIDGET_METHODS(QToolBarWrap, QToolBar)
	
	//from QWidget
	//tpl->PrototypeTemplate()->Set(String::NewSymbol("setWindowTitle"), FunctionTemplate::New(SetWindowTitle)->GetFunction());
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QToolBar"), constructor);
}

Handle<Value> QToolBarWrap::New(const Arguments& args) {
  HandleScope scope;

  QToolBarWrap* w = new QToolBarWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QToolBarWrap::NewInstance(QToolBar *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QToolBarWrap* w = node::ObjectWrap::Unwrap<QToolBarWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QToolBarWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  //QToolBar* q = w->GetWrapped();
	QImpl<QToolBar>* q = (QImpl<QToolBar>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS	
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		
	else if (sign=="visibilityChanged") 
		QObject::connect(q, SIGNAL(visibilityChanged(bool)), rec, SLOT(evtB(bool)));	
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QToolBarWrap::off(const Arguments& args) {
	HandleScope scope;
  QToolBarWrap* w = node::ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  
 	//QToolBar* q = w->GetWrapped();
	QImpl<QToolBar>* q = (QImpl<QToolBar>*)(w->GetWrapped());
	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="visibilityChanged") 
			QObject::disconnect(q, SIGNAL(visibilityChanged(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

// addAction ( QAction * action )
// addAction ( QString text ) -> obj action
Handle<Value> QToolBarWrap::addAction(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();
  
  if (args[0]->IsObject()){
	  QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
	  q->addAction(action_wrap->GetWrapped());  
	}else if (args[0]->IsString()){
  	QAction* a = q->addAction(qt_v8::ToQString(args[0]->ToString()));  
  	return scope.Close(QActionWrap::NewInstance(a));
	}
	return scope.Close(Undefined());
}

Handle<Value> QToolBarWrap::addSeparator(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();
  q->addSeparator();
  return scope.Close(Undefined());
}

// setIconSize (w, h)
Handle<Value> QToolBarWrap::setIconSize(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();
  
  q->setIconSize(QSize(args[0]->IntegerValue(),args[1]->IntegerValue()));
  
  return scope.Close(Undefined());
}

// addWidget (widget)
Handle<Value> QToolBarWrap::addWidget(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();

	QWidgetWrap* widget_wrap = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
	QAction* a = q->addWidget(widget_wrap->GetWrapped());
	
  return scope.Close(QActionWrap::NewInstance(a));
}

// setOrientation
Handle<Value> QToolBarWrap::setOrientation(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();

	q->setOrientation((Qt::Orientation)args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// setAllowedAreas
Handle<Value> QToolBarWrap::setAllowedAreas(const Arguments& args) {
  HandleScope scope;
  QToolBarWrap* w = ObjectWrap::Unwrap<QToolBarWrap>(args.This());
  QToolBar* q = w->GetWrapped();

	q->setAllowedAreas((Qt::ToolBarArea)args[0]->IntegerValue());

  return scope.Close(Undefined());
}
