#include "qtoolbutton.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QToolBar>;

/*************************************************
 QToolButtonWrap
*************************************************/

Persistent<Function> QToolButtonWrap::constructor;

QToolButtonWrap::QToolButtonWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QToolButton>(q_parent, fixScope);
	else
  	q_ = new QToolButton(q_parent);
}

QToolButtonWrap::~QToolButtonWrap() {
  //delete q_;
}

void QToolButtonWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QToolButton"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
      
  ADD_METHOD(menu)
  ADD_METHOD(setMenu)
  ADD_METHOD(setPopupMode)
  ADD_METHOD(setToolButtonStyle)
  
  ADD_METHOD(setDefaultAction)
  
  // eigtl. geerbt von QAbstractButton
  ADD_METHOD(setText)
  ADD_METHOD(text)
  
	ADD_METHOD(isChecked)
	ADD_METHOD(setCheckable)
	ADD_METHOD(setChecked)
	ADD_METHOD(setIcon)
  //setShortcut ( const QKeySequence & key )
  
  
  
	//geerbt
	ADD_WIDGET_METHODS(QToolButtonWrap, QToolButton)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QToolButton"), constructor);  
}

Handle<Value> QToolButtonWrap::New(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = new QToolButtonWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QToolButtonWrap::NewInstance(QToolButton *q) {
  HandleScope scope;
  
  Local<Object> instance = constructor->NewInstance(0, NULL);
  QToolButtonWrap* w = node::ObjectWrap::Unwrap<QToolButtonWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QToolButtonWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
	
	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  //QToolButton* q = w->GetWrapped();
	QImpl<QToolButton>* q = (QImpl<QToolButton>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="clicked") 
		QObject::connect(q, SIGNAL(clicked(bool)), rec, SLOT(evtB(bool)));	
	else if (sign=="pressed") 
		QObject::connect(q, SIGNAL(pressed()), rec, SLOT(evtV()));	
	else if (sign=="released") 
		QObject::connect(q, SIGNAL(released()), rec, SLOT(evtV()));	
	else if (sign=="toggled") 
		QObject::connect(q, SIGNAL(toggled(bool)), rec, SLOT(evtB(bool)));	
//	else if (sign=="triggered") 
//		QObject::connect(q, SIGNAL(triggered(QAction *)), rec, SLOT(evtA(QAction *)));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QToolButtonWrap::off(const Arguments& args) {
	HandleScope scope;
	
  QToolButtonWrap* w = node::ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  
 	//QToolButton* q = w->GetWrapped();
 	QImpl<QToolButton>* q = (QImpl<QToolButton>*)(w->GetWrapped());
 		
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="clicked")
			QObject::disconnect(q, SIGNAL(clicked(bool)), rec, 0);
		else if (sign=="pressed")
			QObject::disconnect(q, SIGNAL(pressed()), rec, 0);
		else if (sign=="released")
			QObject::disconnect(q, SIGNAL(released()), rec, 0);
		else if (sign=="toggled") 
			QObject::disconnect(q, SIGNAL(toggled(bool)), rec, 0);	
		// delete receivers?
		
	  HANDLE_WIDGET_EVENTS_OFF
	  
	}
	return args.This();
}

Handle<Value> QToolButtonWrap::setText(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  q->setText(qt_v8::ToQString(args[0]->ToString()));
  
  return scope.Close(Undefined());
}

Handle<Value> QToolButtonWrap::text(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();
  
  return scope.Close(qt_v8::FromQString(q->text()));
}

// menu () -> object
Handle<Value> QToolButtonWrap::menu(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();
  
  QMenu *menu = q->menu();
  
  return scope.Close(QMenuWrap::NewInstance(menu));
}

// setMenu (object menu)
Handle<Value> QToolButtonWrap::setMenu(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  QMenuWrap* menu_wrap = ObjectWrap::Unwrap<QMenuWrap>(args[0]->ToObject());
  q->setMenu(menu_wrap->GetWrapped());

  return scope.Close(Undefined());
}
  
// setPopupMode (int)
Handle<Value> QToolButtonWrap::setPopupMode(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  q->setPopupMode((QToolButton::ToolButtonPopupMode)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// setToolButtonStyle (int)
Handle<Value> QToolButtonWrap::setToolButtonStyle(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  q->setToolButtonStyle((Qt::ToolButtonStyle)args[0]->IntegerValue());
  
  return scope.Close(Undefined());
}

// isChecked () -> bool
Handle<Value> QToolButtonWrap::isChecked(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();
  
  return scope.Close(Boolean::New(q->isChecked()));
}

// setCheckable (bool)
Handle<Value> QToolButtonWrap::setCheckable(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  q->setCheckable(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

// setChecked (bool)
Handle<Value> QToolButtonWrap::setChecked(const Arguments& args) {
  HandleScope scope;

  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();

  q->setChecked(args[0]->BooleanValue());
  
  return scope.Close(Undefined());
}

// setIcon (string|QImage)
Handle<Value> QToolButtonWrap::setIcon(const Arguments& args) {
  HandleScope scope;
  
  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();
    
  QIcon ico;
  if (args[0]->IsString()) { //A: LOAD ICON FROM FILE
		 ico.addFile(qt_v8::ToQString(args[0]->ToString()));
  } else if (args[0]->IsObject() && qt_v8::ToQString(args[0]->ToObject()->GetConstructorName())=="QImage") { 
	  QImageWrap* img_wrap = ObjectWrap::Unwrap<QImageWrap>(args[0]->ToObject());
	  QImage* img = img_wrap->GetWrapped();
		QPixmap pm = QPixmap::fromImage(*img);
		ico.addPixmap(pm);
	}
  if (!ico.isNull()) q->setIcon(ico);
  
  return scope.Close(Undefined());
}

// setDefaultAction (QAction)
Handle<Value> QToolButtonWrap::setDefaultAction(const Arguments& args) {
  HandleScope scope;
  
  QToolButtonWrap* w = ObjectWrap::Unwrap<QToolButtonWrap>(args.This());
  QToolButton* q = w->GetWrapped();
    
	QActionWrap* action_wrap = ObjectWrap::Unwrap<QActionWrap>(args[0]->ToObject());
	q->setDefaultAction(action_wrap->GetWrapped());  
	  
  return scope.Close(Undefined());
}
