#include "qtreeview.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTreeView>;

/*************************************************
 QTreeViewWrap
*************************************************/

Persistent<Function> QTreeViewWrap::constructor;

QTreeViewWrap::QTreeViewWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTreeView>(q_parent, fixScope);
	else
  	q_ = new QTreeView(q_parent);

}

QTreeViewWrap::~QTreeViewWrap() {
  //delete q_;
}

void QTreeViewWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTreeView"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
	ADD_METHOD(keyboardSearch)
	ADD_METHOD(model)
	ADD_METHOD(reset)
	ADD_METHOD(rootIndex)
	ADD_METHOD(selectAll)
	ADD_METHOD(setAlternatingRowColors)
	ADD_METHOD(setColumnHidden)
	ADD_METHOD(setHeaderHidden)
	ADD_METHOD(setModel)
	ADD_METHOD(setRootIndex)
	ADD_METHOD(setRootIsDecorated)
	ADD_METHOD(setSortingEnabled)
	ADD_METHOD(sortByColumn)
	
	ADD_METHOD(setColumnWidth)
	ADD_METHOD(resizeColumnToContents)
	
	// index
	ADD_METHOD(currentIndex)
	ADD_METHOD(indexAt)
	ADD_METHOD(selectedIndexes)
	ADD_METHOD(setCurrentIndex)
	ADD_METHOD(setSelectedIndexes)
	
	ADD_METHOD(setExpanded);
		
	//geerbt
	ADD_WIDGET_METHODS(QTreeViewWrap, QTreeView)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTreeView"), constructor);
}

Handle<Value> QTreeViewWrap::New(const Arguments& args) {
  HandleScope scope;
  QTreeViewWrap* w = new QTreeViewWrap(args);
  w->Wrap(args.This());
  return args.This();
}

Handle<Value> QTreeViewWrap::NewInstance(QTreeView *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTreeViewWrap* w = node::ObjectWrap::Unwrap<QTreeViewWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTreeViewWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  
  //QTreeView* q = w->GetWrapped();
  QImpl<QTreeView>* q = (QImpl<QTreeView>*)(w->GetWrapped()); 
  //TODO: check if really QImpl!!!
  
  Receiver* rec = new Receiver(callback, args.This());
  
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="collapsed")
		QObject::connect(q, SIGNAL(collapsed(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="expanded")
		QObject::connect(q, SIGNAL(expanded(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
		
	else if (sign=="activated")
		QObject::connect(q, SIGNAL(activated(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="clicked")
		QObject::connect(q, SIGNAL(clicked(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
else if (sign=="doubleClicked")
		QObject::connect(q, SIGNAL(doubleClicked(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="entered")
		QObject::connect(q, SIGNAL(entered(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));
	else if (sign=="pressed")
		QObject::connect(q, SIGNAL(pressed(const QModelIndex &)), rec, SLOT(evtM(const QModelIndex &)));

	else if (sign=="currentChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));
	else if (sign=="currentColumnChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentColumnChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));
	else if (sign=="currentRowChanged")
		QObject::connect(q->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &,const QModelIndex &)), rec, SLOT(evtMM(const QModelIndex &,const QModelIndex &)));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTreeViewWrap::off(const Arguments& args) {
	HandleScope scope;
  QTreeViewWrap* w = node::ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  
 	//QTreeView* q = w->GetWrapped();
 	QImpl<QTreeView>* q = (QImpl<QTreeView>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

			else if (sign=="collapsed")
		QObject::disconnect(q, SIGNAL(collapsed(const QModelIndex &)), rec, 0);
		else if (sign=="expanded")
			QObject::disconnect(q, SIGNAL(expanded(const QModelIndex &)), rec, 0);
			
		else if (sign=="activated")
			QObject::disconnect(q, SIGNAL(activated(const QModelIndex &)), rec, 0);
		else if (sign=="clicked")
			QObject::disconnect(q, SIGNAL(clicked(const QModelIndex &)), rec, 0);
		else if (sign=="doubleClicked")
			QObject::disconnect(q, SIGNAL(doubleClicked(const QModelIndex &)), rec, 0);
		else if (sign=="entered")
			QObject::disconnect(q, SIGNAL(entered(const QModelIndex &)), rec, 0);
		else if (sign=="pressed")
			QObject::disconnect(q, SIGNAL(pressed(const QModelIndex &)), rec, 0);
		
		else if (sign=="currentChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentChanged(const QModelIndex &)), rec, 0);
		else if (sign=="currentColumnChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentColumnChanged(const QModelIndex &)), rec, 0);
		else if (sign=="currentRowChanged")
			QObject::disconnect(q->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &)), rec, 0);
					
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This();
}

// keyboardSearch (string)
Handle<Value> QTreeViewWrap::keyboardSearch(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->keyboardSearch(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

// reset ()
Handle<Value> QTreeViewWrap::reset(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->reset();

  return scope.Close(Undefined());
}

// rootIndex
Handle<Value> QTreeViewWrap::rootIndex(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  QModelIndex m = q->rootIndex();
  
  return scope.Close(QModelIndexWrap::NewInstance(m));
}

// selectAll ()
Handle<Value> QTreeViewWrap::selectAll(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->selectAll();

  return scope.Close(Undefined());
}

// setAlternatingRowColors (bool)
Handle<Value> QTreeViewWrap::setAlternatingRowColors(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->setAlternatingRowColors(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

// setSortingEnabled (bool)
Handle<Value> QTreeViewWrap::setSortingEnabled(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->setSortingEnabled(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

// sortByColumn (int column, *int order)
Handle<Value> QTreeViewWrap::sortByColumn(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->sortByColumn(args[0]->IntegerValue(), (Qt::SortOrder)args[1]->IntegerValue());

  return scope.Close(Undefined());
}

// setColumnHidden (int column, bool hide)
Handle<Value> QTreeViewWrap::setColumnHidden(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->setColumnHidden(args[0]->IntegerValue(), args[1]->BooleanValue());

  return scope.Close(Undefined());
}

// setHeaderHidden (bool hide)
Handle<Value> QTreeViewWrap::setHeaderHidden(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->setHeaderHidden(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

// model() -> object
Handle<Value> QTreeViewWrap::model(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
	QFileSystemModel *model = (QFileSystemModel *)q->model();
	
  return scope.Close(QFileSystemModelWrap::NewInstance(model));
}

// setModel ( QAbstractItemModel * )
Handle<Value> QTreeViewWrap::setModel(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
	QFileSystemModelWrap* file_system_model_wrap = ObjectWrap::Unwrap<QFileSystemModelWrap>(args[0]->ToObject());
	q->setModel(file_system_model_wrap->GetWrapped());
					
  return scope.Close(Undefined());
}

// setRootIndex ( const QModelIndex & )
Handle<Value> QTreeViewWrap::setRootIndex(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
	QModelIndexWrap* model_index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
	
	q->setRootIndex(*model_index_wrap->GetWrapped());
					
  return scope.Close(Undefined());
}

// setRootIsDecorated ( bool show )
Handle<Value> QTreeViewWrap::setRootIsDecorated(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
  q->setRootIsDecorated(args[0]->BooleanValue());
					
  return scope.Close(Undefined());
}

// setColumnWidth (int column, int width)
Handle<Value> QTreeViewWrap::setColumnWidth(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->setColumnWidth(args[0]->IntegerValue(), args[1]->IntegerValue());

  return scope.Close(Undefined());
}

// resizeColumnToContents (int column)
Handle<Value> QTreeViewWrap::resizeColumnToContents(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  q->resizeColumnToContents(args[0]->IntegerValue());

  return scope.Close(Undefined());
}

// currentIndex () -> QModelIndex
Handle<Value> QTreeViewWrap::currentIndex(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
		
	QModelIndex index = q->currentIndex();
	
  return scope.Close(QModelIndexWrap::NewInstance(index));
}

// setCurrentIndex ( const QModelIndex & index ) 
Handle<Value> QTreeViewWrap::setCurrentIndex(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
	QModelIndexWrap* item_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
		
	q->setCurrentIndex(*item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- indexAt (point) -> QModelIndex
Handle<Value> QTreeViewWrap::indexAt(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();

  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
  QPoint* point = point_wrap->GetWrapped();
  QModelIndex index = q->indexAt(*point);

  return scope.Close(QModelIndexWrap::NewInstance(index));
}

// selectedIndexes
Handle<Value> QTreeViewWrap::selectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
			
	QList<QModelIndex> indexes = q->selectionModel()->selectedIndexes();
  
  // create array
  Handle<Array> indexArray = v8::Array::New(indexes.count());
  for (int i=0;i<indexes.count();i++){
  	indexArray->Set(i, QModelIndexWrap::NewInstance(indexes[i]));
  }
  
  return scope.Close(indexArray);
}

// setSelectedIndexes
Handle<Value> QTreeViewWrap::setSelectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
	
	QItemSelection *selection = new QItemSelection(); // QList<QModelIndex>
	
	//JS ARRAY TO QLIST
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++){
  	QModelIndexWrap* index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(obj->Get(i)->ToObject());
  	QItemSelectionRange* range = new QItemSelectionRange(*index_wrap->GetWrapped());
    selection->append( *range );
  }
	
	int flags = args[1]->IntegerValue();
	
	// const QItemSelection & selection, QItemSelectionModel::SelectionFlags command 
	q->selectionModel()->select(
		*selection,
		(QItemSelectionModel::SelectionFlags)flags
	);
	  
  return scope.Close(Undefined());
}

// setExpanded
Handle<Value> QTreeViewWrap::setExpanded(const Arguments& args) {
  HandleScope scope;

  QTreeViewWrap* w = ObjectWrap::Unwrap<QTreeViewWrap>(args.This());
  QTreeView* q = w->GetWrapped();
  
	QModelIndexWrap* index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
	
	q->setExpanded(*index_wrap->GetWrapped(), args[1]->BooleanValue());
    
  return scope.Close(Undefined());
}

