#include "qtreewidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QTreeWidget>;

/*************************************************
 QTreeWidgetWrap
*************************************************/

Persistent<Function> QTreeWidgetWrap::constructor;

QTreeWidgetWrap::QTreeWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QTreeWidget>(q_parent, fixScope);
	else
  	q_ = new QTreeWidget(q_parent);
}

QTreeWidgetWrap::~QTreeWidgetWrap() {
  //delete q_;
}

//static
void QTreeWidgetWrap::Initialize(Handle<Object> target) {
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QTreeWidget"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
  
	ADD_METHOD(addTopLevelItem)
	ADD_METHOD(columnCount)
	ADD_METHOD(currentColumn)
	ADD_METHOD(currentItem)
	ADD_METHOD(insertTopLevelItem)
	ADD_METHOD(invisibleRootItem)
	ADD_METHOD(itemAt)
	ADD_METHOD(selectedItems)
	ADD_METHOD(setColumnCount)
	ADD_METHOD(setCurrentItem)
	ADD_METHOD(setHeaderItem)
	ADD_METHOD(setHeaderLabel)
	ADD_METHOD(setHeaderLabels)
	ADD_METHOD(sortColumn)
	ADD_METHOD(sortItems)
	ADD_METHOD(topLevelItem)
	ADD_METHOD(topLevelItemCount)
	ADD_METHOD(visualItemRect)

	ADD_METHOD(setHeaderHidden)
	
	ADD_METHOD(setDropIndicatorShown)
	ADD_METHOD(setDragDropMode) 
	
	// INDEX
	ADD_METHOD(currentIndex)
	ADD_METHOD(indexAt)
	ADD_METHOD(selectedIndexes)
	ADD_METHOD(setCurrentIndex)
	ADD_METHOD(setSelectedIndexes)

	//geerbt
	ADD_WIDGET_METHODS(QTreeWidgetWrap, QTreeWidget)
	
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QTreeWidget"), constructor);
}

Handle<Value> QTreeWidgetWrap::New(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = new QTreeWidgetWrap(args);
  
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QTreeWidgetWrap::NewInstance(QTreeWidget *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QTreeWidgetWrap* w = node::ObjectWrap::Unwrap<QTreeWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QTreeWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped()); //TODO: check if really QTreeWidgetImpl!!!

	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
		
	else if (sign=="currentItemChanged")
		QObject::connect(q, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), rec, SLOT(evtTT(QTreeWidgetItem*,QTreeWidgetItem*)));
	else if (sign=="itemActivated")
		QObject::connect(q, SIGNAL(itemActivated(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemChanged")
		QObject::connect(q, SIGNAL(itemChanged(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemClicked")
		QObject::connect(q, SIGNAL(itemClicked(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemCollapsed")
		QObject::connect(q, SIGNAL(itemCollapsed(QTreeWidgetItem*)), rec, SLOT(evtT(QTreeWidgetItem*)));
	else if (sign=="itemDoubleClicked")
		QObject::connect(q, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemEntered")
		QObject::connect(q, SIGNAL(itemEntered(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemExpanded")
		QObject::connect(q, SIGNAL(itemExpanded(QTreeWidgetItem*)), rec, SLOT(evtT(QTreeWidgetItem*)));
	else if (sign=="itemPressed")
		QObject::connect(q, SIGNAL(itemPressed(QTreeWidgetItem*,int)), rec, SLOT(evtTI(QTreeWidgetItem*,int)));
	else if (sign=="itemSelectionChanged")
		QObject::connect(q, SIGNAL(itemSelectionChanged()), rec, SLOT(evtV()));

	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QTreeWidgetWrap::off(const Arguments& args) {
	HandleScope scope;
  QTreeWidgetWrap* w = node::ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
 	//QTreeWidget* q = w->GetWrapped();
 	QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
			
		else if (sign=="currentItemChanged") 
			QObject::disconnect(q, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), rec, 0);
		else if (sign=="itemActivated") 
			QObject::disconnect(q, SIGNAL(itemActivated(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemChanged") 
			QObject::disconnect(q, SIGNAL(itemChanged(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemClicked") 
			QObject::disconnect(q, SIGNAL(itemClicked(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemCollapsed") 
			QObject::disconnect(q, SIGNAL(itemCollapsed(QTreeWidgetItem*)), rec, 0);
		else if (sign=="itemDoubleClicked") 
			QObject::disconnect(q, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemEntered") 
			QObject::disconnect(q, SIGNAL(itemEntered(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemExpanded") 
			QObject::disconnect(q, SIGNAL(itemExpanded(QTreeWidgetItem*)), rec, 0);
		else if (sign=="itemPressed") 
			QObject::disconnect(q, SIGNAL(itemPressed(QTreeWidgetItem*,int)), rec, 0);
		else if (sign=="itemSelectionChanged") 
			QObject::disconnect(q, SIGNAL(itemSelectionChanged()), rec, 0);	
		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This();
}

//- addTopLevelItem (QTreeWidgetItem * item)
Handle<Value> QTreeWidgetWrap::addTopLevelItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());  
	q->addTopLevelItem(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- columnCount () -> int
Handle<Value> QTreeWidgetWrap::columnCount(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());

  return scope.Close(Integer::New(q->columnCount()));
}

//- currentColumn () -> int
Handle<Value> QTreeWidgetWrap::currentColumn(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());

  return scope.Close(Integer::New(q->currentColumn()));
}

//- currentItem () -> QTreeWidgetItem *
Handle<Value> QTreeWidgetWrap::currentItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();
  	
  QTreeWidgetItem *item = q->currentItem();
  
  return scope.Close(QTreeWidgetItemWrap::NewInstance(item));
}

//- insertTopLevelItem (int index, QTreeWidgetItem * item)
Handle<Value> QTreeWidgetWrap::insertTopLevelItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[1]->ToObject());  
	q->insertTopLevelItem(args[0]->IntegerValue(), item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- invisibleRootItem () -> QTreeWidgetItem *
Handle<Value> QTreeWidgetWrap::invisibleRootItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  QTreeWidgetItem *item = q->invisibleRootItem();
  
  return scope.Close(QTreeWidgetItemWrap::NewInstance(item));
}

//- itemAt (int x, int y) -> QTreeWidgetItem *
Handle<Value> QTreeWidgetWrap::itemAt(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
  QTreeWidgetItem *item;
  if (args[0]->IsObject()){
	  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
	  QPoint* point = point_wrap->GetWrapped();
	  item = q->itemAt(*point);
	}else{//if (args[0]->IsString()) { 
		item = q->itemAt(args[0]->IntegerValue(), args[1]->IntegerValue());
	}
  return scope.Close(item ? QTreeWidgetItemWrap::NewInstance(item) : Undefined()); // ???
}

//- selectedItems () -> aray of items
Handle<Value> QTreeWidgetWrap::selectedItems(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
  QList<QTreeWidgetItem *> items = q->selectedItems();

  // create array
  Handle<Array> itemsArray = v8::Array::New(items.count());
  for (int i=0;i<items.count();i++){
  	//add to array
  	//itemsArray->Set(i, itemWrap->handle_); // ???
  	itemsArray->Set(i, QTreeWidgetItemWrap::NewInstance(items[i])); // ???
  }
	
  return scope.Close(itemsArray);
}

//- setColumnCount (int columns)
Handle<Value> QTreeWidgetWrap::setColumnCount(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
	q->setColumnCount(args[0]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- setCurrentItem (QTreeWidgetItem * item)
Handle<Value> QTreeWidgetWrap::setCurrentItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());
	q->setCurrentItem(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- setHeaderItem (QTreeWidgetItem * item)
Handle<Value> QTreeWidgetWrap::setHeaderItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());  
	q->setHeaderItem(item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- setHeaderLabel (string label)
Handle<Value> QTreeWidgetWrap::setHeaderLabel(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
	q->setHeaderLabel(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

//- setHeaderLabels (array labels)
Handle<Value> QTreeWidgetWrap::setHeaderLabels(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
  if (args[0]->IsArray()){
		QStringList labels;
	  Local<Object> obj = args[0]->ToObject();
	  Local<Array> props = obj->GetPropertyNames();
	  for(unsigned int i = 0; i < props->Length(); i++) {
	    labels.append( qt_v8::ToQString(obj->Get(i)->ToString()) );
	  }
	  
		q->setHeaderLabels(labels);
	}
	
  return scope.Close(Undefined());
}

//- sortColumn () -> int
Handle<Value> QTreeWidgetWrap::sortColumn(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  return scope.Close(Integer::New(q->sortColumn()));
}

//- sortItems (int column, int order)
Handle<Value> QTreeWidgetWrap::sortItems(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  
	q->sortItems(args[0]->IntegerValue(), (Qt::SortOrder)args[1]->IntegerValue());
	
  return scope.Close(Undefined());
}

//- topLevelItem (int index) -> QTreeWidgetItem *
Handle<Value> QTreeWidgetWrap::topLevelItem(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
	QTreeWidgetItem *item = q->topLevelItem(args[0]->IntegerValue());
	
  return scope.Close(QTreeWidgetItemWrap::NewInstance(item));
}

//- topLevelItemCount () -> int
Handle<Value> QTreeWidgetWrap::topLevelItemCount(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  return scope.Close(Integer::New(q->topLevelItemCount()));
}

//- visualItemRect (QTreeWidgetItem *) -> rect
Handle<Value> QTreeWidgetWrap::visualItemRect(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());
  	
  QTreeWidgetItemWrap* item_wrap = ObjectWrap::Unwrap<QTreeWidgetItemWrap>(args[0]->ToObject());  
	
	return scope.Close( QRectWrap::NewInstance(q->visualItemRect(item_wrap->GetWrapped())) );
}

//- setHeaderHidden (bool)
Handle<Value> QTreeWidgetWrap::setHeaderHidden(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());

	q->setHeaderHidden(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

Handle<Value> QTreeWidgetWrap::setDropIndicatorShown(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());

	q->setDropIndicatorShown(args[0]->BooleanValue());
  return scope.Close(Undefined());
}

//QAbstractItemView::NoDragDrop	0	Does not support dragging or dropping.
//QAbstractItemView::DragOnly	1	The view supports dragging of its own items
//QAbstractItemView::DropOnly	2	The view accepts drops
//QAbstractItemView::DragDrop	3	The view supports both dragging and dropping
//QAbstractItemView::InternalMove	4	The view accepts move (not copy) operations only from itself.

Handle<Value> QTreeWidgetWrap::setDragDropMode(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  
  //QTreeWidget* q = w->GetWrapped();
  QImpl<QTreeWidget>* q = (QImpl<QTreeWidget>*)(w->GetWrapped());

	q->setDragDropMode((QAbstractItemView::DragDropMode)args[0]->IntegerValue());
  return scope.Close(Undefined());
}


// currentIndex () -> QModelIndex
Handle<Value> QTreeWidgetWrap::currentIndex(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();
		
	QModelIndex index = q->currentIndex();
	
  return scope.Close(QModelIndexWrap::NewInstance(index));
}

// selectedIndexes
Handle<Value> QTreeWidgetWrap::selectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();
			
	QList<QModelIndex> indexes = q->selectionModel()->selectedIndexes();
  
  // create array
  Handle<Array> indexArray = v8::Array::New(indexes.count());
  for (int i=0;i<indexes.count();i++){
  	indexArray->Set(i, QModelIndexWrap::NewInstance(indexes[i]));
  }
  
  return scope.Close(indexArray);
}

// setSelectedIndexes
Handle<Value> QTreeWidgetWrap::setSelectedIndexes(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();
	
	QItemSelection *selection = new QItemSelection(); // QList<QModelIndex>
	
	//JS ARRAY TO QLIST
  Local<Object> obj = args[0]->ToObject();
  Local<Array> props = obj->GetPropertyNames();
  for(unsigned int i = 0; i < props->Length(); i++){
  	QModelIndexWrap* index_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(obj->Get(i)->ToObject());
  	QItemSelectionRange* range = new QItemSelectionRange(*index_wrap->GetWrapped());
    selection->append( *range );
  }
	
	int flags = args[1]->IntegerValue();
	
	// const QItemSelection & selection, QItemSelectionModel::SelectionFlags command 
	q->selectionModel()->select(
		*selection,
		(QItemSelectionModel::SelectionFlags)flags
	);
	  
  return scope.Close(Undefined());
}

// setCurrentIndex ( const QModelIndex & index ) 
Handle<Value> QTreeWidgetWrap::setCurrentIndex(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();
	
	QModelIndexWrap* item_wrap = ObjectWrap::Unwrap<QModelIndexWrap>(args[0]->ToObject());
		
	q->setCurrentIndex(*item_wrap->GetWrapped());
	
  return scope.Close(Undefined());
}

//- indexAt (point) -> QModelIndex
Handle<Value> QTreeWidgetWrap::indexAt(const Arguments& args) {
  HandleScope scope;

  QTreeWidgetWrap* w = ObjectWrap::Unwrap<QTreeWidgetWrap>(args.This());
  QTreeWidget* q = w->GetWrapped();

  QPointWrap* point_wrap = ObjectWrap::Unwrap<QPointWrap>(args[0]->ToObject());
  QPoint* point = point_wrap->GetWrapped();
  QModelIndex index = q->indexAt(*point);

  return scope.Close(QModelIndexWrap::NewInstance(index));
}