#include "qwidget.h"
#include "qwidgeteventstpl.h"

using namespace v8;

template class QImpl<QWidget>;


/*************************************************
 QWidgetWrap
*************************************************/

Persistent<Function> QWidgetWrap::constructor;

// QWidgetWrap()
QWidgetWrap::QWidgetWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QWidget>(q_parent, fixScope);
	else
  	q_ = new QWidget(q_parent);

	//TEST
	//instances.append(this);
}

QWidgetWrap::~QWidgetWrap() {
  //delete q_;
}

//static
void QWidgetWrap::Initialize(Handle<Object> target) {
	const char * classname = "QWidget";
	
	//TEST
	//gMap->insert(std::pair<int, int>(23, 666)); 
	
	//classMap.insert(std::pair<std::string, std::string>(classname, classname));
	
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol(classname));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);  
	
	InitializeMethods(tpl);
	      
  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol(classname), constructor);
}

//static
void QWidgetWrap::InitializeMethods(Handle<FunctionTemplate> tpl) {
	
  // Wrapped methods
  ADD_METHOD(on)
  ADD_METHOD(off)
  
  ADD_WIDGET_METHODS(QWidgetWrap, QWidget)
}

Handle<Value> QWidgetWrap::New(const Arguments& args) {
  HandleScope scope;
  
  QWidgetWrap* w = new QWidgetWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QWidgetWrap::NewInstance(QWidget *q) {
  HandleScope scope;

	// TEST
//	for (int i = 0; i < instances.size(); ++i) {
//		if (instances.at(i)->q_ == q){
//			return scope.Close(instances.at(i)->ToObject());
//		}
//	}

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QWidgetWrap* w = node::ObjectWrap::Unwrap<QWidgetWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

// on (event, function)
Handle<Value> QWidgetWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);
			
	QString sign = qt_v8::ToQString(args[0]->ToString()); 
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QWidgetWrap* w = node::ObjectWrap::Unwrap<QWidgetWrap>(args.This());
  QImpl<QWidget>* q = (QImpl<QWidget>*)(w->GetWrapped()); //TODO: check if really QImpl!!!
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested")
		
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));	
  
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

//################################################
// NEW
// http://doc.qt.digia.com/qt/qobject.html#disconnect-2
//################################################
// off()
// off(event)
// off(event, function) TODO
Handle<Value> QWidgetWrap::off(const Arguments& args) {
	HandleScope scope; // ???
	
  QWidgetWrap* w = node::ObjectWrap::Unwrap<QWidgetWrap>(args.This());
  
 	//QWidget* q = w->GetWrapped();
 	QImpl<QWidget>* q = (QImpl<QWidget>*)(w->GetWrapped()); //TODO: check if really QImpl!!!
	
	if (args.Length()==0){
		
		// QObject * sender, char * signal, QObject * receiver, char * method
		QObject::disconnect(q, 0, 0, 0);

		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested")
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	
		
		HANDLE_WIDGET_EVENTS_OFF
				
	}
		
	return args.This(); // return action ?
}

