#include "../getwrap.h"

#include <QMultiHash>

// Class QImpl<WC>
template<class WC> class QImpl : public WC {
	
 public:
  QImpl(QPrinter* printer, QWidget* parent=0, bool fix=false);
  QImpl(QWidget* parent=0, bool fix=false);
  ~QImpl();
    
 public:
 	bool fixScope;
 	QMultiHash<QString, v8::Persistent<v8::Value>> eventCallbacks;
 	
 private:
	void changeEvent(QEvent* e);
	void closeEvent(QCloseEvent* e);
	void contextMenuEvent(QContextMenuEvent* e);
	void dragEnterEvent(QDragEnterEvent *event);
	void dragMoveEvent(QDragMoveEvent *event);
	void dropEvent(QDropEvent *event);
	void hideEvent(QEvent* e);
	void keyPressEvent(QKeyEvent* e);
	void keyReleaseEvent(QKeyEvent* e);  
	void mouseMoveEvent(QMouseEvent* e);
	void paintEvent(QPaintEvent* e);
	void showEvent(QEvent* e);
  
		//  void mousePressEvent(QMouseEvent* e);
		//  void mouseReleaseEvent(QMouseEvent* e);
		
		//virtual void 	tabletEvent ( QTabletEvent * event )
		//virtual void 	wheelEvent ( QWheelEvent * event )
		//virtual bool 	winEvent ( MSG * message, long * result )
};

// QWidgetImpl()
template<class WC> QImpl<WC>::QImpl(QPrinter* printer, QWidget* parent, bool fix) : WC(printer, parent) {
  fixScope = fix;
}
template<class WC> QImpl<WC>::QImpl(QWidget* parent, bool fix) : WC(parent) {
  fixScope = fix;
}

template<class WC> QImpl<WC>::~QImpl() {
 	foreach (v8::Persistent<v8::Value> cb, eventCallbacks) cb.Dispose();
	eventCallbacks.clear();
}

template<class WC> void QImpl<WC>::changeEvent(QEvent* e) {	
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QEventWrap::NewInstance((QEvent*)e)
  };
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("changeEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
	  	// TEST: src widget as "this"
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
	  	// ORG
	  	cb->Call(Context::GetCurrent()->Global(), argc, argv);
	  }
 	}
  
}

template<class WC> void QImpl<WC>::closeEvent(QCloseEvent* e) {	
  HandleScope scope;
  
  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QEventWrap::NewInstance((QEvent*)e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("closeEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
			cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::contextMenuEvent(QContextMenuEvent* e) {	
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    //QEventWrap::NewInstance((QEvent*)e) // TODO: 
    QContextMenuEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("contextMenuEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::dropEvent(QDropEvent *e){
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QDropEventWrap::NewInstance(e)
  };
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dropEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

//TEST
template<class WC> void QImpl<WC>::dragEnterEvent(QDragEnterEvent *e){
	e->acceptProposedAction();
		
//  HandleScope scope;
//  
//  const unsigned argc = 1;
//  Handle<Value> argv[argc] = {
//    QDragEnterEventWrap::NewInstance(e)
//  };
//  
//  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dragEnterEvent");
// 	for (int i = 0; i < callbacks.size(); ++i){
//	  if (fixScope){
//			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
//		}else{
//		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
//		}
//	}

}


template<class WC> void QImpl<WC>::dragMoveEvent(QDragMoveEvent *e){
	e->acceptProposedAction();
	
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QDragMoveEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dragMoveEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::keyPressEvent(QKeyEvent* e) {
	e->accept(); // TEST
  //e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QKeyEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("keyPressEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::keyReleaseEvent(QKeyEvent* e) {
	e->accept(); // TEST
  //e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QKeyEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("keyReleaseEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::mouseMoveEvent(QMouseEvent* e) {
	//e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QMouseEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("mouseMoveEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
	
  WC::mouseMoveEvent(e);
}

template<class WC> void QImpl<WC>::paintEvent(QPaintEvent* e) {
	HandleScope scope;
  //e->ignore();//bringt nix

  const unsigned argc = 0;
  Handle<Value> argv[1] = {};
	  
//  const unsigned argc = 1;
//  Handle<Value> argv[argc] = {
//    QEventWrap::NewInstance((QEvent*)e) // TODO: QPaintEventWrap::NewInstance(*e)
//  };
	  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("paintEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
  }
    
  WC::paintEvent(e);
}

template<class WC> void QImpl<WC>::hideEvent(QEvent* e) {
  HandleScope scope;
  
  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("hideEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

template<class WC> void QImpl<WC>::showEvent(QEvent* e) {
  HandleScope scope;
  
  const unsigned argc = 1;
  Handle<Value> argv[argc] = {
    QEventWrap::NewInstance(e)
  };

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("showEvent");
 	for (int i = 0; i < callbacks.size(); ++i){
 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
	  if (fixScope){
			cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
		}else{
		  cb->Call(Context::GetCurrent()->Global(), argc, argv);
		}
	}
}

//test: used by editors
//template<class WC> void QImpl<WC>::printPreview(QPrinter *printer) {
//	print(printer);
//}
