#include "../getwrap.h"

#include <QMultiHash>

// Class QImpl<WC>
template<class WC> class QImpl : public WC {
	
 public:
  QImpl(QWidget* parent=0, bool fix=false); //was: QWidget* parent=0
  ~QImpl();
  
 public:
 	bool fixScope;
 	QMultiHash<QString, v8::Persistent<v8::Value>> eventCallbacks;
 	
 private:
	void changeEvent(QEvent* e);
	void closeEvent(QCloseEvent* e);
	void contextMenuEvent(QContextMenuEvent* e);
	void dragEnterEvent(QDragEnterEvent *event);
	void dragLeaveEvent(QDragLeaveEvent *event);
	void dragMoveEvent(QDragMoveEvent *event);
	void dropEvent(QDropEvent *event);
	void hideEvent(QEvent* e);
	void keyPressEvent(QKeyEvent* e);
	void keyReleaseEvent(QKeyEvent* e);  
	void mouseMoveEvent(QMouseEvent* e);
	void mousePressEvent(QMouseEvent* e);
	void mouseReleaseEvent(QMouseEvent* e);
	void paintEvent(QPaintEvent* e);
	void showEvent(QEvent* e);

		//virtual void 	tabletEvent ( QTabletEvent * event )
		//virtual void 	wheelEvent ( QWheelEvent * event )
		//virtual bool 	winEvent ( MSG * message, long * result )
};

// QWidgetImpl()
template<class WC> QImpl<WC>::QImpl(QWidget* parent, bool fix) : WC(parent) {
  fixScope = fix;
}

template<class WC> QImpl<WC>::~QImpl() {
 	foreach (v8::Persistent<v8::Value> cb, eventCallbacks) cb.Dispose();
	eventCallbacks.clear();
}

//################################################
// changeEvent
//################################################
template<class WC> void QImpl<WC>::changeEvent(QEvent* e) {	
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("changeEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QEventWrap::NewInstance((QEvent*)e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
		  	// TEST: src widget as "this"
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
		  	// ORG
		  	cb->Call(Context::GetCurrent()->Global(), argc, argv);
		  }
	 	}
	}
  //WC::changeEvent(e);
}

//################################################
// closeEvent
//################################################
template<class WC> void QImpl<WC>::closeEvent(QCloseEvent* e) {	
  HandleScope scope;
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("closeEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QEventWrap::NewInstance((QEvent*)e) //???
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
				cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::closeEvent(e);
}

//################################################
// contextMenuEvent
//################################################
template<class WC> void QImpl<WC>::contextMenuEvent(QContextMenuEvent* e) {	
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("contextMenuEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QContextMenuEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::contextMenuEvent(e);
}

//################################################
// dragEnterEvent
//################################################
template<class WC> void QImpl<WC>::dragEnterEvent(QDragEnterEvent *e){
	e->acceptProposedAction();


	// ALL BELOW WAS: UNCOMMENTED!
  HandleScope scope;
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dragEnterEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    //QDragEnterEventWrap::NewInstance(e) // crash
	    QPointWrap::NewInstance(e->pos()) // test
	  };
	 	for (int i=cnt-1; i>=0; i--){
 			Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::dragEnterEvent(e);
}

//################################################
// dragLeaveEvent [NEW]
//################################################
template<class WC> void QImpl<WC>::dragLeaveEvent(QDragLeaveEvent *e){

  HandleScope scope;
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dragLeaveEvent");
  int cnt = callbacks.size();
  if (cnt){
//	  const unsigned argc = 1;
//	  Handle<Value> argv[argc] = {
//	    QEventWrap::NewInstance((QEvent*)e) //???
//	  };
	 	for (int i=cnt-1; i>=0; i--){
 			Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), 0, 0);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), 0, 0);
			}
		}
	}
	
  WC::dragLeaveEvent(e);
}

//################################################
// dragMoveEvent
//################################################
template<class WC> void QImpl<WC>::dragMoveEvent(QDragMoveEvent *e){
	e->acceptProposedAction();
	
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dragMoveEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    //QDragMoveEventWrap::NewInstance(e) // crash
	    QPointWrap::NewInstance(e->pos()) // test
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}

  //WC::dragMoveEvent(e);
}

//################################################
// dropEvent
//################################################
template<class WC> void QImpl<WC>::dropEvent(QDropEvent *e){
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("dropEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QDropEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::dropEvent(e);
}

//################################################
// keyPressEvent
//################################################
template<class WC> void QImpl<WC>::keyPressEvent(QKeyEvent* e) {
	e->accept(); // TEST
  //e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("keyPressEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QKeyEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::keyPressEvent(e);
}

//################################################
// keyReleaseEvent
//################################################
template<class WC> void QImpl<WC>::keyReleaseEvent(QKeyEvent* e) {
	e->accept(); // TEST
  //e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("keyReleaseEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QKeyEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::keyReleaseEvent(e);
}

//################################################
// mouseMoveEvent
//################################################
template<class WC> void QImpl<WC>::mouseMoveEvent(QMouseEvent* e) {
	//e->ignore(); // ensures event bubbles up
  
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("mouseMoveEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QMouseEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  WC::mouseMoveEvent(e);
}

//################################################
// mousePressEvent
//################################################
template<class WC> void QImpl<WC>::mousePressEvent(QMouseEvent* e) {  
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("mousePressEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QMouseEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  WC::mousePressEvent(e);
}

//################################################
// mouseReleaseEvent
//################################################
template<class WC> void QImpl<WC>::mouseReleaseEvent(QMouseEvent* e) {  
  HandleScope scope;

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("mouseReleaseEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QMouseEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  WC::mouseReleaseEvent(e);
}

//################################################
// paintEvent
//################################################
template<class WC> void QImpl<WC>::paintEvent(QPaintEvent* e) {
	HandleScope scope;
  //e->ignore();//bringt nix

  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("paintEvent");
  int cnt = callbacks.size();
  if (cnt){  	
		const unsigned argc = 1;
		Handle<Value> argv[argc] = {
			//QEventWrap::NewInstance((QEvent*)e)
			QRectWrap::NewInstance(e->rect()) // test
		};
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv); // was: 0,0
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv); // was: 0,0
			}
	  }
	}
    
  WC::paintEvent(e);
}

//################################################
// hideEvent
//################################################
template<class WC> void QImpl<WC>::hideEvent(QEvent* e) {
  HandleScope scope;
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("hideEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::hideEvent(e);
}

//################################################
// showEvent
//################################################
template<class WC> void QImpl<WC>::showEvent(QEvent* e) {
  HandleScope scope;
  
  QList<v8::Persistent<v8::Value>> callbacks = eventCallbacks.values("showEvent");
  int cnt = callbacks.size();
  if (cnt){
	  const unsigned argc = 1;
	  Handle<Value> argv[argc] = {
	    QEventWrap::NewInstance(e)
	  };
	 	for (int i=cnt-1; i>=0; i--){
	 		Handle<Function> cb = Persistent<Function>::Cast(callbacks.at(i));
		  if (fixScope){
				cb->Call( GetInstanceWrap(this)->ToObject(), argc, argv);
			}else{
			  cb->Call(Context::GetCurrent()->Global(), argc, argv);
			}
		}
	}
	
  //WC::showEvent(e);
}

