#include "qwizardpage.h"
#include "qwidgeteventstpl.h"
#include "../Types/qpixmap.h"

using namespace v8;

template class QImpl<QWizardPage>;

/*************************************************
 QWizardPageWrap
*************************************************/

Persistent<Function> QWizardPageWrap::constructor;

QWizardPageWrap::QWizardPageWrap(const Arguments& args) : q_(NULL) {
	QWidget * q_parent = 0;
	//QWidgetImpl* q_parent = 0;
	if (args.Length()>0 && args[0]->IsObject()){
    QWidgetWrap* w_parent = ObjectWrap::Unwrap<QWidgetWrap>(args[0]->ToObject());
    q_parent = w_parent->GetWrapped();
	}
	hasEvents = args.Length()>1?args[1]->BooleanValue():false;
	bool fixScope = args.Length()>2?args[2]->BooleanValue():false;
	
	if (hasEvents)
		q_ = new QImpl<QWizardPage>(q_parent, fixScope);
	else
  	q_ = new QWizardPage(q_parent);
}

QWizardPageWrap::~QWizardPageWrap() {
  //delete q_;
}

void QWizardPageWrap::Initialize(Handle<Object> target) {
  // Prepare constructor template
  Local<FunctionTemplate> tpl = FunctionTemplate::New(New);
  tpl->SetClassName(String::NewSymbol("QWizardPage"));
  tpl->InstanceTemplate()->SetInternalFieldCount(1);

  // Wrapped methods
	ADD_METHOD(on)
	ADD_METHOD(off)
	
//QString 	buttonText ( QWizard::WizardButton which ) const
//QPixmap 	pixmap ( QWizard::WizardPixmap which ) const
//setButtonText ( QWizard::WizardButton which, const QString & text )
//setPixmap ( QWizard::WizardPixmap which, const QPixmap & pixmap )
//void 	setSubTitle ( const QString & subTitle )
//setTitle ( const QString & title )
//QString 	subTitle () const
//QString 	title () const

	ADD_METHOD(buttonText)
	ADD_METHOD(pixmap)
	ADD_METHOD(setButtonText)
	ADD_METHOD(setPixmap)
	ADD_METHOD(setSubTitle)
	ADD_METHOD(setFinalPage)
	ADD_METHOD(setTitle)
	ADD_METHOD(subTitle)
	ADD_METHOD(title)

	//geerbt
	ADD_WIDGET_METHODS(QWizardPageWrap, QWizardPage)

  constructor = Persistent<Function>::New(tpl->GetFunction());
  target->Set(String::NewSymbol("QWizardPage"), constructor);
}

Handle<Value> QWizardPageWrap::New(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = new QWizardPageWrap(args);
  w->Wrap(args.This());

  return args.This();
}

Handle<Value> QWizardPageWrap::NewInstance(QWizardPage *q) {
  HandleScope scope;

  Local<Object> instance = constructor->NewInstance(0, NULL);
  QWizardPageWrap* w = node::ObjectWrap::Unwrap<QWizardPageWrap>(instance);
  w->SetWrapped(q);

  return scope.Close(instance);
}

Handle<Value> QWizardPageWrap::on(const Arguments& args) {
	HandleScope scope;
	ASSERT_TYPE(args[0],IsString);
	ASSERT_TYPE(args[1],IsFunction);

	QString sign = qt_v8::ToQString(args[0]->ToString());
	Local<Function> callback = Local<Function>::Cast(args[1]);

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  //QWizardPage* q = w->GetWrapped();
	QImpl<QWizardPage>* q = (QImpl<QWizardPage>*)(w->GetWrapped());
	
	Receiver* rec = new Receiver(callback, args.This());
	
	// SIGNALS
	if (sign=="customContextMenuRequested") //WIDGET
		QObject::connect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, SLOT(evtP(const QPoint &)));
	
	else if (sign=="completeChanged")
		QObject::connect(q, SIGNAL(completeChanged()), rec, SLOT(evtV()));
		
	else {
		delete rec;
		rec = 0;
		HANDLE_WIDGET_EVENTS
  }
	
	//return args.This();
	return scope.Close(rec ? QObjectWrap::NewInstance(rec) : Undefined());
}

Handle<Value> QWizardPageWrap::off(const Arguments& args) {
	HandleScope scope;
  QWizardPageWrap* w = node::ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  
 	//QWizardPage* q = w->GetWrapped();
 	QImpl<QWizardPage>* q = (QImpl<QWizardPage>*)(w->GetWrapped());
 	 	
	if (args.Length()==0){
		QObject::disconnect(q, 0, 0, 0);
		
		HANDLE_WIDGET_EVENTS_OFF_ALL
		
	}else {
		QString sign = qt_v8::ToQString(args[0]->ToString());
		
		QObject* rec = 0;
		if (args.Length()>1){
	  	QObjectWrap* object_wrap = ObjectWrap::Unwrap<QObjectWrap>(args[1]->ToObject());
	  	rec = object_wrap->GetWrapped();
		}
		
		if (sign=="customContextMenuRequested") //WIDGET
			QObject::disconnect(q, SIGNAL(customContextMenuRequested(const QPoint &)), rec, 0);	

		else if (sign=="completeChanged") 
			QObject::disconnect(q, SIGNAL(completeChanged()), rec, 0);

		// delete receivers?
		
		HANDLE_WIDGET_EVENTS_OFF
		
	}
	return args.This();
}


//QString 	buttonText ( QWizard::WizardButton which ) const
Handle<Value> QWizardPageWrap::buttonText(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->buttonText((QWizard::WizardButton)args[0]->IntegerValue())));
}

//QPixmap 	pixmap ( QWizard::WizardPixmap which ) const
Handle<Value> QWizardPageWrap::pixmap(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();
  
	QPixmap pm = q->pixmap((QWizard::WizardPixmap)args[0]->IntegerValue());
	return scope.Close(QPixmapWrap::NewInstance(pm));
}


//setButtonText ( QWizard::WizardButton which, const QString & text )
Handle<Value> QWizardPageWrap::setButtonText(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

	q->setButtonText((QWizard::WizardButton)args[0]->IntegerValue(), qt_v8::ToQString(args[1]->ToString()));

  return scope.Close(Undefined());
}

//setPixmap ( QWizard::WizardPixmap which, const QPixmap & pixmap )
Handle<Value> QWizardPageWrap::setPixmap(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

  QPixmapWrap* pixmap_wrap = ObjectWrap::Unwrap<QPixmapWrap>(args[1]->ToObject());
  QPixmap* pixmap = pixmap_wrap->GetWrapped();
    
	q->setPixmap( (QWizard::WizardPixmap)args[0]->IntegerValue(), *pixmap);

  return scope.Close(Undefined());
}

//void 	setSubTitle ( const QString & subTitle )
Handle<Value> QWizardPageWrap::setSubTitle(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

	q->setSubTitle(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

//void 	setFinalPage ( bool finalPage )
Handle<Value> QWizardPageWrap::setFinalPage(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();
  
	q->setFinalPage(args[0]->BooleanValue());

  return scope.Close(Undefined());
}

//setTitle ( const QString & title )
Handle<Value> QWizardPageWrap::setTitle(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

	q->setTitle(qt_v8::ToQString(args[0]->ToString()));

  return scope.Close(Undefined());
}

//QString 	subTitle () const
Handle<Value> QWizardPageWrap::subTitle(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->subTitle()));
}

//QString 	title () const
Handle<Value> QWizardPageWrap::title(const Arguments& args) {
  HandleScope scope;

  QWizardPageWrap* w = ObjectWrap::Unwrap<QWizardPageWrap>(args.This());
  QWizardPage* q = w->GetWrapped();

  return scope.Close(qt_v8::FromQString(q->title()));
}

